package jais;

import java.util.*;
import org.apache.log4j.*;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import jais.utils.*;
import jais.world.WorldModel;
import java.io.*;

/**
 * <p>Title: JaisTeam</p>
 * <p>Description: This Class will start a whole team of Jais-Players</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: Jais</p>
 * @author Stephan Diederich
 */
public class JaisTeam
{
  private ArrayList m_playerList;              // where we store our players in
  private int m_numberOfPlayers;               // how many Players to start at
  private boolean m_commandLineReading = true; // indicate when we have to stop, or not start to read from commandline
  private static Logger logger;                // logger from the apache project
  private BufferedReader m_input;
  /**
   * Constructor starts up f_numberOfPlayers and begins reading from commandline
   * @param f_teamName  Our Teamname
   * @param f_serverName  Name of the Server we want to connect to
   * @param f_serverPort  Port of the server
   * @param f_numberOfPlayers Number of Players to connect
   * @throws xJaisTeam if we there are probs while initializing the players
   * @roseuid 3CE6898F00E0
   */
  public JaisTeam(String f_teamName, String f_serverName, int f_serverPort, int f_numberOfPlayers) throws xJaisTeam
  {
    //store our member variables
    m_numberOfPlayers=f_numberOfPlayers;
    m_playerList=new ArrayList(m_numberOfPlayers);  //initialize our playerlist
    int l_connectedPlayers=0;
    /*Initialize the whole team*/
    try
    {
      if (m_numberOfPlayers>1)//if we should connect more than one player, we connect a goalie as first player
      {
        m_playerList.add(new Jais(this, f_teamName,f_serverName,f_serverPort,true,1,false));
        l_connectedPlayers++;
      }
      for (int i=l_connectedPlayers; i<m_numberOfPlayers; i++)
                 { // create the rest of the team here
        m_playerList.add(new Jais(this, f_teamName,f_serverName,f_serverPort,false,i+1,false));
      }
    }
    catch(xJais ex)
    {
      logger.fatal("Exception caught while initializing Team ! : "+ex.getMessage()+" ...aborting all...");
      for(ListIterator it = m_playerList.listIterator();it.hasNext();)
      {
        //quit all players that were instantiated as we could not connect our team
        Jais l_tempJais=(Jais)it.next();
        l_tempJais.quit();
      }
      throw (new xJaisTeam("Error while initializing !"));
    }
    /*Call the startup method of the players*/
    try
    {
      for (Iterator it=m_playerList.iterator();it.hasNext();)
      {
        ((Jais)it.next()).startup();
      }
    }
    catch (xJais ex)
    {
      logger.fatal("Exception caught while starting up Team ! : "+ex.getMessage()+" ...aborting all...");
      for(ListIterator it = m_playerList.listIterator();it.hasNext();)
      {
            /*If not all the players can startup, quit our complete team */
        Jais l_tempJais=(Jais)it.next();
        l_tempJais.quit();
      }
      throw (new xJaisTeam("Error while starting up !"));
    }
    /*Start reading commands from commandline*/
    readFromCommandline();
  }

  /**
   * The main method of this class stores all necessary commandline arguments
   * in a Hashtable and then instanciates JaisTeam
   * It also creates our logger
   * @param args = commandline parameters
   * @roseuid 3CE6898F0112
   */
  public static void main(String[] args)
  {
    /*Configuration of the logger*/
    logger = Logger.getLogger("jais");
    FileAppender fa = new FileAppender();
    fa.setFile("jaisteam.txt");
    fa.setLayout(new PatternLayout(PatternLayout.TTCC_CONVERSION_PATTERN));
    fa.setAppend(false);
    fa.activateOptions();
    logger.addAppender(fa);
    BasicConfigurator.configure();
    //initialize the hashtable for getting "better" commandline params
    Hashtable l_filter=new Hashtable(10);
    l_filter.put("h","help");
    l_filter.put("l","loglevel");
    l_filter.put("n","numberOfPlayers");
    Hashtable l_args=CommandLineParser.Parse(args,l_filter);
    //if commandline was fine
    if (l_args!=null)
    {
      logger.setLevel(Level.toLevel((String)l_args.get("loglevel")));
      System.out.println("Current logging level:" + logger.getLevel()); //check our level
      try
      {
        JaisTeam team=new JaisTeam(     (String)l_args.get("Teamname"),
                                        (String)l_args.get("Servername"),
                                        Integer.parseInt((String)l_args.get("Serverport")),
                                        Integer.parseInt((String)l_args.get("numberOfPlayers"))
                                        );
      }
      catch(xJaisTeam xTeam)
      {
        logger.fatal("Exception caught while starting the team ... aborting");
        return;
      }
    }
    else //there were errors from the commandline
    {
      Usage();
      return;
    }
    System.out.println("JaisTeam says Goodbye and wishes a nice day!");
    System.exit(0); //needed as we use swing(awt)
  }//main()

  /**
   * This method prints only the Usage of JaisTeam to the Commandline
   * @roseuid 3CEBD85202CB
   */
  public static void Usage()
  {
    System.out.println("\nUSAGE: java JaisTeam [TEAMNAME] [SERVERNAME] [SERVERPORT] -[OPTIONS]\n");
    System.out.println("TEAMNAME = your chosen teamname (default: Jais)");
    System.out.println("SERVERNAME = Server to connect to (default: localhost)");
    System.out.println("SERVERPORT = Port on which to connect (default: 6000)");
    System.out.println("OPTIONS = \n\t-n NUMBER for the number of players to connect (default: 11) "
                       +"\n\t-h for help "
                       +"\n\t-l NUMBER loglevel n=0..6;  0 for ALL; 6 for OFF");
    System.out.println("");
  }//Usage()

  /**
     * This method is called by the communication if it thinks, there is no more server present
     * @param f_timesWaited how many times it retried to receive stuff
     */
  public void informClientDisconnected()
  {
    m_numberOfPlayers--;
    if (m_numberOfPlayers==0)
    {
      m_commandLineReading=false;
      System.out.println("All Players disconnected ... hit RETURN to quit");
    }
  }
  /**
   * This method reads all commandline parameters which are typed in while the game
   * is running and passes them to the corresponding player
   * @roseuid 3CEBF7D0029B
   */
  private void readFromCommandline()
  {
    m_input = new BufferedReader(new InputStreamReader(System.in));
    String l_message= new String();
    String l_tempString= new String();
    int l_targetPlayer=-1;
    while(m_commandLineReading) //default=true, set to false if we want to quit
    {
      System.out.print("\nType help for help \n>");
      try
      {  l_message=m_input.readLine(); }
      catch (Exception ex)
      {
        logger.info("Exception caught in JaisTeam::readFromCommandline:" + ex.getMessage());
      }
      if (l_message.equalsIgnoreCase("Exit") || l_message.equalsIgnoreCase("E"))
      {
        m_commandLineReading=false;
        // stop all players...
        for(ListIterator it = m_playerList.listIterator();it.hasNext();)
        {
          Jais l_tempJais=(Jais)it.next();
          l_tempJais.quit();
        }
      }
      else if (l_message.equalsIgnoreCase("help"))
        System.out.println( "\nCommands are:"
                            +"\n[Playernumber] visualization [ON | OFF]"
                            +"\n[Playernumber] exit for kicking one player"
                            +"\n\"Exit\" for exiting the game");
      else // no error and no command for JaisTeam, so pass it to the corresponding player
      {
        StringTokenizer l_tokenizer = new StringTokenizer(l_message);
        try
        {
          l_targetPlayer=Integer.parseInt(l_tokenizer.nextToken());
          if (!(l_targetPlayer>0 && l_targetPlayer <=m_numberOfPlayers))
            System.out.println("No such PlayerNumber! (1..n)");
          else
          {
            l_tempString="";// make it empty;
            while(l_tokenizer.hasMoreTokens())
              l_tempString+=" "+l_tokenizer.nextToken();
            ((Jais)m_playerList.get(l_targetPlayer-1)).parseKbInfo(l_tempString); //correct the player number, as we start from 0;
          }
        }
        catch (Exception ex)
        {
          System.out.println("Didn't understand your command. Please retry \n" );
        }
      }
    }// while()
  }//readfromCommandline()
}//Jaisteam()