package jais.communication;

import org.apache.log4j.Logger;
import org.apache.log4j.BasicConfigurator;
import java.net.*;
import java.io.*;
import jais.*;


/**
 *
 * <p>Title: Communication class</p>
 * <p>Description: This class provides the possibility to send an receive data from the soccer server </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p> </p>
 * @author Raul Zanzinger
 *
 */
public class Communication extends Thread implements ConnectionStuffInterface, PlayerStuffInterface
{
  private String m_serverName;                  // name of robucup server which will be resolved
  private int m_serverPort;                     // port of the robocup server
  private String m_teamName;                    // name of the team to connect
  private boolean m_isGoalie;                   // wether the player to connect is goalie or not
  private InetAddress m_serverAddress;          // the resolved server adress
  private DatagramSocket m_socket;              // new datagram socket
  private String m_stuffToSendToServer;         // catches every message which has to
  // be sent to server
  private Jais m_jais;                          // a reference on jais
  private Thread m_thisThread;                  // for the stop-thread (receiving data)
  private String m_receivedMessageFromServer="";// catches every received message from the server
  private boolean m_connectedToServer=false;    // a flag to decide wether connected or not
  private int m_timeoutAfter=1000;              // the timeout for reading from the socket
  private boolean m_disconnectCommand=false;    // will be set true when communic. should diconnect
  private int m_playerNumber;
  private int m_timeoutCounter=5;               // counter how often communication is allowed to
  // retry receiving stuff from server
  //the logger
  private static Logger logger=Logger.getLogger("jais.communication");


  /**
   * @roseuid 3CEB65340225
   *
   * @param f_teamName a user choosen name else standart
   * @param f_serverName name of the server to which the client should connect
   * @param f_serverPort port of the server to which the client should connect
   * @param f_isGoalie ==true when the player to connect is a goalie
   * @param f_jais a reference on main module jais
   * @param f_playerNumber the player number which the user wants to reconnect
   *
   * @throws xCommunication
   */
  public Communication( String f_teamName,
                        String f_serverName,
                        int  f_serverPort,
                        boolean f_isGoalie,
                        Jais f_jais,
                        int f_playerNumber) throws xCommunication
  {
    this.setName("Communication"+f_playerNumber);
    //logger.info("Succ. instan. Commun.!?");
    //logger.info("1 Server name is: "+f_serverName);
    //logger.info("2 Server port is: "+f_serverPort);
    m_jais       = f_jais;
    m_serverName = f_serverName;
    m_serverPort = f_serverPort;
    m_teamName   = f_teamName;
    m_isGoalie   = f_isGoalie;
    m_isGoalie   = f_isGoalie;
    m_playerNumber=f_playerNumber;

    // try to handel with / to bind socket
    try
    {
      //logger.info("Bind socket to a port...");
      // set datagram socket and bind it to a port
      this.m_socket = new DatagramSocket();
      // set server address
      this.m_serverAddress = InetAddress.getByName(m_serverName);
      // set server port
      this.m_serverPort = m_serverPort;
      m_socket.setSoTimeout(m_timeoutAfter);
      //logger.info("2 Server port: "+ this.m_serverPort);
      //logger.info("The resolved server address is: "+this.m_serverAddress);
      //logger.info("Binded socket to a port!");
    }
    catch (SocketException se)
    {
      //logger.info("Could not bind socket to a port! "+se.getMessage());
      throw(new xCommunication("Could not bind socket to a port!"));
    }
    catch (UnknownHostException uhe)
    {
      //logger.info("The choosen host is not reachable!"+uhe.getMessage()
                 // +" serverport: "+this.m_serverPort);
      throw(new xCommunication("The choosen host is not reachable: '"
                               +this.m_serverAddress+"'"));
    }
  }


  /****************************************************************************************/
/* Here starts the implementation of the ConnectionStuffInterface
 /****************************************************************************************/

 /**
  * This method establishs the first connection to the server
  *
  * @throws xCommunication
  */
  public void connectToServer() throws xCommunication
  {
    String l_goalieSpaceHolder;
    //check if goalie has be init.
    if(!m_isGoalie)
    {
      // mask/creat the message which will be sent, as goalie
      l_goalieSpaceHolder = "";
    }
    else
    {
      // mask/creat the message which will be sent, as normal player
      l_goalieSpaceHolder = "(goalie)";
    }
    //logger.info("Connecting to server...");
    sendMessageToServer("(init " + m_teamName + " (version 7.0) "
                        + l_goalieSpaceHolder + ")");
    //logger.info("Waiting for data from server...");
    // to prevent a little server/client 'crash' while init. a whole team
    receiveDataFromServer();
    //for the first time: start receiving-data-from-server-thread
    startThread();
  }



    /* This method establishs the possibilty to reconnect to the server.*/
  public void reconnectToServer(int f_playerNumber) throws xCommunication
  {
    //logger.info("Starting to reconnect to server...");
    //logger.info("Reconnecting to server...");
    sendMessageToServer("(reconnect " + m_teamName + " " + f_playerNumber + ")");
  }



    /* This method provides the posiblity to disconnect the player from the server.
  * The method uses sendMessageToServer() to send the bye signal to the server*/
  public void disconnectFromServer() throws xCommunication
  {
    //stop the running receiving thread
    stopThread();

    //has jais been connected?
    if(m_connectedToServer==true)
    {
      //logger.info("Disconnecting from server... Bye from Commu.");

      //from now it is shure: we want to disconnect
      m_disconnectCommand=true;
      // set min. timeout, as we dont want to wait a long time
      m_timeoutAfter=1;
      //to ensure no waiting for socket timeout; set new m_timeoutAfter
      try
      {
        m_socket.setSoTimeout(m_timeoutAfter);
      }
      catch (Exception ex)
      {
        //logger.info("Could not min. socket time out for disconnection: "
                   // +ex.getMessage());
      }

      // set the bye signal as server message
      sendMessageToServer("(bye)");
    }
    //so jais has not been connected to a server (no data  received)
    else
    {
      //logger.info("Bye from Communication: Jais was not connected to"
                  //+" the server (no server data received/not regularly connected)!");
    }
    //close the opended socket
    m_socket.disconnect();
    m_socket.close();
  }


  /****************************************************************************************/
/* Here starts the implementation of the PlayerStuffInterface
 /****************************************************************************************/


 /**
  * This method sends jais' messages to the server
  *
  * @roseuid 3CEB64DA02BC
  *
  * @param f_stuffToSendToServer the message which should be sent to the server
  *
  * @throws xCommunication
  */
  public void sendMessageToServer(String f_stuffToSendToServer) throws xCommunication
  {
    m_stuffToSendToServer = f_stuffToSendToServer;
    //try to send data to the server
    try
    {
//      logger.fatal("Server Message is: "+m_stuffToSendToServer + " time:" +System.currentTimeMillis() );
      // init buffer
      byte[] l_buffer = new byte[1024];
      // m_stuffToSendToServer -> byte field
      l_buffer = m_stuffToSendToServer.getBytes();
      //logger.info("3 Server port is: "+m_serverPort);
      // creat new datagram packet
      DatagramPacket l_packet = new DatagramPacket(l_buffer,
          l_buffer.length,
          m_serverAddress,
          this.m_serverPort
          );
      m_socket.send(l_packet);
    }
    catch (IOException ioe)
    {
      //logger.info("Error while reading from socket: "+ioe.getMessage());
      throw(new xCommunication("(socket error) Could not send data to server!"));
    }
    //logger.info("...Succeeded to send data to server!");
  }


  /**
   * This method starts the receiveDataFromServer thread
   *
   */
  public void run()
  {
    Thread l_thisThread=currentThread();
    while (l_thisThread==m_thisThread)
    {
      try
      {
        receiveDataFromServer();
      }
      catch (xCommunication xCom)
      {
        m_timeoutCounter--;
        if(m_timeoutCounter==0)
        {
          m_jais.informNoMoreServerPresent(m_timeoutCounter);
          try
          {
            this.disconnectFromServer();
          }
          catch (xCommunication xCommu)
          {
            //nothing to do here
          }
        }
      }
    }
  }


  /**
   * This method sends the command (only from the goalie) to the
   * server to catch the ball. Playmode must be play_on.
   * Successfull catching changes playmode.
   *
   * @roseuid 3CEB64DA0302
   *
   * @param f_direction catch the ball from this angel
   *
   * @throws xCommunication
   */
  public void catchBall(double f_direction) throws xCommunication
  {
    try
    {
      sendMessageToServer("(catch "+f_direction+")");
    }
    catch (xCommunication xCom)
    {
      //logger.info("Could not send catchBall action! "+xCom.getMessage());
      throw(new xCommunication("Could not send catchBall action!"));
    }
  }


  /**
   * This method provides the possibility to send dash action commands
   * to the server
   *
   * @roseuid 3CEB64DA0320
   *
   * @param f_dashPower the power of dash command
   *
   * @throws xCommunication
   */
  public void dash(double f_dashPower) throws xCommunication
  {
    try
    {
//      logger.fatal("Dash sent with power :" + f_dashPower);
      sendMessageToServer("(dash "+f_dashPower+")");
    }
    catch (xCommunication xCom)
    {
      //logger.info("Could not send dash action! "+xCom.getMessage());
      throw(new xCommunication("Could not send dash action!"));
    }
  }


  /**
   * This method provides the kick action commands which have to be
   * sent to the server.
   *
   * @roseuid 3CEB64DA033E
   *
   * @param f_kickAngel the angel to kick to
   * @param f_kickPower the power of following kick
   *
   * @throws xCommunication
   */
  public void kick(double f_kickPower, double f_kickAngel) throws xCommunication
  {
    try
    {
      sendMessageToServer("(kick " + f_kickPower + " " + f_kickAngel + ")");
    }
    catch (xCommunication xCom)
    {
      //logger.info("Could not send kick action! "+xCom.getMessage());
      throw(new xCommunication("Could not send kick action!"));
    }
  }


  /**
   * This method provides the turn neck action commands which have to be
   * sent to the server.
   *
   * @roseuid 3CEB64DA0366
   *
   * @param f_turnNeckAngel the angel to turn the neck to
   *
   * @throws xCommunication
   */
  public void turnNeck(double f_turnNeckAngel)  throws xCommunication
  {
    try
    {
      sendMessageToServer("(turn_neck " + f_turnNeckAngel + ")");
    }
    catch (xCommunication xCom)
    {
      //logger.info("Could not send turn neck action! "+xCom.getMessage());
      throw(new xCommunication("Could not send turn neck action!"));
    }
  }


  /**
   * This method provides the option to send move actions to the server.
   * Playmode has to before kick off.
   * @roseuid 3CEB64DA0384
   *
   * @param f_moveToXPos the x element of the point to move to
   * @param f_moveToYPos the y element of the point to move to
   *
   * @throws xCommunication
   */
  public void move(double f_moveToXPos, double f_moveToYPos) throws xCommunication
  {
    try
    {
      sendMessageToServer("(move " + f_moveToXPos + " " + f_moveToYPos + ")");
    }
    catch (xCommunication xCom)
    {
      //logger.info("Could not send move action! "+xCom.getMessage());
      throw(new xCommunication("Could not send move action!"));
    }
  }


  /**
   * This method provides the option to send turn actions to the server.
   *
   * @roseuid 3CEB64DA03A2
   *
   * @param f_turnAngel to angel how far to turn th neck
   *
   * @throws xCommunication
   */
  public void turn(double f_turnAngel) throws xCommunication
  {
    try
    {
      sendMessageToServer("(turn " + f_turnAngel + ")");
    }
    catch (xCommunication xCom)
    {
      //logger.info("Could not send turn action! "+xCom.getMessage());
      throw(new xCommunication("Could not send turn action!"));
    }
  }


  /**
   * This method provides the option to send say actions to the server.
   *
   * @roseuid 3CEB64DA03CA
   *
   * @param f_sayMessage the string of the say message which should be send to the server
   *
   * @throws xCommunication
   */
  public void say(String f_sayMessage) throws xCommunication
  {
    try
    {
      sendMessageToServer("(turn_neck " + f_sayMessage + ")");
    }
    catch (xCommunication xCom)
    {
      //logger.info("Could not send say action! "+xCom.getMessage());
      throw(new xCommunication("Could not send say action!"));
    }
  }


  /**
   * This method provides the option to send change view actions to the server.
   *
   * @param f_width  four stages of view width are possible
   * @param f_highViewQuality high or low quality view can be choosen
   *
   * @throws xCommunication
   */
  public void changeView(String f_width , boolean f_highViewQuality) throws xCommunication
  {
    String l_viewQuality;

    //check if player wishes high/low (1/0) view qual.
    if(f_highViewQuality){l_viewQuality="high";}
    else{l_viewQuality="low";}
    try
    {
      sendMessageToServer("(change_view "+f_width+" "+l_viewQuality+")");
    }
    catch (xCommunication xCom)
    {
      //logger.info("Could not send change view action! "+xCom.getMessage());
      throw(new xCommunication("Could not send change view action!"));
    }
  }


  /****************************************************************************************/
/* Here starts the implementation of the class' private methods
 /****************************************************************************************/


 /**
  * This method provides the possibility to receive data from the server side.
  * The action which this method provides is done in a thread because of its
  * blocking reading from socket.
  *
  * @roseuid 3CEB31E90300
  *
  * @throws xCommunication
  */
  private void receiveDataFromServer() throws xCommunication
  {
    // set buffer for server message
    byte[] l_buffer = new byte[2048];
    // create datagram packet for receiving
    DatagramPacket l_packet = new DatagramPacket(l_buffer, l_buffer.length);
    //m_receivedMessageFromServer="";
    try
    {
      // receive packet, the method blocks until a datagram arrives
      m_socket.receive(l_packet);
      // get the server port to read out new data
      m_serverPort = l_packet.getPort();
      // l_receivedMessageFromServer catches the received server message
      m_receivedMessageFromServer = new String(l_packet.getData());
      //eliminate white spaces
      m_receivedMessageFromServer = m_receivedMessageFromServer.trim();

      //Check if jais has once been connected
      if(m_connectedToServer==false)
      {
        //info for the first time:
        //logger.info("Connected to server...(received something)");

        //for the first time: set m_connectedToServer=true
        //(Jais received something)
        m_connectedToServer=true;
      }

      //logger.info("Received server Mess.: '"+m_receivedMessageFromServer+"'");
      //logger.info("Lenght of server Message: '"+m_receivedMessageFromServer.length()+"'");
      m_timeoutCounter=5; //reset our timeoutcounter if we receive a message
      //check for incoming message type
      determIncomingMessageType();
    }

    catch (IOException ioe)
    {
      //has jais waited m_timeoutAfter [ms] to receive data from server
      //so jais will terminate communication
      if(m_disconnectCommand==false)
      {
        //logger.info("Timeout while reading from socket: "+ioe.getMessage());
        //logger.info("Jais tried "+m_timeoutAfter+"ms to receive data "+
                    //"from server...exiting communication!");
        if(m_connectedToServer)
        {
          throw(new xCommunication("Timeout while reading from socket... "+
                                   "was connected to server! "));
        }
        else
        {
          throw(new xCommunication("Timeout while reading from socket... "+
                                   "not connected to server! "));
        }
      }
    }
  }


  /**
   * This method determs the incoming message typ; of type:
   * init, see, sense, hear or error.
   *
   * @roseuid 3CEA584F0080
   *
   * @throws xCommunication
   */
  private void determIncomingMessageType() throws xCommunication
  {
    //check if new message has been a sense body mess.
    if (m_receivedMessageFromServer.startsWith("(sense"))
    {
//      //logger.fatal("Sense_Body event detected..." + System.currentTimeMillis()+"*******************");
      SenseBodyEvent newSenseBodyEvent = new SenseBodyEvent(m_receivedMessageFromServer);
      m_jais.onSenseBody(newSenseBodyEvent);
    }
    //check if new message has been a see mess.
    else if (m_receivedMessageFromServer.startsWith("(see"))
    {
//      //logger.fatal("Visual event detected..."+System.currentTimeMillis()+"*******************");
      VisualEvent newVisualEvent = new VisualEvent(m_receivedMessageFromServer);
      m_jais.onSee(newVisualEvent);
    }
    //check if new message has been a sense hear mess.
    else if (m_receivedMessageFromServer.startsWith("(hear"))
    {
      //logger.info("Hear event detected...");
      HearEvent newHearEvent = new HearEvent(m_receivedMessageFromServer);
      m_jais.onHear(newHearEvent);
    }
    //check if new message has been a init. mess.:
    else if (m_receivedMessageFromServer.startsWith("(init"))
    {
      //logger.info("Init event detected...");
      InitialEvent newInitialEvent = new InitialEvent(m_receivedMessageFromServer);
      m_jais.onInitialEvent(newInitialEvent);
    }
    //check if new message was server param
    else if (m_receivedMessageFromServer.startsWith("(server"))
    {
      //logger.info("Detected server params...");
      InitialEvent newInitialEvent = new InitialEvent(m_receivedMessageFromServer);
    }
    //check if new message was player param
    else if (m_receivedMessageFromServer.startsWith("(player") )
    {
      //logger.debug("Detected player params...");
      InitialEvent newInitialEvent = new InitialEvent(m_receivedMessageFromServer);
    }
    // check if new message has been an error mess. as reaction on
    // client control command: unknown command!
    else if (m_receivedMessageFromServer.startsWith("(error unknown"))
    {
      //logger.warn("Error message from server side as reaction on "
//                  +"client control command: '"+m_receivedMessageFromServer+"'!");
      //logger.warn("The last recently sent message to server from Jais has been: "
//                  +m_stuffToSendToServer);
    }
    // check if new message has been an error mess. as reaction on
    // client control command: illegal command form!
    else if (m_receivedMessageFromServer.startsWith("(error illegal"))
    {
      //logger.warn("Error message from server side as reaction on "
//                  +"client control command: '"+m_receivedMessageFromServer+"'!");
      //logger.warn("The last recently sent message to server from Jais has been: "
//                  +m_stuffToSendToServer);
    }
    // check if new message has been an error mess. as reaction on
    // init command: no more such player or team name!
    else if (m_receivedMessageFromServer.startsWith("(error no_more_team_or_player_or_goalie"))
    {
      //logger.fatal("Error message from server side as reaction on "
//                   +"init command (because e.g. team name given away: '"
//                   +m_receivedMessageFromServer+"'!");

      //logger.fatal("The last recently sent message to server from Jais has been: "
//                   +m_stuffToSendToServer);

      //received something but haven't been regularly connected
      m_connectedToServer=false;

      throw(new xCommunication("Error message from server side as reaction on"
                               +"init command: '"+m_receivedMessageFromServer+"'!"));
    }
    // check if new message has been an error mess. as reaction on
    // reconnect command: no more such player!
    else if (m_receivedMessageFromServer.startsWith("(error no_more_team_or_player"))
    {
      //logger.fatal("Error message from server side as reaction on "
//                   +"reconnect command: '"+m_receivedMessageFromServer+"'!");
      //logger.fatal("The last recently sent message to server from Jais has been: "
//                   +m_stuffToSendToServer);

      //received something but haven't been regularly connected
      m_connectedToServer=false;

      throw(new xCommunication("Error message from server side as reaction on"
                               +"reconnect command: '"+m_receivedMessageFromServer+"'!"));
    }
    // check if new message has been an error mess. as reaction on
    // reconnect command: reconnect failed!
    else if (m_receivedMessageFromServer.startsWith("(error reconnect"))
    {
      //logger.fatal("Error message from server side as reaction on "
//                   +"reconnect command: '"+m_receivedMessageFromServer+"'!");
      //logger.fatal("The last recently sent message to server from Jais has been: "
//                   +m_stuffToSendToServer);

      //received something but haven't been regularly connected
      m_connectedToServer=false;

      throw(new xCommunication("Error message from server side as reaction on "
                               +"reconnect command: '"+m_receivedMessageFromServer+"'!"));
    }
    //for case unknown data receiving
    else
    {
      //logger.info("Unknown server message received! ...but going on. Message was: "
//                  + m_receivedMessageFromServer);
      //logger.info("The last recently sent message to server from Jais has been: "
//                  +m_stuffToSendToServer);
    }
  }


  //start receive data thread
  private void startThread()
  {
    m_thisThread=this;
    this.start();
  }


  /**
   * This method stops the thread method (receiveDataFromServer) which
   * receives data from the server
   */
  private synchronized void stopThread()
  {
    m_thisThread=null;
  }


  /**
   * This method frees the binded system resources after terminating
   * the player.
   */
  public void finalize()
  {
    //logger.info("Ok, closing my socket!");
  }
}