package jais.communication;

import org.apache.log4j.Logger;
import org.apache.log4j.BasicConfigurator;
import jais.objects.AurealInfo;
import java.util.ArrayList;
import java.util.StringTokenizer;

/**
 * <p>Title: HearEvent</p>
 * <p>Description: This class is called by the Communication class and it parses the incoming Hear message and stores the elements in the arrylist m_objects which is used by the World Model</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Oliver Beck
 */
public class HearEvent extends Event
{
//   private static Logger logger=Logger.getLogger("jais.communication"); //get our logger

   /**
    * this method is called by the Communication class
    * it produces the arraylist and calls the parsing method
    * @param f_message contains the message to parse
    * @roseuid 3CEB63BB0182
    */
   public HearEvent(String f_message)
   {
    super (f_message);
    m_objects=new ArrayList(1);
    parseData();
   }

   /**
    * this method is called by the HearEvent method. it parses the message
    * and stores the elements in the arrylist
    * @roseuid 3CEB63BB018D
    */
   public void parseData()
   {
    StringTokenizer l_tokenizer = new StringTokenizer(m_message,"() ");
    AurealInfo l_hear_info = new AurealInfo();
    String l_token = null;

    // here begins the parsing
    try
    {
     l_tokenizer.nextToken(); // first token 'hear'
     l_hear_info.m_time = Integer.parseInt(l_tokenizer.nextToken()); // then element time
     //then finding out who sends the message:referee,online_coach_right,online_coach_left,self
     l_token = l_tokenizer.nextToken();
     int l_sender=0;
     while (l_sender < AurealInfo.NUMBER_OF_SENDERS)
     {
        if (l_token.equals(AurealInfo.AUREAL_SENDER[l_sender]))
            break;
      l_sender++;
     }
     l_hear_info.m_sender = l_sender; // set the sender
     // when the sender is determined, one of the 4 following sender-parsers is called
     switch(l_sender)
     {
      // referee
      case 0:
        //finding out playmode
        l_token = l_tokenizer.nextToken();
        int l_playmode=0;
        while (l_playmode < AurealInfo.NUMBER_OF_PLAYMODES)
        {
          if (l_token.startsWith(AurealInfo.PLAYMODE_STRINGS[l_playmode]))
            break;
          l_playmode++;
        }
        l_hear_info.m_playmode = l_playmode; // set the playmode
        break;
      // online coach right
      case 1:
        l_hear_info.m_message="";
        while(l_tokenizer.hasMoreTokens())
            l_hear_info.m_message+=l_tokenizer.nextToken();
        break;
      // online coach left
      case 2:
        l_hear_info.m_message="";
        while(l_tokenizer.hasMoreTokens())
            l_hear_info.m_message+=l_tokenizer.nextToken();
        break;
      // self
      case 3:
        l_hear_info.m_message="";
        while(l_tokenizer.hasMoreTokens())
            l_hear_info.m_message+=l_tokenizer.nextToken();
        break;
      // other player
      case 4:
        l_hear_info.m_direction=Double.parseDouble(l_token); //set the direction
        l_hear_info.m_message="";
        while(l_tokenizer.hasMoreTokens())
            l_hear_info.m_message+=l_tokenizer.nextToken();
        break;
     }
     m_objects.add(l_hear_info); // elements are inserted into the list
    }
     catch (Exception e)
     {
//      logger.info("Error parsing hear information, while parsing String:" + m_message + " exception :" +e.getMessage());
     }
   }
}
