package jais.communication;

import org.apache.log4j.Logger;
import org.apache.log4j.BasicConfigurator;
import jais.objects.*;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.lang.*;

/**
 * <p>Title: InitialEvent class</p>
 * <p>Description: This class is called by the Communication class and it parses the incoming Initial message and stores the elements in the ServerConf Class which is used by the World Model</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Oliver Beck
 */
public class InitialEvent extends Event
{
   private InitInfo m_initInfo;
//   private static Logger logger=Logger.getLogger("jais.communication"); //get our logger
   private StringTokenizer m_tokenizer;
   public static final String[] INITIAL_TYPE_STRINGS = {
                  "init",
                  "server_param",
                  "player_param",
                  "player_type"
                  };

   /**
    * this method is called by the Communication class
    * it produces the arraylist and calls the parseData method
    * @param f_message contains the message to parse
    * @roseuid 3CEB652E00BE
    */
   public InitialEvent(String f_message)
   {
    super(f_message);
    m_tokenizer = new StringTokenizer(m_message,"() ");
    m_objects=new ArrayList(1);
    parseData();
    m_tokenizer=null;
   }

   /**
    * this method is called by the parseData method. it parses the message
    * and stores the elements in the ServerConfig class
    * @roseuid 3CEA8D76025C
    */
   public void parseServerParamData()
   {
    m_tokenizer.nextToken(); // token 'server_param'

    ServerConf.GOAL_WIDTH               = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.INERTIA_MOMENT           = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PLAYER_SIZE              = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PLAYER_DECAY             = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PLAYER_RAND              = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PLAYER_WEIGHT            = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PLAYER_SPEED_MAX         = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PLAYER_ACCEL_MAX         = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.STAMINA_MAX              = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.STAMINA_INC              = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.RECOVER_INIT             = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.RECOVER_DTHR             = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.RECOVER_MIN              = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.RECOVER_DEC              = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.EFFORT_INIT              = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.EFFORT_DTHR              = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.EFFORT_MIN               = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.EFFORT_DEC               = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.EFFORT_ITHR              = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.EFFORT_INC               = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.KICK_RAND                = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.TEAM_ACTUATOR_NOISE      = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PRAND_FACTOR_L           = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PRAND_FACTOR_R           = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.KICK_RAND_FACTOR_L       = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.KICK_RAND_FACTOR_R       = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.BALL_SIZE                = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.BALL_DECAY               = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.BALL_RAND                = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.BALL_WEIGHT              = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.BALL_SPEED_MAX           = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.BALL_ACCEL_MAX           = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.DASH_POWER_RATE          = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.KICK_POWER_RATE          = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.KICK_MARGIN              = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CONTROL_RADIUS           = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CONTROL_RADIUS_WIDTH     = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.MAXPOWER                 = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.MINPOWER                 = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.MAXMOMENT                = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.MINMOMENT                = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.MAX_NECK_MOMENT          = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.MIN_NECK_MOMENT          = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.MAX_NECK_ANGLE           = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.MIN_NECK_ANGLE           = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.VISIBLE_ANGLE            = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.VISIBLE_DISTANCE         = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.WIND_DIR                 = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.WIND_FORCE               = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.WIND_ANGLE               = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.WIND_RAND                = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.KICKABLE_MARGIN          = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CATCHABLE_AREA_L         = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CATCHABLE_AREA_W         = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CATCH_PROBABILITY        = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.GOALIE_MAX_MOVES         = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.CKICK_MARGIN             = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.OFFSIDE_ACTIVE_AREA_SIZE = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.WIND_NONE                = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.WIND_RANDOM              = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.SAY_COACH_CNT_MAX        = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.SAY_COACH_MSG_SIZE       = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CLANG_WIN_SIZE           = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CLANG_DEFINE_WIN         = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CLANG_META_WIN           = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CLANG_ADVICE_WIN         = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CLANG_INFO_WIN           = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CLANG_MESS_DELAY         = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CLANG_MESS_PER_CYCLE     = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.HALF_TIME                = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.SIMULATOR_STEP           = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.SEND_STEP                = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.RECV_STEP                = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.SENSE_BODY_STEP          = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.LCM_STEP                 = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.SAY_MSG_SIZE             = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.HEAR_MAX                 = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.HEAR_INC                 = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.HEAR_DECAY               = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.CATCH_BAN_CYCLE          = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.SLOW_DOWN_FACTOR         = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.USE_OFFSIDE              = Integer.parseInt(m_tokenizer.nextToken());
    ServerConf.FORBID_KICK_OFF_OFFSIDE  = Integer.parseInt(m_tokenizer.nextToken());
    ServerConf.OFFSIDE_KICK_MARGIN      = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.AUDIO_CUT_DIST           = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.DIST_QSTEP               = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.LAND_QSTEP               = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.DIR_QSTEP                = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.DIST_QSTEP_L             = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.DIST_QSTEP_R             = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.LAND_QSTEP_L             = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.LAND_QSTEP_R             = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.DIR_QSTEP_L              = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.DIR_QSTEP_R              = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.COACH_MODE               = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.CWR_MODE                 = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.OLD_HEAR                 = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.SEND_VI_STEP             = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.START_GOAL_L             = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.START_GOAL_R             = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.FULLSTATE_L              = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.FULLSTATE_R              = Double.parseDouble(m_tokenizer.nextToken());

    ServerConf.DROP_TIME                = Double.parseDouble(m_tokenizer.nextToken());
   }

   /**
    * this method is called by the parseData method. it parses the message
    * and stores the elements in the ServerConfig class
    * @roseuid 3CEA8D840248
    */
   public void parsePlayerParamData()
   {
    m_tokenizer.nextToken(); // token 'player_param'

    ServerConf.PLAYER_TYPES                 = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.SUBS_MAX                     = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PT_MAX                       = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PLAYER_SPEED_MAX_DELTA_MIN   = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PLAYER_SPEED_MAX_DELTA_MAX   = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.STAMINA_INC_MAX_DELTA_FACTOR = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PLAYER_DECAY_DELTA_MIN       = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PLAYER_DECAY_DELTA_MAX       = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.INERTIA_MOMENT_DELTA_FACTOR  = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.DASH_POWER_RATE_DELTA_MIN    = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.DASH_POWER_RATE_DELTA_MAX    = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.PLAYER_SIZE_DELTA_FACTOR     = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.KICKABLE_MARGIN_DELTA_MIN    = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.KICKABLE_MARGIN_DELTA_MAX    = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.KICK_RAND_DELTA_FACTOR       = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.EXTRA_STAMINA_DELTA_MIN      = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.EXTRA_STAMINA_DELTA_MAX      = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.EFFORT_MAX_DELTA_FACTOR      = Double.parseDouble(m_tokenizer.nextToken());
    ServerConf.EFFORT_MIN_DELTA_FACTOR      = Double.parseDouble(m_tokenizer.nextToken());
   }

   /**
    * this method is called by the parseData method. it parses the message
    * and stores the elements in the ServerConfig class
    * @roseuid 3CEA8DD90269
    */
   public void parsePlayerTypeData()
   {
    m_tokenizer.nextToken(); // token 'player_type'
    ServerConf.ID               = Integer.parseInt(m_tokenizer.nextToken());
    if (ServerConf.ID==0)    // we use only player 0 at the moment, as we dont have an online Coach so simply discard the others
    {
        ServerConf.PLAYER_SPEED_MAX = Double.parseDouble(m_tokenizer.nextToken());
        ServerConf.STAMINA_INC_MAX  = Double.parseDouble(m_tokenizer.nextToken());
        ServerConf.PLAYER_DECAY     = Double.parseDouble(m_tokenizer.nextToken());
        ServerConf.INERTIA_MOMENT   = Double.parseDouble(m_tokenizer.nextToken());
        ServerConf.DASH_POWER_RATE  = Double.parseDouble(m_tokenizer.nextToken());
        ServerConf.PLAYER_SIZE      = Double.parseDouble(m_tokenizer.nextToken());
        ServerConf.KICKABLE_MARGIN  = Double.parseDouble(m_tokenizer.nextToken());
        ServerConf.KICK_RAND        = Double.parseDouble(m_tokenizer.nextToken());
        ServerConf.EXTRA_STAMINA    = Double.parseDouble(m_tokenizer.nextToken());
        ServerConf.EFFORT_MAX       = Double.parseDouble(m_tokenizer.nextToken());
        ServerConf.EFFORT_MIN       = Double.parseDouble(m_tokenizer.nextToken());
    }
   }

   /**
    * this method is called by the InitialEvent method and call the parser for
    * the corresponding inital-type: init,player params,server params,player type.
    * only the elements of the init-message are stored in the array list. the
    * elements of the other initial-types are stored in the ServerConfig Class
    * @roseuid 3CEB63BB03A9
    */
   public void parseData()
   {
    InitInfo l_init_info = new InitInfo();
    StringTokenizer l_tokenizer = new StringTokenizer(m_message,"() ");
    String l_token = null;

    try
    {
     l_token = l_tokenizer.nextToken();

     int l_initial_type = 0;
     // determine inital type
     while (l_initial_type < INITIAL_TYPE_STRINGS.length)
     {
      if (l_token.equals(INITIAL_TYPE_STRINGS[l_initial_type]))
       break;
       l_initial_type++;
     }

     // when the initial-type is determined the corresponding parser is called
     switch (l_initial_type)
     {
      // initial type: init
      case 0:
       if (l_tokenizer.nextToken().equals("l")) // find out side
       // set side
        l_init_info.m_playingOnLeftSide = true;
       else l_init_info.m_playingOnLeftSide = false;

       l_init_info.m_uniformNumber = Integer.parseInt(l_tokenizer.nextToken()); // set uniform number

       // finding out playmode
       l_token = l_tokenizer.nextToken();
        int l_playmode=0;
        while (l_playmode < AurealInfo.NUMBER_OF_PLAYMODES)
        {
          if (l_token.equals(AurealInfo.PLAYMODE_STRINGS[l_playmode]))
            break;
          l_playmode++;
        }
       l_init_info.m_playmode = l_playmode; // set playmode

       m_objects.add(l_init_info); // elements are inserted into the list
       break;

      // initial type: server param
      case 1:
       parseServerParamData();
       break;

      // inital type: player param
      case 2:
       parsePlayerParamData();
       break;

      // initial type: player type
      case 3:
       parsePlayerTypeData();
       break;
      default:
//       logger.info("Could not determine type of object");
     }
    } // end of try
    catch(Exception e)
    {
//     logger.info("Error parsing see message :"+e.getMessage());
    }
   }
}
