package jais.communication;

import java.util.*;


import jais.objects.*;
import org.apache.log4j.*;

/**
 * <p>Title: SenseBodyEvent</p>
 * <p>Description: This class is called by the Communication class and it parses the incoming SenseBody message and stores the elements in the arrylist m_objects which is used by the World Model</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Oliver Beck
 */
public class SenseBodyEvent extends Event
{
//  private static Logger logger=Logger.getLogger("jais.communication"); //get our logger

  /**
   * @roseuid 3CEB63BD0009
   *
   * this method is called by the Communication class
   * it produces the arraylist and calls the parsing method
   */
  public SenseBodyEvent(String f_message)
  {
    super(f_message);
    m_objects=new ArrayList(1);
    parseData();
  }

  /**
   * @param f_message
   * @roseuid 3CEB63BD000A
   *
   * this method is called by the SenseBodyEvent method. it parses the message
   * and stores the elements in the arrylist
   */
  public void parseData()
  {
    SenseBodyInfo l_sense_body_info = new SenseBodyInfo();
    StringTokenizer l_tokenizer = new StringTokenizer(m_message,"() ");

    // here begins the parsing
    try
    {
      l_tokenizer.nextToken(); // token 'sense_body'

      l_sense_body_info.m_time = Integer.parseInt(l_tokenizer.nextToken()); // element time

      l_tokenizer.nextToken(); // token 'view_mode'
      l_sense_body_info.m_ViewQuality=l_tokenizer.nextToken(); // element view quality
      l_sense_body_info.m_viewMode=l_tokenizer.nextToken(); // element view mode

      l_tokenizer.nextToken(); // token 'stamina'
      l_sense_body_info.m_stamina = Double.parseDouble(l_tokenizer.nextToken()); // element stamina
      l_sense_body_info.m_effort = Double.parseDouble(l_tokenizer.nextToken()); // element effort

      l_tokenizer.nextToken(); // token 'speed'
      l_sense_body_info.m_amountOfSpeed = Double.parseDouble(l_tokenizer.nextToken()); // element amount of speed
      l_sense_body_info.m_directionOfSpeed = Integer.parseInt(l_tokenizer.nextToken()); // element direction of speed

      l_tokenizer.nextToken(); // token 'head_angle'
      l_sense_body_info.m_neckDirection = Integer.parseInt(l_tokenizer.nextToken()); // element head angle

      l_tokenizer.nextToken(); // token 'kick'
      l_sense_body_info.m_kickCount = Integer.parseInt(l_tokenizer.nextToken()); // element kick count

      l_tokenizer.nextToken(); // token 'dash'
      l_sense_body_info.m_dashCount = Integer.parseInt(l_tokenizer.nextToken()); // element dash count

      l_tokenizer.nextToken(); // token 'turn'
      l_sense_body_info.m_turnCount = Integer.parseInt(l_tokenizer.nextToken()); // element turn count

      l_tokenizer.nextToken(); // token 'say'
      l_sense_body_info.m_sayCount = Integer.parseInt(l_tokenizer.nextToken()); // element say count

      l_tokenizer.nextToken(); // token 'turn_neck'
      l_sense_body_info.m_turnNeckCount = Integer.parseInt(l_tokenizer.nextToken()); // element turn neck count

      l_tokenizer.nextToken(); // token 'catch'
      l_sense_body_info.m_catchCount = Integer.parseInt(l_tokenizer.nextToken()); // element catch count

      l_tokenizer.nextToken(); // token 'move'
      l_sense_body_info.m_moveCount = Integer.parseInt(l_tokenizer.nextToken()); // element move count

      l_tokenizer.nextToken(); // token 'change_view'
      l_sense_body_info.m_changeViewCount = Integer.parseInt(l_tokenizer.nextToken()); // element change view count

      m_objects.add(l_sense_body_info); // elements are inserted into the list
    }
    catch(Exception e)
    {
//      logger.info("Error parsing see message :"+e.getMessage());
    }
  }
}