package jais.communication;

import org.apache.log4j.Logger;
import org.apache.log4j.BasicConfigurator;
import jais.objects.*;
import java.util.ArrayList;
import java.util.StringTokenizer;
import java.lang.*;

/**
 * <p>Title: VisualEvent</p>
 * <p>Description: This class is called by the Communication class and it parses the incoming See message and stores the elements in the arrylist m_objects which is used by the World Model</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Oliver Beck
 */
public class VisualEvent extends Event
{
//   private static Logger logger=Logger.getLogger("jais.communication"); //get our logger
   // names and types of the different obj who can appear in the See message
   public static final String[] OBJ_NAME_STRINGS  = {"p","b","g","f","l","B","F","G","P"};
   public static final String[] GOAL_TYPE_STRINGS = {"l","r"};
   public static final String[] LINE_TYPE_STRINGS = {"l","r","t","b"};
   private String m_token = null;
   private StringTokenizer m_tokenizer;

   /**
    * this method is called by the Communication class
    * it produces the arraylist and calls the parsing method
    * @param f_message contains message to parse
    * @roseuid 3CEB63BD01FE
    */
   public VisualEvent(String f_message)
   {
    super(f_message);
    m_tokenizer=new StringTokenizer(m_message,"() ",true);
    m_objects=new ArrayList();
    parseData();
    m_tokenizer=null;
    m_token=null;
   }

   /**
    * this method is called by the VisualEvent method. it parses the message
    * and stores the elements in the arrylist
    * @roseuid 3CEB63BD0208
    */
   public void parseData()
   {
    int l_time;
    PlayerInfo l_player_info;
    BallInfo   l_ball_info;
    GoalInfo   l_goal_info;
    FlagInfo   l_flag_info;
    LineInfo   l_line_info;
    double l_tempValue;

    // here begins the parsing
    try
    {
     m_tokenizer.nextToken(); // token '('
     m_tokenizer.nextToken(); // token 'see'
     m_tokenizer.nextToken(); // token ' '
     l_time=Integer.parseInt(m_tokenizer.nextToken()); // element time
     m_tokenizer.nextToken(); // token ' '

     if(m_tokenizer.hasMoreTokens()) // if player sees nothing message only contains the time
     {
      m_token = m_tokenizer.nextToken(); // token '('

      // here begins the main parsing: if the player sees anything, the
      // message contains more then the time
      // every time a element (direction,distance etc.) is stored in the arraylist
      // the method findEnd() is called to check if the message is going on or finished
      while (m_tokenizer.hasMoreTokens())
      {
       m_tokenizer.nextToken(); // token '('
       m_token = m_tokenizer.nextToken(); // first letter of obj name

       // find out obj name
       int l_obj_name = 0;
       while (l_obj_name < OBJ_NAME_STRINGS.length)
       {
        if (m_token.equals(OBJ_NAME_STRINGS[l_obj_name]))
            break;
        l_obj_name++;
       }

       // when the obj name is determined, one of the 5 following obj parsers is called
       switch(l_obj_name)
       {
        // player: there are two kinds the player message can start with:
        // 'p' means the player is in front of him
        // 'P' means the player is behind him
        case 0: case 8:
         l_player_info = new PlayerInfo();
         l_player_info.setVisible(true);
         l_player_info.setTime(l_time);

         if (l_obj_name==8) // check if it's a 'P'
            l_player_info.setBehind(true);

         // in relation to the distance the player sees the other players,
         // he can't see always every information (teamname,backnumber,goalie y/n)
         // the players contains of.
         // the following 'tree' finds out, what information the player is seeing.
         if(l_obj_name==0)  // check if it's a 'p'
         {
          m_token = m_tokenizer.nextToken();
          if (!m_token.equals(")"))
          {
           m_token = m_tokenizer.nextToken();
           if (!m_token.equals("goalie"))
           {
            m_token = m_token.substring(1,m_token.length()-1);
            l_player_info.setTeamName(m_token);
            m_token = m_tokenizer.nextToken();
            if (!m_token.equals(")"))
            {
             m_token = m_tokenizer.nextToken();
             if (!m_token.equals("goalie"))
             {
              l_player_info.setBackNumber(Integer.parseInt(m_token));
              m_token = m_tokenizer.nextToken();
              if (!m_token.equals(")"))
              {
               m_token = m_tokenizer.nextToken();
               l_player_info.setGoalie(true);
               m_tokenizer.nextToken(); // token ')'
              }
             }
             else {l_player_info.setGoalie(true);m_tokenizer.nextToken();}
            }
           }
           else {l_player_info.setGoalie(true);m_tokenizer.nextToken();}
          }
         } // end of if(l_obj_name==0)
         else
         {
            m_tokenizer.nextToken(); // token ')'
         }
         m_tokenizer.nextToken(); // token ' '

         // set distance (case 0 = player)
         l_tempValue=Double.parseDouble(m_tokenizer.nextToken());
         l_player_info.setDistance(l_tempValue);
         m_tokenizer.nextToken();
         if (findEnd())
         {
          // in 3 of 4 cases in see message the object info ranking is:
          // 1st: 'distance' 2nd: 'direction'
          // in 1 case: 1st: 'direction' so content of direction is unfortunately
          // set to distance
          l_player_info.setDistance(ObjectInfo.NOT_EXISTING);
          l_player_info.setDirection(l_tempValue);
          m_objects.add(l_player_info);
          continue;
         }

         // set direction (case 0 = player)
         l_player_info.setDirection(Double.parseDouble(m_token));
         m_tokenizer.nextToken();
         if (findEnd())
         {
          m_objects.add(l_player_info);
          continue;
         }

         // set distance change (case 0 = player)
         l_player_info.setDistChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken();
         if (findEnd())
         {
          m_objects.add(l_player_info);
          continue;
         }

         // set direction change (case 0 = player)
         l_player_info.setDistChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken();
         if (findEnd())
         {
          m_objects.add(l_player_info);
          continue;
         }

         // set body face direction (case 0 = player)
         l_player_info.setDistChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken();
         if (findEnd())
         {
          m_objects.add(l_player_info);
          continue;
         }

         // set head face direction (case 0 = player)
         l_player_info.setDirChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken();// )
         if (findEnd())
         {
          m_objects.add(l_player_info);
          continue;
         }
         break;

        // ball: there are two kinds the ball message can start with:
        // 'b' means the ball is in front of the player
        // 'B' means the ball is behind him
        case 1 : case 5 :
         l_ball_info=new BallInfo();
         l_ball_info.setVisible(true);
         l_ball_info.setTime(l_time);
         m_tokenizer.nextToken(); // token ')'
         m_tokenizer.nextToken(); // token ' '

         if (l_obj_name==5) // check if it's a 'B'
            l_ball_info.setBehind(true);

         // set distance (case 1 = ball)
         l_tempValue=Double.parseDouble(m_tokenizer.nextToken());
         l_ball_info.setDistance(l_tempValue);
         m_tokenizer.nextToken();
         if (findEnd())
         {
          // in 3 of 4 cases in see message the object info ranking is:
          // 1st: 'distance' 2nd: 'direction'
          // in 1 case: 1st: 'direction' so content of direction is unfortunately
          // set to distance
          l_ball_info.setDistance(ObjectInfo.NOT_EXISTING);
          l_ball_info.setDirection(l_tempValue);
          m_objects.add(l_ball_info);
          continue;
         }

         // set direction (case 1 = ball)
         l_ball_info.setDirection(Double.parseDouble(m_token));
         m_tokenizer.nextToken();
         if (findEnd())
         {
          m_objects.add(l_ball_info);
          continue;
         }

         // set distance change (case 1 = ball)
         l_ball_info.setDistChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken();
         if (findEnd())
         {
          m_objects.add(l_ball_info);
          continue;
         }

         // set direction change (case 1 = ball)
         l_ball_info.setDirChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken();// )
         if (findEnd())
         {
          m_objects.add(l_ball_info);
          continue;
         }
         break;

        // goal: there are two kinds the goal message can start with:
        // 'g' means the goal is in front of the player
        // 'G' means the goal is behind him
        case 2: case 7:
         l_goal_info=new GoalInfo();
         l_goal_info.setVisible(true);
         l_goal_info.setTime(l_time);

         if (l_obj_name==7) // check if it's a 'G'
           l_goal_info.setBehind(true);

         if (l_obj_name==2) // check if it's a 'g'
         {
          m_tokenizer.nextToken(); // token ' '
          // finding out goal type: left or right
          m_token = m_tokenizer.nextToken();
          int l_goal_type = 0;
          while (l_goal_type < GOAL_TYPE_STRINGS.length)
          {
           if (m_token.equals(GOAL_TYPE_STRINGS[l_goal_type]))
            break;
           l_goal_type++;
          }
          l_goal_info.setGoalType(l_goal_type); // set goal type

          m_tokenizer.nextToken(); // token ')'
         }
         else {m_tokenizer.nextToken();} // token ')' of (G)
         m_tokenizer.nextToken(); // token ' '

         // set distance (case 2 = goal)
         l_tempValue=Double.parseDouble(m_tokenizer.nextToken());
         l_goal_info.setDistance(l_tempValue);
         m_tokenizer.nextToken();
         if (findEnd())
         {
          // in 3 of 4 cases in see message the object info ranking is:
          // 1st: 'distance' 2nd: 'direction'
          // in 1 case: 1st: 'direction' so content of direction is unfortunately
          // set to distance
          l_goal_info.setDistance(ObjectInfo.NOT_EXISTING);
          l_goal_info.setDirection(l_tempValue);
          m_objects.add(l_goal_info);
          continue;
         }

         // set direction (case 2 = goal)
         l_goal_info.setDirection(Double.parseDouble(m_token));
         m_tokenizer.nextToken(); // ')'
         if (findEnd())
         {
          m_objects.add(l_goal_info);
          continue;
         }

         // set distance change (case 2 = goal)
         l_goal_info.setDistChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken();
         if (findEnd())
         {
          m_objects.add(l_goal_info);
          continue;
         }

         // set direction change (case 2 = goal)
         l_goal_info.setDirChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken();// )
         if (findEnd())
         {
         m_objects.add(l_goal_info);
         continue;
         }
         break;

        // flag: there are two kinds the flag message can start with:
        // 'f' means the flag is in front of the player
        // 'F' means the flag is behind him
        case 3: case 6:
         l_flag_info=new FlagInfo();
         l_flag_info.setVisible(true);
         l_flag_info.setTime(l_time);

         if (l_obj_name==6) // check if it's a 'F'
           l_flag_info.setBehind(true);

         if (l_obj_name==3) // check if it's a 'f'
         {
          m_tokenizer.nextToken(); // token ' '

          // finding out flag type
          String l_flag_type = "";
          // letters after f
          m_token = m_tokenizer.nextToken();
          while(!m_token.equals(")"))
          {
           l_flag_type += m_token;
           m_token = m_tokenizer.nextToken();
          }

          l_flag_info.setFlagType(l_flag_type); // set flag type
         } // end of if (l_obj_name==3)
         else {m_tokenizer.nextToken();} // token ')' of (F)

         m_tokenizer.nextToken(); // token ' '

         // set distance (case 3 = flag)
         l_tempValue=Double.parseDouble(m_tokenizer.nextToken());
         l_flag_info.setDistance(l_tempValue);
         m_tokenizer.nextToken(); // ' '
         if (findEnd())
         {
          // in 3 of 4 cases in see message the object info ranking is:
          // 1st: 'distance' 2nd: 'direction'
          // in 1 case: 1st: 'direction' so content of direction is unfortunately
          // set to distance
          l_flag_info.setDistance(ObjectInfo.NOT_EXISTING);
          l_flag_info.setDirection(l_tempValue);
          m_objects.add(l_flag_info);
          continue;
         }

         // set direction (case 3 = flag)
         l_flag_info.setDirection(Double.parseDouble(m_token));
         m_tokenizer.nextToken();
         if (findEnd())
         {
          m_objects.add(l_flag_info);
          continue;
         }
         // set distance change (case 3 = flag)
         l_flag_info.setDistChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken();
         if (findEnd())
         {
          m_objects.add(l_flag_info);
          continue;
         }
         // set direction change (case 3 = flag)
         l_flag_info.setDirChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken(); // )
         if (findEnd())
         {
          m_objects.add(l_flag_info);
          continue;
         }
         break;

         // line
        case 4:
         l_line_info=new LineInfo();
         l_line_info.setVisible(true);
         l_line_info.setTime(l_time);

         m_tokenizer.nextToken(); // token ' '

         // finding out line type
         m_token = m_tokenizer.nextToken();
         int l_line_type = 0;
         while (l_line_type < LINE_TYPE_STRINGS.length)
         {
          if (m_token.equals(LINE_TYPE_STRINGS[l_line_type]))
            break;
          l_line_type++;
         }

         l_line_info.setLineType(l_line_type); // set line type

         m_tokenizer.nextToken(); // token ')'
         m_tokenizer.nextToken(); // token ' '

         // set distance (case 4 = line)
         l_tempValue=Double.parseDouble(m_tokenizer.nextToken());
         l_line_info.setDistance(l_tempValue);
         m_tokenizer.nextToken();
         if (findEnd())
         {
          // in 3 of 4 cases in see message the object info ranking is:
          // 1st: 'distance' 2nd: 'direction'
          // in 1 case: 1st: 'direction' so content of direction is unfortunately
          // set to distance
          l_line_info.setDistance(ObjectInfo.NOT_EXISTING);
          l_line_info.setDirection(l_tempValue);
          m_objects.add(l_line_info);
          continue;
         }

         // set direction (case 4 = line)
         l_line_info.setDirection(Double.parseDouble(m_token));
         m_tokenizer.nextToken();
         if (findEnd())
         {
          m_objects.add(l_line_info);
          continue;
         }

         // set distance change (case 4 = line)
         l_line_info.setDistChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken();
         if (findEnd())
         {
          m_objects.add(l_line_info);
          continue;
         }

         // set direction change (case 4 = line)
         l_line_info.setDirChange(Double.parseDouble(m_token));
         m_tokenizer.nextToken();// )
         if (findEnd())
         {
          m_objects.add(l_line_info);
          continue;
         }
         break;
        default:
//        logger.info("Could not determine type of object");
       } // end of switch object name
      } // end of while
     } // end of if(m_tokenizer.hasMoreTokens())
    } // end of try
    catch (Exception e)
    {
//     logger.info("Error parsing see message :"+e.getMessage());
//     logger.debug("Message to parse was discarded:"+m_message);
     return;
    }
   } // end of method

  /**
   * this method is called by the parseData method. it checks if the message
   * is going on or finished
   * @return boolean true if elements of objects or if hole message is finished
   */
  private boolean findEnd()
  {
    if (!m_tokenizer.hasMoreTokens())
     return true;
    else
    {
     m_token = m_tokenizer.nextToken();
     if (m_token.equals(")"))
      return true;
     else
     {
      if (m_token.equals(" "))
      {
       m_token = m_tokenizer.nextToken();
       if (m_token.equals("("))
       return true;
       else return false;
      }
      else return false;
     }
    }
  }
} // end of class



