package jais.objects;

import jais.communication.HearEvent;

/**
 * <p>Title: AurealInfo</p>
 * <p>Description: This class is a data container for a Hear Message. So the members are public</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Oliver Beck, Stephan Diederich
 */
public class AurealInfo
{
    /* defined all the playmodes here*/
    public static final int BEFORE_KICK_OFF	   = 0;
    public static final int PLAY_ON		   = 1;
    public static final int TIME_OVER	           = 2;
    public static final int KICK_OFF_L		   = 3;
    public static final int KICK_OFF_R		   = 4;
    public static final int KICK_IN_L		   = 5;
    public static final int KICK_IN_R		   = 6;
    public static final int FREE_KICK_L		   = 7;
    public static final int FREE_KICK_R		   = 8;
    public static final int CORNER_KICK_L	   = 9;
    public static final int CORNER_KICK_R	   = 10;
    public static final int GOAL_L		   = 11;
    public static final int GOAL_R		   = 12;
    public static final int GOAL_KICK_L		   = 13;
    public static final int GOAL_KICK_R		   = 14;
    public static final int DROP_BALL		   = 15;
    public static final int OFFSIDE_L		   = 16;
    public static final int OFFSIDE_R		   = 17;
    public static final int FOUL_L		   = 18;
    public static final int FOUL_R		   = 19;
    public static final int GOALIE_CATCH_BALL_L    = 20;
    public static final int GOALIE_CATCH_BALL_R	   = 21;
    public static final int TIME_UP_WITHOUT_A_TEAM = 22;
    public static final int TIME_UP		   = 23;
    public static final int HALF_TIME		   = 24;
    public static final int TIME_EXTENDED	   = 25;

    public static final int NUMBER_OF_PLAYMODES    = 26;

    /*corresponding strings to the playmodes*/
    public static final String[] PLAYMODE_STRINGS = {
		        "before_kick_off",
                        "play_on",
		        "time_over",
		        "kick_off_l",
                        "kick_off_r",
		        "kick_in_l",
		        "kick_in_r",
		        "free_kick_l",
		        "free_kick_r",
		        "corner_kick_l",
		        "corner_kick_r",
                        "goal_l",
                        "goal_r",
		        "goal_kick_l",
        	        "goal_kick_r",
		        "drop_ball",
                        "offside_l",
	                "offside_r",
                        "foul_l",
                        "foul_r",
        	        "goalie_catch_ball_l",
		        "goalie_catch_ball_r",
                        "time_up_without_a_team",
                        "time_up",
                        "half_time",
                        "time_extended"
		        };

    /*aural senders*/
    public static final int AUREAL_SENDER_REFEREE     = 0;
    public static final int AUREAL_SENDER_COACH_LEFT  = 1;
    public static final int AUREAL_SENDER_COACH_RIGHT = 2;
    public static final int AUREAL_SENDER_SELF	      = 3;
    public static final int AUREAL_SENDER_OTHER       = 4;

    public static final int NUMBER_OF_SENDERS         = 4;

    /*corresponding strings*/
    public static final String[] AUREAL_SENDER={
                "referee",
                "online_coach_right",
                "online_coach_left",
                "self"
                };

    // different contents of the hear message
    public int m_time;
    public int m_sender;
    public int m_playmode;
    public double m_direction;
    public String m_message;

    /**
     * Access Method for time
     * @return time we got that object
     */
    public int getTime(){return m_time;}

    /**
     *Access Method for sender of this message
     * @return sender of this message, defined in SENDER...
     */
    public int getSender(){return m_sender;};

    /**
     * Access Method for direction the message came from
     * @return angle in degrees the message came from
     */
    public double getDirection(){return m_direction;};

    /**
     * Constructor initializes this Object with the NOT_EXISTING value defined in ObjectInfo
     * @roseuid 3CEB69290069
     */
    public AurealInfo()
    {
        m_time=ObjectInfo.NOT_EXISTING;
        m_sender=ObjectInfo.NOT_EXISTING;
        m_playmode=ObjectInfo.NOT_EXISTING;
        m_direction=ObjectInfo.NOT_EXISTING;
        m_message=new String();
    }
}
