package jais.objects;

/**
 *
 * <p>Title: Class Coordinate - a 2d Vector/Coordinate</p>
 * <p>Description: This class can be used for a 2d Vector or Coordinate and provides several methods for calculating with them</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p> </p>
 * @author Claus Kestel, Stephan Diederich
 *
 */
public class Coordinate {
  double m_x, m_y;

  private final double NEARLY   =   4; //constant for checking if another Coordinate is nearly the same as this

  /**
   * Constructor creates a new Coordinate, with f_x and f_y as parameters
   * @param f_x the x-value for this Coordinate
   * @param f_y the y-value for this Coordinate
   */
  public Coordinate(double f_x, double f_y)
  {
    m_x = f_x;
    m_y = f_y;
  }

  /**
   * Method for accessing the x-part of this Coordinate
   * @return x-part of this Coordinate
   */
  public double getX(){ return m_x; }

  /**
   * Method for accessing the y-part of this Coordinate
   * @return y-part of this Coordinate
   */
  public double getY(){ return m_y; }

  /**
   * Method for setting the x-part of this Coordinate
   * @param f_x the x-part to which it should be setted
   */
  public void setX(double f_x){ m_x=f_x; }

  /**
   * Method for setting the y-part of this Coordinate
   * @param f_y the y-part to which it should be setted
   */
  public void setY(double f_y){ m_y=f_y; }

  /**
   * Decides if the coordinate is exactly the same as the argument
   * @param f_coord Coordinate to compare to
   * @return true if it is exactly the same, false if not
   */
  public boolean exactlyTheSameAs(Coordinate f_coord)
  {
    return (m_x == f_coord.getX() && m_y == f_coord.getY());
  }

 /**
   * Decides if the coordinate is nearly the same as the argument(
   * @param f_coord Coordinate to compare to
   * @return true if it is nearly the same, false if not
   */
  public boolean nearlyTheSameAs(Coordinate f_coord)
  {
    return ((Math.abs(m_x - f_coord.getX())< NEARLY) && (Math.abs(m_y - f_coord.getY()) < NEARLY) ) ;
  }

  /**
   * method for representing this coordinate rounded to long as a string
   * @return this coordinate represented in a string
   */
  public String toString(){
    return "(" + m_x + ", " + m_y + ")";
  }

  /**
   * Add two Coordinates (vectors in coordinate-system)
   * @param f_coord Coordinate to add to this one
   * @return Coordinate as result of this calculation
   */
  public Coordinate add(Coordinate f_coord)
  {
    return (new Coordinate(this.m_x+f_coord.getX(),this.m_y+f_coord.getY()));
  }

  /**
   * sub f_coord from this coordinate
   * @param f_coord Coordinate to substract from this one
   * @return Coordinate as result of this calculation
   */
  public Coordinate sub(Coordinate f_coord)
  {
    return (new Coordinate(this.m_x-f_coord.getX(),this.m_y-f_coord.getY()));
  }

  /**
   * scalar multiplication of f_factor and this coordinate
   * @param f_factor value to multiply with
   * @return Coordinate scaled with f_factor
   */
  public Coordinate scaleWith(double f_factor)
  {
    return (new Coordinate(this.m_x*f_factor, this.m_y *f_factor));
  }

  public Coordinate normalize()
  {
    return (this.scaleWith(Math.sqrt(m_x*m_x+m_y*m_y)));
  }

  /**
   * This method calculates the distance between this coordinate and f_coord
   * @param f_coord coordinate to calculate distance to
   * @return distance as double
   */
  public double getDistanceTo(Coordinate f_coord)
  {
    return Math.sqrt( Math.pow(f_coord.getX()-this.getX(),2) + Math.pow(f_coord.getY()-this.getY(),2));
  }
}
