package jais.objects;

/**
 * <p>Title: ObjectInfo</p>
 * <p>Description: This class contains methods which sets or delivers some elements of the see message</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Oliver Beck, Stephan Diederich
 */

public class ObjectInfo
{
   public static final int NOT_EXISTING = -10000; //for values that doesnt exist, (server didnt give Info)!

    // These are the variables set by Parser. They are directly given from the server
    protected double  m_distance;
    protected double  m_direction;
    protected double  m_distChange;
    protected double  m_dirChange;
    protected int     m_time;
    private boolean   m_isBehind;

    // These variables are for use in the worldmodel
    private boolean m_isVisible=false;;
    protected Coordinate m_absPosition;
    protected Coordinate m_absPositionOfNextRound;

    /**
     * Access method delivers the distance of the object
     * @return the distance of the object
     */
    public double getDistance() {return m_distance;}

    /**
     * This method sets the distance of the object
     * @param f_distance contains the distance of the object
     */
    public void setDistance(double f_distance) {m_distance=f_distance;}

    /**
     * Access method if object is behind the player or not
     * @return boolean true if object is behind player
     */
    public boolean isBehind() {return m_isBehind;}

    /**
     * This method sets a boolean value true or false if the object is behind the player or not
     * @param f_isBehind contains boolean true if the object is behind the player
     */
    public void setBehind(boolean f_isBehind) {m_isBehind=f_isBehind;}

    /**
     * Access method delivers the distance change of the object
     * @return the distance change of the object
     */
    public double getDistChange() {return m_distChange;}

    /**
     * This method sets the distance change of the object
     * @param f_distChange contains the value of the distance change of the object
     */
    public void setDistChange(double f_distChange) {m_distChange =f_distChange;}

    /**
     * Access method delivers the direction change of the object
     * @return the direction change of the object
     */
    public double getDirChange() {return m_dirChange;}

    /**
     * This method sets the direction change of the object
     * @param f_dirChange contains the value of the direction change
     */
    public void setDirChange(double f_dirChange) {m_dirChange=f_dirChange;}

    /**
     * Access method delivers the direction in degrees of the object
     * @return value in degrees of the direction of the object
     */
    public double getDirection() {return m_direction;}

    /**
     * This method sets the direction in degrees of the object
     * @param f_direction contains the direction of the object in degrees
     */
    public void setDirection(double f_direction) {m_direction=f_direction;}

    /**
     * This method sets a boolean value true or false if the object is visible or not
     * @param f_isVisible contains a boolean true if object is visible
     */
    public void setVisible(boolean f_isVisible) {m_isVisible=f_isVisible;}

    /**
     * Access method delivers boolean true if object is visible
     * @return boolean true if object is visible
     */
    public boolean isVisible() {return m_isVisible;}

    /**
     * This method sets the absolute position of the object
     * @param f_absPosition contains the absolute position of the object
     */
    public void setAbsPosition(Coordinate f_absPosition) { m_absPosition=f_absPosition;}

    /**
     * Access method delivers absolute position of the object
     * @return absolute position of the object
     */
    public Coordinate getAbsPosition() {return m_absPosition;}

    /**
     * This method sets the absolute position of the next round
     * @param f_absPositionOfNextRound contains a value with the absolute position of the next round
     */
    public void setAbsPositionOfNextRound(Coordinate f_absPositionOfNextRound) { m_absPositionOfNextRound=f_absPositionOfNextRound;}

    /**
     * Access method delivers the absolute position of the next round
     * @return value with the absolute position of the next round
     */
    public Coordinate getAbsPositionOfNextRound() {return m_absPositionOfNextRound;}

    /**
     * Access method delivers the time the info about the object was received
     * @return value containing the time the info about the object was received
     */
    public int getTime(){return m_time;}

    /**
     * This method sets the time the info about the object was received
     * @param f_time contains the time the info about the object was received
     */
    public void setTime(int f_time) {m_time=f_time;}


   /**
    * Access method delivers the distance to the specified object
    * @return distance to the specified Object
    * @param f_coord contains the position of the specified object
    * @roseuid 3CEB66B400C8
    */
   public double getDistanceTo(Coordinate f_coord)
   {
    return (Math.sqrt((Math.pow(Math.abs(m_absPosition.getX() - f_coord.getX()), 2)) +
       (Math.pow(Math.abs(m_absPosition.getY() - f_coord.getY()), 2))));
        //Pythagoras *gg*
   }

   /**
    * Constructor initializes this Object with the NOT_EXISTING value defined in ObjectInfo
    * @roseuid 3CEB67CD0109
    */
    public ObjectInfo()
    {
        m_absPosition=new Coordinate(0,0); //initialize this... damnshit nullpointer exceptions!
        m_absPositionOfNextRound=null;
        m_distance=NOT_EXISTING;
        m_direction=NOT_EXISTING;
        m_distChange=NOT_EXISTING;
        m_dirChange=NOT_EXISTING;
        m_time=NOT_EXISTING;
        m_isBehind=false;
    }
}
