package jais.objects;

/**
 * <p>Title: SenseBodyInfo</p>
 * <p>Description: This class contains members and methods which sets or delivers
 * the infos of the SenseBody message</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * @author Oliver Beck
 */
public class SenseBodyInfo
{
  //different view modes
  public static final int NARROW = 0;
  public static final int NORMAL = 1;
  public static final int WIDE = 2;

  // These are the variables set by Parser. They are directly given from the server
  public double m_neckDirection;
  public int    m_time;
  public String m_ViewQuality;
  public String m_viewMode;
  public double m_stamina;
  public double m_effort;
  public double m_amountOfSpeed;
  public int    m_directionOfSpeed;
  public int    m_kickCount;
  public int    m_dashCount;
  public int    m_turnCount;
  public int    m_sayCount;
  public int    m_turnNeckCount;
  public int    m_catchCount;
  public int    m_moveCount;
  public int    m_changeViewCount;


  /**
   * This method returns the corresponding angle to the given viewmode
   * @param f_viewmode the viewmode we want the angle of
   * @return Angle of f_viewmode
   */
  public static int getAngleFromViewMode(String f_viewmode)
  {
    if(f_viewmode.equalsIgnoreCase("wide"))
      return 180;
    else if(f_viewmode.equalsIgnoreCase("normal"))
      return 90;
    else if(f_viewmode.equalsIgnoreCase("narrow"))
      return 45;
    return -1;
  }


  /**
   * This method sets the neck direction
   * @param f_neckDirection contains the value which is setted
   */
  public void setNeckDirection(double f_neckDirection) { m_neckDirection=f_neckDirection;}

  /**
   * Access method for time
   * @return the time
   */
  public int getTime() {return m_time;}

}