package jais.player;


import jais.objects.*;
import jais.player.*;
import jais.world.WorldModel;
import jais.communication.*;
import java.util.*;
import org.apache.log4j.*;

/**
 *
 * <p>Title: KI </p>
 * <p>Description: This class decides which action we should perform</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p> </p>
 * @author Christian Traub, Stephan Diederich and Claus Kestel
 *
 */


public class KI extends Thread implements Player

{
  Thread m_KIThread; // a reference to ourself
  boolean m_threadSuspended=false; //is our thread currently suspended
  private static Logger logger=Logger.getLogger("jais.player");


  //Feste Aktionsradien [PlayState(Defense=0/Offense=1)][RueckenNummer]


  private final Coordinate[] HOMEPOS = {
    /*goalie*/     new Coordinate(0,0), new Coordinate(-50,0),
    /*defense*/    new Coordinate(-25,20), new Coordinate(-30,7), new Coordinate(-30,-7),new Coordinate(-25,-20),
    /*midfield*/   new Coordinate(0,10), new Coordinate(0,0), new Coordinate(0,-10),
    /*attackers*/  new Coordinate(30,18), new Coordinate(25,0), new Coordinate(30,-18)};

  private final Coordinate GOALIE_HOME_POS = new Coordinate(-50, 0);

  private final double NORMPOWER = 50;    //(ServerConf.MAXPOWER / 2);
  private final double MINPOWER = 25;    //(ServerConf.MAXPOWER / 4);
  private final double MAXPOWER = 100;  //ServerConf.MAXPOWER;

  private final int HOMERANGE_R = 6;
  private final Coordinate MY_GOAL_POS = new Coordinate(-52.5, 0);
  private final Coordinate ENEMY_GOAL_POS = new Coordinate(52.5, 0);
  private final int MAX_GOAL_KICK_DISTANCE = 30;
  private final int MIN_DISTANCE_FOR_DRIBBEL = 10;
  private final int MAX_PASS_DISTANCE =30;
  private final int MAX_LOOKING_FORWARD_TIME = 10;
  private final int MAX_ANGLE_FOR_GOAL_KICK = 75;
  private final int FREE_ANGLE_FOR_GOAL_KICK = 5;

  private final int FREE_ANGLE = 15;

  ArrayList m_enemyTeam;
  ArrayList m_ownTeam;
  WorldModel m_myWorld;
  ActionManager m_actions;
  StateManager m_myState;

  /**
   * Creates a new <code>KI</code> instance.
   *
   * @param f_world
   * @param f_myCom
   * @param f_playerNumber
   * @roseuid 3CE8EDED0385
   */
  public KI(WorldModel f_world, Communication f_myCom,int f_playerNumber)
  {
    this.setName("KI"+f_playerNumber);
    m_myWorld = f_world;
    m_myState = new StateManager();
    m_actions = new ActionManager(m_myWorld, f_myCom, f_playerNumber, m_myState);
  }

  /**
   * This method is started after a new visual info
   *
   * @roseuid 3CE80E3F03D8
   */
  public void run()
  {
    Thread l_thisThread = currentThread();

    while(l_thisThread==m_KIThread)
    {
      //ok, we were waked up, so theres new information avaiable. discard last actions and decide new
      m_actions.flush();
      //if (m_actions.listEmpty())
      {
        try
        {
          m_actions.InformVisualUpdateHappened();
          m_enemyTeam=m_myWorld.getEnemyPlayers();
          m_ownTeam=m_myWorld.getFriendlyPlayers();
          root();  //Start our tree in the root
        }
        catch (Exception ex)
        {
          logger.debug("Caught Execption from root in KI"+ex.getMessage());
          //ex.printStackTrace();
          logger.debug("Report this to jais-devel@lists.sourceforge.net. KI is still Working!");
        }
      }
      m_threadSuspended=true; //we want to suspend ourself after one run
      try
      {
        synchronized(this)
        {
          while (m_threadSuspended && l_thisThread==m_KIThread) //check again if we should sleep,
          {
            //logger.info("\t" + getName() + ": Going to wait now for something new");
            wait(); // this command lets our thread sleep until we wake him up
          }
        }
      }
      catch(java.lang.Exception ex)
      {
        //logger.info("\t" + getName() + ": ARGG! Got an exception while sleeping ... "+ ex.getMessage());
      }
    }//of while()
    //logger.info("\t" + getName() + ": i've left my while-loop. run() will stop immediatly

  }


  /**
   * This "actionListener" waits for our sync-Event and passes it to the ActionManager
   */
  public void onSenseBody()
  {
    m_actions.activatePerformCounter();  //perform the next skill in our "Aktion-List"
  }

  /**
   * This is the root of our decisiontree
   */
  private void root()
  {
    manageViewMode();
    switch(m_myWorld.getPlaymode())
    {
      case AurealInfo.PLAY_ON:
        onPlay();
        break;
      case AurealInfo.BEFORE_KICK_OFF:
        setPlayers(false);
        break;
      case AurealInfo.GOAL_L:
        setPlayers(false);
        break;
      case AurealInfo.GOAL_R:
        setPlayers(true);
        break;
      case AurealInfo.KICK_OFF_L:
        Kick_Off(true);
        break;
      case AurealInfo.KICK_OFF_R:
        Kick_Off(false);
        break;
      case AurealInfo.KICK_IN_L:
        Kick_In(true);
        break;
      case AurealInfo.KICK_IN_R:
        Kick_In(false);
        break;
      case AurealInfo.CORNER_KICK_L:
        Kick_In(true);
// 		Corner_Kick(true);
        break;
      case AurealInfo.CORNER_KICK_R:
        Kick_In(false);
        // 		Corner_Kick(false);
        break;
      case AurealInfo.FREE_KICK_L:
        Kick_In(true);
        // 		freeKick(true);
        break;
      case AurealInfo.FREE_KICK_R:
        Kick_In(false);
        // 		freeKick(false);
        break;
      case AurealInfo.GOAL_KICK_L:
        goalKick(true);
        break;
      case AurealInfo.GOAL_KICK_R:
        goalKick(false);
        break;
      case AurealInfo.OFFSIDE_L:
        Kick_In(false);
        // 		freeKick(false);
        break;
      case AurealInfo.OFFSIDE_R:
        Kick_In(true);
        // freeKick(false);
        break;
      default:
        onPlay();
      // logger.info("Hmmm, not implemented playmode error; trying 'play_on'");
    }
  }
  /**
   * Moves our Player to the homepositions
   * @param f_forLeftSide
   */
  private void setPlayers(boolean f_forLeftSide   )
  {
    Coordinate l_tempCoord = new Coordinate(0,0);
    //m_myState.getState()

    if ((m_myWorld.isPlayingLeft()&& f_forLeftSide) || (!m_myWorld.isPlayingLeft()&& !f_forLeftSide )){
      if ( HOMEPOS[m_myWorld.getMyself().getBackNumber()].getX() < 0)
        l_tempCoord.setX(HOMEPOS[m_myWorld.getMyself().getBackNumber()].getX());
      else l_tempCoord.setX(-5);
      l_tempCoord.setY(HOMEPOS[m_myWorld.getMyself().getBackNumber()].getY());
    }
    else{
      if ( HOMEPOS[m_myWorld.getMyself().getBackNumber()].getX() < 0)
        l_tempCoord.setX(HOMEPOS[m_myWorld.getMyself().getBackNumber()].getX());
      else if ( Math.abs(HOMEPOS[m_myWorld.getMyself().getBackNumber()].getY()) < 11)
        l_tempCoord.setX(-10);
      else
        l_tempCoord.setX(-5);
      l_tempCoord.setY(HOMEPOS[m_myWorld.getMyself().getBackNumber()].getY());
    }



    if(m_myWorld.getMyself().getDistanceTo( l_tempCoord) > 3){
      m_actions.moveTo(l_tempCoord);
    }
    else if(!m_myWorld.getBall().isVisible()) //we want to look clever. so look at the ball on beginning
      searchBall();
    else
      watchBall();
  }

  /**
   * Decisions in a kickoff-situation
   * @param f_forLeftSide
   */
  private void Kick_Off(boolean f_forLeftSide)
      {	// Erstmal zur Ausgangsposition

    // Bin ich dran?
    if ((m_myWorld.isPlayingLeft()&& f_forLeftSide) || (!m_myWorld.isPlayingLeft()&& !f_forLeftSide )){
      // Mache ich den Anstoss?
      if (!m_myWorld.getBall().isVisible() )
        searchBall();
      else if (nearestToTheBall()){
        if(m_myWorld.getBall().isBallKickableBy(m_myWorld.getMyself()))
          m_actions.kickTo(MAXPOWER,ENEMY_GOAL_POS,m_myWorld.getViewModeAsInt()); // schn ausrichten und voll draufhauen
        else
          m_actions.runTo(NORMPOWER,m_myWorld.getBall(), m_myWorld.getViewModeAsInt());
      }
    }
    else {
      // Ich bin nicht dran
      gotoHomePosition();
      return;

    }
  }


  /**
   * Decisions in a kickin-situation
   * @param f_forLeftSide
   */
  private void Kick_In(boolean f_forLeftSide)
  {
    PlayerInfo l_tempPlayer;
    //Ball suchen,falls ich ihn nicht sehe
    if (!m_myWorld.getBall().isVisible())
    {
      searchBall();
      return;
    }

    // Sind wir dran?
    if ((m_myWorld.isPlayingLeft()&& f_forLeftSide) || (!m_myWorld.isPlayingLeft()&& !f_forLeftSide ))
    {
      // Bin ich am nchsten am Ball?
      if (nearestToTheBall())
      {
        if (m_myWorld.getMyDistanceTo(m_myWorld.getBall())>3.5)
        {
          m_actions.runTo(MINPOWER+5,new Coordinate(m_myWorld.getBall().getAbsPosition().getX()-3,m_myWorld.getBall().getAbsPosition().getY()),m_myWorld.getViewModeAsInt());
        }
        if(m_myWorld.getBall().getDistance() < ServerConf.KICKABLE_MARGIN + ServerConf.PLAYER_SIZE/2 +ServerConf.BALL_SIZE/2)
        {
          if(m_myWorld.getMyself().getAbsPosition().getX()<-30)
            l_tempPlayer =checkForPassPartner(25);
          else if(m_myWorld.getMyself().getAbsPosition().getX()>40)
            l_tempPlayer =checkForPassPartner(-10);
          else
            l_tempPlayer =checkForPassPartner(13);
          if (l_tempPlayer != null)
            passTo(l_tempPlayer,m_myWorld.getViewModeAsInt());
          else
            kickToCleverPosition(-1);
        }
        else
        {
          m_actions.runTo(MINPOWER, m_myWorld.getBall(), m_myWorld.getViewModeAsInt());
        }
      }
      else gotoHomePosition();
    }
    else
    {
      // Wir sind nicht dran :-(
      // Gehe ein wenig zurck, wenn ich  zu nah am Gegner stehe(stamina abzug)
      if(m_myWorld.getMyDistanceTo(m_myWorld.getBall()) < 13)
      {
        m_actions.run((int)-NORMPOWER);
      }
      else
        gotoHomePosition();
    }
  }//KickIn

  /**
   * Decisions in a goalkick-situation
   * @param f_forLeftSide
   */
  private void goalKick(boolean f_forLeftSide)
  {
    PlayerInfo l_tempPlayer;
    if(m_myWorld.getMyself().isGoalie())
    {
      if ((m_myWorld.isPlayingLeft()&& f_forLeftSide) || (!m_myWorld.isPlayingLeft()&& !f_forLeftSide ))
      {
        // Is Ball Visible
        if(m_myWorld.getBall().isVisible())
        {
          if (m_myWorld.getMyDistanceTo(m_myWorld.getBall())> 3)
          {
            m_actions.runTo(MINPOWER+5, new Coordinate(m_myWorld.getBall().getAbsPosition().getX()-2.5,m_myWorld.getBall().getAbsPosition().getY()), m_myWorld.getViewModeAsInt());
            return;
          }
          if(m_myWorld.getBall().getDistance() < ServerConf.KICKABLE_MARGIN  + ServerConf.PLAYER_SIZE/2 +ServerConf.BALL_SIZE/2)
          {
            l_tempPlayer =checkForPassPartner(25);
            if (l_tempPlayer != null)
              passTo(l_tempPlayer,m_myWorld.getViewModeAsInt());
            else
              kickToCleverPosition(-1);
          }
          else
          {
            m_actions.runTo(MINPOWER+5, m_myWorld.getBall(), m_myWorld.getViewModeAsInt());
          }
        }
        else
          searchBall();
      }
    }
    else //not the goalie
    {
      if (m_myWorld.getBall().isVisible())
        gotoHomePosition();
      else
        searchBall();
    }
  }

  /**
   * Are we offense or defense
   */
  private void checkPlayState()
  {
    if (m_myWorld.getBall().getAbsPosition().getX() > 5)
      m_myState.setOffensive();
    else
      m_myState.setDefensive();
/*
   if (!m_enemyTeam.isEmpty())
   {
    for (ListIterator it = m_enemyTeam.listIterator(); it.hasNext();)
       if (m_myWorld.getBall().isBallKickableBy((PlayerInfo)it.next()))
         m_myState.setDefensive();
    }
   if (!m_ownTeam.isEmpty())
   {
    for (ListIterator it = m_ownTeam.listIterator(); it.hasNext();)
      if (m_myWorld.getBall().isBallKickableBy((PlayerInfo)it.next()))
        m_myState.setOffensive();
    } //check empty list   */
  }

  /**
   * Watch the ball
   * @roseuid 3CE80EAF0138
   */
  private void watchBall()
  {
    if (Math.abs(m_myWorld.getBodyDirectionTo(m_myWorld.getBall()))>0.2)
      m_actions.turn(m_myWorld.getBodyDirectionTo(m_myWorld.getBall()));
  }

  /**
   * Run to the ball
   * @roseuid 3CE80F0D029F
   */
  private void takeTheBall()
  {
    int l_zaehler = 0;
    int l_estimatedCyclesToReachTheBall = 0;
    double l_smallestDistance;
    double l_nextDistance = 0;
    double l_ballSpeed = 0;
    Coordinate l_diffPos;
    if (m_myWorld.getBall().getAbsPositionOfNextRound() != null)
    {
      if (m_myWorld.getMyDistanceTo(m_myWorld.getBall()) >= m_myWorld.getMyDistanceTo(m_myWorld.getBall().getAbsPositionOfNextRound()))
      {
        //Ball kommt zu mir
        l_smallestDistance = m_myWorld.getMyDistanceTo(m_myWorld.getBall());
        while (l_zaehler < MAX_LOOKING_FORWARD_TIME )
        {
          l_zaehler++;
          l_nextDistance = m_myWorld.getMyDistanceTo(getBallPosInRound(l_zaehler));
          if (l_nextDistance < l_smallestDistance)
          {
            l_smallestDistance = l_nextDistance;
            l_estimatedCyclesToReachTheBall = l_zaehler;
          }
        }
      }
      else
      {
        //Ball entfernt sich
        Coordinate l_myTempPos =m_myWorld.getMyself().getAbsPosition();
        l_diffPos = m_myWorld.getMyself().getAbsPositionOfNextRound().sub(l_myTempPos);
        l_smallestDistance = 100;
        while (l_zaehler < MAX_LOOKING_FORWARD_TIME )
        {
          l_zaehler++;
          l_myTempPos=l_myTempPos.add(l_diffPos.scaleWith(m_myWorld.getViewModeAsInt()));
          l_nextDistance = l_myTempPos.getDistanceTo(getBallPosInRound(l_zaehler));
          if (l_nextDistance < l_smallestDistance)
          {
            l_smallestDistance = l_nextDistance;
            l_estimatedCyclesToReachTheBall = l_zaehler;
          }
        }
      }
      m_actions.runTo(MAXPOWER,getBallPosInRound(l_estimatedCyclesToReachTheBall), m_myWorld.getViewModeAsInt());
    }
    else //ball doesnt move so simply go to its position
      m_actions.runTo(MAXPOWER,m_myWorld.getBall(), m_myWorld.getViewModeAsInt());
  }

  /**
   * This method is called if we dont see the ball but we want to find it
   * @roseuid 3CE80F420242
   */
  private void searchBall()
  {
    //check where the ball was the last time we saw it and turn to that direction
    // if it is behind us, simply run a bit backwards
    if (m_myWorld.getBall().getDirection() > 0)
      m_actions.turn(SenseBodyInfo.getAngleFromViewMode(m_myWorld.getViewMode())-15);
    else
      m_actions.turn(-SenseBodyInfo.getAngleFromViewMode(m_myWorld.getViewMode())+15);
  }


  /**
   * This is the decisiontree in normal playmode
   * @roseuid 3CE8E81800AD
   */
  private void onPlay()
  {
    Coordinate l_tempCoord;
    PlayerInfo l_tempPlayer;
    checkPlayState();   //First check out our tactic

    if (m_myWorld.getMyself().isGoalie())
      isGoalie();
    else
    {
      int l_cycle=isBallKickableByMe();
      if (m_myWorld.getBall().isVisible() && l_cycle >= 0)  //hab den ball
      {
        l_tempCoord = checkGoalKick();
        if (l_tempCoord != null)
        {
          m_actions.kickTo(MAXPOWER, l_tempCoord,l_cycle);
        }
        else
        {
          l_tempPlayer =checkForPassPartner(10);
          if (l_tempPlayer != null)
          {
            passTo(l_tempPlayer,l_cycle);
          }
          else
          {
            if (checkDribble())
            {
              dribble(l_cycle);
            }
            else
            {
              kickToCleverPosition(l_cycle);
            }
          }
        }
      }
      else
      {
        if (m_myWorld.getBall().isVisible())
        {
          //very good, i see the ball, now checking what to do
          //check if i'm the nearest Player to the ball
          if (!nearestToTheBall())
          {
            //fine, someone is nearer, looking what to do now
            if (m_myState.getState() == StateManager.OFFENSE)
              gotoBestPositionInRange();
            else   //playing defense
            {
              l_tempPlayer = enemyInRange();
              if (l_tempPlayer != null )
                upsetEnemy(l_tempPlayer);    //try to 'destroy' the enemy player
              else   //no enemy in my range.
              {
                gotoHomePosition();
              }
            }
          }
          else{
            //nearest to the ball
            takeTheBall();
          }
        }
        else
        {
          searchBall();  //ball not visible
        }
      }//havent got the ball
    }//not goalie
  }//on play

  /**
   * Go the the best position near the homeposition
   * @roseuid 3CE80F300109
   */
  private void gotoBestPositionInRange()
  {
    /** @todo  do something clever here */
    gotoHomePosition();
    PlayerInfo l_player=enemyInRange();
    if (l_player!=null) // we should walk free then
      goBetweenBallAndEnemyAndWatchBall(l_player);
  }

  /**
   * Go to the homeposition
   * @roseuid 3CE80E9C0196
   */
  private void gotoHomePosition()
  {
    if (m_myState.m_currentViewWidth!=SenseBodyInfo.WIDE)
    {
      m_actions.changeView(SenseBodyInfo.WIDE,true);
      m_myState.m_currentViewWidth=SenseBodyInfo.WIDE;
    }
    double l_offsideBarrier=getOffsideBarrier(m_enemyTeam,m_ownTeam); //seems there are
    if(m_myWorld.getMyself().getAbsPosition().getX()>l_offsideBarrier-3)
    {
      m_actions.runTo(NORMPOWER , new Coordinate( l_offsideBarrier-5, HOMEPOS[m_myWorld.getMyself().getBackNumber()].getY()), m_myWorld.getViewModeAsInt());
    }
    else if(atHomepos())
      watchBall();
    else
    {
      if (m_myWorld.getSenseBodyInfo().m_stamina > 1500)
        m_actions.runTo(NORMPOWER , HOMEPOS[m_myWorld.getMyself().getBackNumber()], m_myWorld.getViewModeAsInt());
    }
  }

  /**
   * The goalie should go between ball and goal and watch the ball, but don't go farer away from the goal than a given distance
   * @param f_radius
   * @roseuid 3CE80E66000F
   */
  private void goBetweenBallGoalAndWatchIt(double f_radius)
  {
    Coordinate l_runToPos = new Coordinate(0, 0);
    double l_angleOfBall = 0;
    l_angleOfBall = Math.atan((Math.abs(MY_GOAL_POS.getY() - m_myWorld.getBall().getAbsPosition().getY()) ) / (Math.abs(MY_GOAL_POS.getX() - m_myWorld.getBall().getAbsPosition().getX()) ));
    l_runToPos.setX(MY_GOAL_POS.getX() + (Math.cos(l_angleOfBall) * f_radius));
    if (m_myWorld.getBall().getAbsPosition().getY() >= 0)
      l_runToPos.setY(MY_GOAL_POS.getY() + (Math.sin(l_angleOfBall) * f_radius));
    else
      l_runToPos.setY(MY_GOAL_POS.getY() + (-Math.sin(l_angleOfBall) * f_radius));
    if (Math.abs(m_myWorld.getMyself().getDistanceTo(l_runToPos)) > 0.8)
      m_actions.runTo(NORMPOWER, l_runToPos, m_myWorld.getViewModeAsInt());
    else
      watchBall();
  }

  /**
   * Go to a given Enemy
   * @param f_player
   */
  private void gotoEnemy(PlayerInfo f_player)
  {
    m_actions.runTo(NORMPOWER, (f_player.getAbsPositionOfNextRound() != null) ? f_player.getAbsPositionOfNextRound() : f_player.getAbsPosition(), m_myWorld.getViewModeAsInt());
  }

  /**
   * Returns true if I'am the goalie
   */
  private void isGoalie()
  {
    if (m_myWorld.getBall().isVisible())
    {
      int l_cycles = isBallCatchable();
      if (l_cycles>=0)
      {
        m_actions.catchTheBall(l_cycles);
        return;
      }
      else
      {
        int l_cycle=isBallKickableByMe();
        if (l_cycle >= 0)
        {
          PlayerInfo l_tempPlayer =checkForPassPartner(30);
          if (l_tempPlayer != null)
          {
            passTo(l_tempPlayer,l_cycle);
          }
          else
          {
            kickToCleverPosition(l_cycle);
          }
        }
        else
        {
         // if (nearestToTheBall())
         //   takeTheBall();
          Coordinate l_coord = (m_myWorld.getBall().getAbsPositionOfNextRound()!=null) ? m_myWorld.getBall().getAbsPositionOfNextRound() : m_myWorld.getBall().getAbsPosition();
          if(l_coord.getX()<-41 && Math.abs(l_coord.getY())<14)
          {
            m_actions.runTo(MAXPOWER,l_coord,m_myWorld.getViewModeAsInt());
          }
          else
          {
            if(l_coord.sub(m_myWorld.getBall().getAbsPosition()).getX()>0 && l_coord.sub(m_myWorld.getBall().getAbsPosition()).getY()>0)
            {
               goBetweenBallGoalAndWatchIt(ServerConf.GOAL_WIDTH/2);
            }
            else
              if (nearestToTheBall())
                takeTheBall();
            else
              goBetweenBallGoalAndWatchIt(ServerConf.GOAL_WIDTH/2);
          }
        }
      }
    }
    else
    {
      searchBall();
    }
  }

  private int isBallCatchable()
  {
    if ((Math.abs(m_myWorld.getBall().getAbsPosition().getY())<20 && m_myWorld.getBall().getAbsPosition().getX()<-36)
        && m_myWorld.getMyself().getDistanceTo(getBallPosInRound(0)) <= ServerConf.CATCHABLE_AREA_L )
    {
      return 0;
    }
    else if (m_myWorld.getBall().getAbsPositionOfNextRound()!=null
             && (Math.abs(m_myWorld.getBall().getAbsPositionOfNextRound().getY())<20 && m_myWorld.getBall().getAbsPositionOfNextRound().getX()<-36)
             && m_myWorld.getMyDistanceTo(m_myWorld.getBall().getAbsPositionOfNextRound()) <= ServerConf.CATCHABLE_AREA_L  )
    {
      return 1;
    }
    return -1;
  }




  /**
   * Return the best position for a goalkick if a goalkick is posible
   * @return Coordinate
   */
  /** Find the best position on goal  **/
  private Coordinate checkGoalKick()
  {
    if (!m_enemyTeam.isEmpty())
    {
      if (m_myWorld.getMyDistanceTo(ENEMY_GOAL_POS) > MAX_GOAL_KICK_DISTANCE)  //Distance to great for a succesful goal kick
        return null;

      PlayerInfo l_enemyPlayer = null;
      Coordinate l_shortCorner = ENEMY_GOAL_POS;
      Coordinate l_longCorner = ENEMY_GOAL_POS;
      Coordinate l_middleOfGoal = ENEMY_GOAL_POS;
      double l_angleToGoal = 0;

      l_angleToGoal = Math.atan((Math.abs(ENEMY_GOAL_POS.getY() - m_myWorld.getMyself().getAbsPosition().getY()) ) / (Math.abs(ENEMY_GOAL_POS.getX() - m_myWorld.getMyself().getAbsPosition().getX())));
      if (l_angleToGoal > MAX_ANGLE_FOR_GOAL_KICK)
        return null;

//Checking where I'm standing
      if (m_myWorld.getMyself().getAbsPosition().getY() > ENEMY_GOAL_POS.getY())  //standing 'under' the middle goal
      {
        l_shortCorner.setY(ENEMY_GOAL_POS.getY() + (ServerConf.GOAL_WIDTH / 2 ) - 2);
        l_longCorner.setY(ENEMY_GOAL_POS.getY() - (ServerConf.GOAL_WIDTH / 2 ) + 2);
      }
      else   //standing 'over' the middle of the goal
      {
        l_longCorner.setY(ENEMY_GOAL_POS.getY() + (ServerConf.GOAL_WIDTH / 2 ) - 2);
        l_shortCorner.setY(ENEMY_GOAL_POS.getY() - (ServerConf.GOAL_WIDTH / 2 ) + 2);
      }

//now checking if the way to the goal is free
      for (ListIterator it = m_enemyTeam.listIterator(); it.hasNext();)
      {
        l_enemyPlayer = (PlayerInfo)it.next();
        if ( l_middleOfGoal!=null && Math.abs(m_myWorld.getBodyDirectionTo(l_enemyPlayer) - m_myWorld.getBodyDirectionTo(l_middleOfGoal)) < FREE_ANGLE_FOR_GOAL_KICK)
        {
          l_middleOfGoal = null;
        }
        if (l_shortCorner != null && Math.abs(m_myWorld.getBodyDirectionTo(l_enemyPlayer) - m_myWorld.getBodyDirectionTo(l_shortCorner)) < FREE_ANGLE_FOR_GOAL_KICK)
        {
          l_shortCorner = null;
        }
        if (l_longCorner!=null && Math.abs(m_myWorld.getBodyDirectionTo(l_enemyPlayer) - m_myWorld.getBodyDirectionTo(l_longCorner)) < FREE_ANGLE_FOR_GOAL_KICK)
        {
          l_longCorner = null;
        }
      }
      if (l_middleOfGoal != null)
        return l_middleOfGoal;
      if (l_shortCorner != null)
        return l_shortCorner;
      return l_longCorner;  //if also null, return null

    }
    else   //enemyList empty
    {
      if (m_myWorld.getMyDistanceTo(ENEMY_GOAL_POS) > MAX_GOAL_KICK_DISTANCE)
        return null;
      else
        return ENEMY_GOAL_POS;
    }

  }






 /* private Coordinate checkGoalKick()
  {
    double l_myAngleToGoal = 0.0;

    PlayerInfo l_players;
    boolean l_kickPossible = true;
    Collection l_yPos = new TreeSet();
    double l_bestGoalKickPos = 0.0;
    double l_lastPos = -5;    //- ServerConf.GOAL_WIDTH / 2;
    double l_greatestDist = 0.0;
    double l_greatestDistPos = 0.0;
    double l_distance = 0.0;
    double l_thisPos = 0.0;
    Coordinate l_shootOnPos  = new Coordinate(52.5,0);
    if (m_myWorld.getMyself().getDistanceTo(ENEMY_GOAL_POS) < MAX_GOAL_KICK_DISTANCE)
    {
      if (!m_enemyTeam.isEmpty())
      {
        for (ListIterator it = m_enemyTeam.listIterator(); it.hasNext();)
        {
          l_players = (PlayerInfo) it.next();
          if (l_players.getAbsPosition().getX() > 47)
            l_yPos.add(new Double(l_players.getAbsPosition().getY()));
          l_yPos.add(new Double(ServerConf.GOAL_WIDTH / 2));
          l_yPos.add(new Double(-ServerConf.GOAL_WIDTH / 2));
          for (Iterator n = l_yPos.iterator(); n.hasNext();)
          {
            l_thisPos = ((Double)n.next()).doubleValue();
            l_distance = Math.abs(l_thisPos - l_lastPos);
            l_lastPos = l_thisPos;
            if (l_distance > l_greatestDist)
            {
              l_greatestDist = l_distance;
              l_greatestDistPos = l_lastPos;
            }
          }
        }
      } //check empty lsit
      l_greatestDistPos += (l_greatestDist / 2);
      l_shootOnPos.setX(52.5);
      l_shootOnPos.setY(l_greatestDistPos/2-.5);
      l_myAngleToGoal = m_myWorld.getBodyDirectionTo(l_shootOnPos);
      if (!m_enemyTeam.isEmpty())
      {
        for (ListIterator it = m_enemyTeam.listIterator();it.hasNext();)
        {
          l_players = (PlayerInfo) it.next();
          if ((Math.abs(l_myAngleToGoal - FREE_ANGLE)  < Math.abs(m_myWorld.getBodyDirectionTo(l_players.getAbsPosition()))) && (Math.abs(l_myAngleToGoal + FREE_ANGLE) > Math.abs(m_myWorld.getBodyDirectionTo(l_players.getAbsPosition() ))))
            l_kickPossible = false;  //hope it works ;)
        }
      } //check empty list
    }
    else //Distance to great;
      l_kickPossible = false;
    if (l_kickPossible)
    {
      return l_shootOnPos;
    }
    else
      return null;
  }
*/
/**
 * Returns true if I am the nearest to the ball in the next cycles
 * @return boolean
 */
  private boolean nearestToTheBall()
  {
    final double TOLERANCE = 2; //this is the tolerance we have (so surely SOMEONE goes to the ball)
    final int TAKE_THE_BALL_SCHWELLE = 4;
    int l_cyclesToLookForward = 30;

    int l_zaehler = 0;
    double l_minDistanceToBall = 100;
    boolean l_fastestInACycle = false;
    double l_distanceToBall;
    if (m_myWorld.getMyDistanceTo(m_myWorld.getBall()) < TAKE_THE_BALL_SCHWELLE)
    {
      // if we are nearer than TAKE_THE_BALL_SCHWELLE meters we allways run to the ball
      return true;
    }
    if (!m_ownTeam.isEmpty())
    {
      if( m_myWorld.getBall().getAbsPositionOfNextRound() == null)
      {
        for (ListIterator it = m_ownTeam.listIterator(); it.hasNext();)
        {
          l_distanceToBall=((PlayerInfo)it.next()).getDistanceTo(m_myWorld.getBall().getAbsPosition());
          if (l_distanceToBall < l_minDistanceToBall)
            l_minDistanceToBall = l_distanceToBall;
        }   //playerList
        if (l_minDistanceToBall + TOLERANCE > m_myWorld.getMyDistanceTo(m_myWorld.getBall()))
          return true;
      }
      else{
        for (l_zaehler = 0; l_zaehler < l_cyclesToLookForward; l_zaehler++)
        {
          l_minDistanceToBall = 100;
          for (ListIterator it = m_ownTeam.listIterator(); it.hasNext();)
          {
            l_distanceToBall=((PlayerInfo)it.next()).getDistanceTo(getBallPosInRound(l_zaehler));
            if (l_distanceToBall < l_minDistanceToBall)
              l_minDistanceToBall = l_distanceToBall;
          }   //playerList
        }
        if (l_minDistanceToBall + TOLERANCE > m_myWorld.getMyDistanceTo(getBallPosInRound(l_zaehler)))
          return true;

      }//cycles
    }  //player list empty
    else
      return true; //if we are alone we surely are fastest to ball
    return false;
  }

  /**
   * This method checks all own players if we can pass to him, and finds the "best"
   * (the one in range (MAX_PASS_DISTANCE) and not in offside and nearest to goal, not "decked" by enemies)
   * @param f_metersBackwards How many meters we are allowed to pass backwards
   * @return a PlayerInfo if we found a passpartner, otherwise null
   */
  private PlayerInfo checkForPassPartner(int f_mustBeMetersForward)
  {
    //get a cleosan copy of our ownTeam
    //Search pass partners
    if (!m_ownTeam.isEmpty())
    {
      PlayerInfo l_passPartner=null;           // the one we pass to
      PlayerInfo l_possiblePassPartner; //this would be a possibility
      PlayerInfo l_enemyPlayer; //used later
      double l_offsideBarrier=this.getOffsideBarrier(m_enemyTeam,m_ownTeam); //get the Offside Barrier

      for (ListIterator it = m_ownTeam.listIterator(); it.hasNext();)
      {
        l_possiblePassPartner = (PlayerInfo) it.next();
        if ( (l_offsideBarrier-2>l_possiblePassPartner.getAbsPosition().getX()) //offside
             && (m_myWorld.getMyDistanceTo(l_possiblePassPartner) < MAX_PASS_DISTANCE) //be less than max pass distance
             &&  (
             /* min forward*/        (f_mustBeMetersForward>=0 && l_possiblePassPartner.getAbsPosition().getX() - m_myWorld.getMyself().getAbsPosition().getX() > f_mustBeMetersForward)
             /* max backward*/   ||  (f_mustBeMetersForward<0 && l_possiblePassPartner.getAbsPosition().getX() + m_myWorld.getMyself().getAbsPosition().getX() < f_mustBeMetersForward )
             )
             )
        {
          if (!m_enemyTeam.isEmpty())
          {
            for(ListIterator enemyIterator = m_enemyTeam.listIterator();it.hasNext();)
            {
              l_enemyPlayer=(PlayerInfo)it.next();
              if (Math.abs(m_myWorld.getBodyDirectionTo(l_possiblePassPartner)-m_myWorld.getBodyDirectionTo(l_enemyPlayer)) > FREE_ANGLE
                  || (Math.abs(m_myWorld.getBodyDirectionTo(l_possiblePassPartner)-m_myWorld.getBodyDirectionTo(l_enemyPlayer)) < FREE_ANGLE
                  && m_myWorld.getMyDistanceTo(l_possiblePassPartner) +2 < m_myWorld.getMyDistanceTo(l_enemyPlayer)
                  )
                  )
              {
                continue;
              }
              else
              {
                l_possiblePassPartner=null;
                break;
              }
            }
          }
        }
        else //this own player was not good (too far away or too far backwards, or offside), so we dont want him
        {
          l_possiblePassPartner=null;
        }
        if(l_possiblePassPartner!=null)//check if this is the best one
        {
          if(l_passPartner==null) // we dont have one at the moment so store this guy
            l_passPartner=l_possiblePassPartner;
          else //chose the one nearest to goal
            if (l_passPartner.getAbsPosition().getX()<l_possiblePassPartner.getAbsPosition().getX())
              l_passPartner=l_possiblePassPartner;
        }
      }//for (all in ownTeam) loop
      if (l_passPartner!=null)
        logger.debug("Found Pass Partner @:"+l_passPartner.getAbsPosition()+ "#"+l_passPartner.getBackNumber());
      else
        logger.info("No Pass Partner found :(");
      return l_passPartner;
    }
    else //own team is empty
    {
      return null;
    }
  }

  /**
   * This method computer the offside barrier (it skips the goalie(the player farest))
   * @param f_enemyTeam the team we compute the barrier from
   * @param f_ownTeam
   * @return 100 if team is empty, otherwise the x-value of the guy standing at the last position
   */
  private double getOffsideBarrier(ArrayList f_enemyTeam, ArrayList f_ownTeam)
  {
    double l_offsideBarrier=-0;
    double l_goalie=0;
    double l_currentValue;
    double l_ownPlayerMaxX=0;
    if(!f_ownTeam.isEmpty())
    {
      for(ListIterator it=f_ownTeam.listIterator();it.hasNext();)
      {
        l_currentValue=((PlayerInfo)it.next()).getAbsPosition().getX();
        if(l_currentValue>l_ownPlayerMaxX)
          l_ownPlayerMaxX=l_currentValue;
      }
    }
    if (!f_enemyTeam.isEmpty())
      for(ListIterator it=f_enemyTeam.listIterator();it.hasNext();)
      {
        l_currentValue=((PlayerInfo)it.next()).getAbsPosition().getX();
        if(l_currentValue>l_goalie)
          l_goalie=l_currentValue;
        else if (l_currentValue>l_offsideBarrier)
          l_offsideBarrier=l_currentValue;
      }
      return Math.max(l_offsideBarrier,l_ownPlayerMaxX);
  }

  /**
   * This method checks if we have enough place to dribble
   * @return true if there is enough place, false if not
   */
  private boolean checkDribble()
  {
    if (!m_enemyTeam.isEmpty())
    {
      if (m_myWorld.isPlayerBehind()) //theres a player behind me...so dont dribble
        return false;
      for (ListIterator it = m_enemyTeam.listIterator(); it.hasNext();)
      {
        if (m_myWorld.getMyDistanceTo((PlayerInfo)it.next()) < MIN_DISTANCE_FOR_DRIBBEL)
        {
        return false;
      }
      }
    }
    return true;
  }

  /**
   * Upset a given Enemy
   * @param f_player
   */
  private void upsetEnemy(PlayerInfo f_player)
  {
    if (m_myWorld.getBall().isBallKickableBy(f_player))
    {
      gotoEnemy(f_player);
    }
    else
    {
      if ((m_myWorld.getMyself().getAbsPosition()).getX() > 0)
        goBetweenBallGoalAndWatchIt(f_player.getDistanceTo(MY_GOAL_POS) * 0.85);
      else
        goBetweenBallAndEnemyAndWatchBall(f_player);
    }
  }

  /**
   * Returns an enemy if it is in range
   * @return PlayerInfo
   */
  private PlayerInfo enemyInRange()
  {
    double l_minDistanceToEnemy = 100;
    PlayerInfo l_nearestEnemyPlayer = new PlayerInfo();
    PlayerInfo l_enemyPlayer;
    if (!m_enemyTeam.isEmpty())
    {
      for (ListIterator it = m_enemyTeam.listIterator(); it.hasNext();)
      {
        l_enemyPlayer = (PlayerInfo) it.next();
        if (l_enemyPlayer.getDistanceTo(m_myWorld.getMyself().getAbsPosition()) < l_minDistanceToEnemy)
        {
          l_minDistanceToEnemy = l_enemyPlayer.getDistanceTo(m_myWorld.getMyself().getAbsPosition());
          l_nearestEnemyPlayer = l_enemyPlayer;
        }
      }
    }
    if (l_minDistanceToEnemy < HOMERANGE_R)
      return l_nearestEnemyPlayer;
    else
      return null;
  }


  /**
   * Returns true if I am at home
   * @return boolean
   */
  private boolean atHomepos()
  {
    //if (m_myWorld.getMyself().getDistanceTo(HOMEPOS[m_myState.getState()][m_myWorld.getMyself().getBackNumber()]) > HOMERANGE_R)
    if (m_myWorld.getMyself().getDistanceTo(HOMEPOS[m_myWorld.getMyself().getBackNumber()]) > HOMERANGE_R)
      return false;
    else
      return true;   //at homepos
  }

  /**
   * Passto  to a given player in a given cycle
   * @param f_player
   * @param f_cycle
   */
  private void passTo(PlayerInfo f_player, int f_cycle)
  {
    double l_kickpower = Math.min(100, (m_myWorld.getMyDistanceTo(f_player) * 4.5));
    m_actions.kickTo(l_kickpower, f_player, f_cycle);
  }


  /**
   * Dribble in a given cycle to the goal
   * @param f_cycle
   */
  private void dribble(int f_cycle)
  {
    double l_power;
    Coordinate l_dribbleToPos;
    l_power = MAXPOWER;
    l_dribbleToPos = ENEMY_GOAL_POS;
    m_actions.dribbelTo(l_power, l_dribbleToPos, f_cycle);
  }


  /**
   * This method calculates at which position our Ball is after f_cycles
   * @param f_cycles defines how far we should look into the future (e.g. if f_cycles==1 we take AbsPosOfNextRound
   * @return Coordinate of future position
   */
  private Coordinate getBallPosInRound(int f_cycles)
  {
    Coordinate l_deltaBall;
    Coordinate l_finalPos;
    if (f_cycles !=0 && m_myWorld.getBall().getAbsPositionOfNextRound() != null)
    {
      l_deltaBall = m_myWorld.getBall().getAbsPositionOfNextRound().sub(m_myWorld.getBall().getAbsPosition());
      l_finalPos = l_deltaBall.scaleWith(f_cycles*Math.pow(ServerConf.BALL_DECAY,f_cycles-1)); // Verlngern
      l_finalPos=m_myWorld.getBall().getAbsPosition().add(l_finalPos);
      return l_finalPos;
    }
    else
      return m_myWorld.getBall().getAbsPosition();
  }

  /**
   * With this method the KI is started at the beginning
   *
   */
  public void startKI()
  {
    m_KIThread=this;
    m_KIThread.start();
    m_actions.startActionPerformer();
  }


  /**
   * this method stops our Thread
   */
  public synchronized void endTread()
  {
    m_KIThread = null; //get a clean outcome
    m_actions.stopActionPerformer();
    notify();
  }


  /**
   * this method says our thread that there's something to do
   */
  public synchronized void onNewData()
  {
    m_threadSuspended=false; // ok, he can run again if he wants to
    notify();
  }


  /**
   * Returns the cyces in future when the ball is kickable if it is visible at all
   * @return int
   */
  private int isBallKickableByMe()
  {
    if (m_myWorld.getMyDistanceTo(m_myWorld.getBall())< ServerConf.KICKABLE_MARGIN + (ServerConf.PLAYER_SIZE / 2 ) + (ServerConf.BALL_SIZE / 2))
    {
      return 0;
    }
    else if (m_myWorld.getBall().getAbsPositionOfNextRound()!=null && m_myWorld.getMyDistanceTo( m_myWorld.getBall().getAbsPositionOfNextRound()) < ServerConf.KICKABLE_MARGIN + (ServerConf.PLAYER_SIZE / 2 ) + (ServerConf.BALL_SIZE / 2))
    {
      return 1;
    }
    return -1;
  }

  /**
   * This method manages the viewMode of our Players.
   * If the ball IS NOT visible it turns to normal mode, if it is not visible for a longer time, it turns to WIDE mode
   * If the ball IS visible and distance is < 40 it turns to WIDE, else NORMAL , else to NARROW
   */
  private void manageViewMode()
  {

    if(!m_myWorld.getBall().isVisible())
    {
      m_myState.m_searchingBall=true;
      if (m_myWorld.sorry_whatTimeDoWeHave()-m_myWorld.getBall().getTime()> 6)
      {
        if (m_myState.m_currentViewWidth!=SenseBodyInfo.WIDE)
        {
          m_actions.changeView(SenseBodyInfo.WIDE,true);
          m_myState.m_currentViewWidth=SenseBodyInfo.WIDE;
        }
      }
      else
      {
        if (m_myState.m_currentViewWidth!=SenseBodyInfo.NORMAL)
        {
          m_actions.changeView(SenseBodyInfo.NORMAL,true);
          m_myState.m_currentViewWidth=SenseBodyInfo.NORMAL;
        }
      }
    }
    else //ball is visible
    {
      m_myState.m_searchingBall=false;
     /* if (m_myWorld.getMyDistanceTo(m_myWorld.getBall())< 4 && Math.abs(m_myWorld.getBall().getDirection())<StateManager.VIEW_ANGLES[SenseBodyInfo.NARROW]/2)
      {
        if (m_myState.m_currentViewWidth!=SenseBodyInfo.NARROW)
          m_actions.changeView(SenseBodyInfo.NARROW,true);
        m_myState.m_currentViewWidth=SenseBodyInfo.NARROW;

      }

      else */if (m_myWorld.getMyDistanceTo(m_myWorld.getBall())< 35 && Math.abs(m_myWorld.getBall().getDirection())<StateManager.VIEW_ANGLES[SenseBodyInfo.NORMAL]/2)
      {
        if (m_myState.m_currentViewWidth !=SenseBodyInfo.NORMAL)
        {
          m_actions.changeView(SenseBodyInfo.NORMAL,true);
          m_myState.m_currentViewWidth=SenseBodyInfo.NORMAL;
        }
      }
      else
      {
        if (m_myState.m_currentViewWidth !=SenseBodyInfo.WIDE)
        {
          m_actions.changeView(SenseBodyInfo.WIDE,true);
          m_myState.m_currentViewWidth=SenseBodyInfo.WIDE;
        }
      }
    }
  }

  private void kickToCleverPosition(int f_cycles)
  {
    PlayerInfo l_enemyPlayer = null;
    if(f_cycles==-1)
      f_cycles=m_myWorld.getViewModeAsInt();
    if (m_myWorld.getMyself().getAbsPosition().getX() > 10 || m_enemyTeam.isEmpty())
    {
      if (Math.abs(m_myWorld.getMyself().getAbsPosition().getY()) < 20)
        m_actions.kickTo(MAXPOWER, ENEMY_GOAL_POS, f_cycles);
      else
        m_actions.kickTo(NORMPOWER+15, new Coordinate(45, 0), f_cycles);
      return;
    }
    for (ListIterator it = m_enemyTeam.listIterator(); it.hasNext();)
    {
      l_enemyPlayer = (PlayerInfo)it.next();
      if (m_myWorld.getMyDistanceTo(l_enemyPlayer) < MIN_DISTANCE_FOR_DRIBBEL)
      {
        if (l_enemyPlayer.getDirection() < 0)
        {
          if (m_myWorld.getMyself().getAbsPosition().getX() > -30)
            m_actions.kickTo(MAXPOWER-20, new Coordinate(30, 15), f_cycles);
          else
            m_actions.kickTo(MAXPOWER-20, new Coordinate(0, 20), f_cycles);
        }
        else
        {
          if (m_myWorld.getMyself().getAbsPosition().getX() > -30)
            m_actions.kickTo(MAXPOWER-20, new Coordinate(30, -15), f_cycles);
          else
            m_actions.kickTo(MAXPOWER-20, new Coordinate(0, -20), f_cycles);
        }
        return;
      }
    }
    m_actions.kickTo(MAXPOWER, ENEMY_GOAL_POS, f_cycles);
  }

  private void goBetweenBallAndEnemyAndWatchBall(PlayerInfo f_player)
  {
    Coordinate l_playerPos=(m_myWorld.getBall().getAbsPositionOfNextRound() != null) ? m_myWorld.getBall().getAbsPositionOfNextRound() : m_myWorld.getBall().getAbsPosition();
    Coordinate l_coord=l_playerPos.sub((f_player.getAbsPositionOfNextRound()!=null)?f_player.getAbsPositionOfNextRound():f_player.getAbsPosition());
    //now we've the vector from the guy to the ball, so move onto that, half a meter away from the player
    l_coord=l_playerPos.add(l_coord.normalize());
    if (m_myWorld.getMyDistanceTo(l_coord)<2)
      watchBall();
    else
      m_actions.runTo(NORMPOWER,l_coord,m_myWorld.getViewModeAsInt());
  }
}