package jais.utils;

import java.util.Hashtable;

/**
 * <p>Title: CommandLineParser</p>
 * <p>Description: This should easy the use of command line arguments</p>
 * <p> This class is currently only for use in JAIS, because of its nonnatural syntax
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: JAIS</p>
 * @author Stephan Diederich (based on a code snippet from sourceforge from Joel
 * Fouse)
 */
public abstract class CommandLineParser
{

   /**
    * This Parse gets the String field from the commandline and works out a hashtable
    * @param f_args
    * @return Hashtable with options as keys and values as entries ()
    * @roseuid 3CE6899402A0
    */
   public static Hashtable Parse(String[] f_args)
   {
    return Parse(f_args,new Hashtable());
   }

   /**
    * This Parse gets the String field from the commandline and works out a hashtable
    * A Hashtable can be given to interpret short values and longvalues,
    * e.g. "v" and "visualization"
    * @param f_args
    * @param f_argFilter
    * @return Hashtable with options as keys and their values as entries ()
    * @roseuid 3CE6899402BE
    */
   public static Hashtable Parse(String[] f_args, java.util.Hashtable f_argFilter)
   {
    Hashtable l_params = new Hashtable();
    String[] l_rejects = new String[f_args.length];
    for (int i = 0; i < f_args.length; i++) {
       // f_args[i] starts with -- and has a value, we store it in l_params with long version if it exists
       if (f_args[i].startsWith("--") && (f_args[i].indexOf("=") > 2))
       {
          String[] l_argtmp = { f_args[i].substring(2,f_args[i].indexOf("=")),f_args[i].substring(f_args[i].indexOf("=")+1)};
          l_params.put(Filter(l_argtmp[0],f_argFilter),l_argtmp[1]);
       }
       else
         if (f_args[i].startsWith("-")) //hm should be clear
         {
           if ((f_args.length > i + 1) && !(f_args[i+1].startsWith("-"))) //something like "-player 11"
           {
              l_params.put(Filter(f_args[i].substring(1),f_argFilter), f_args[++i]);
           }
           else
           { // we think this is a flagset, e.g. -xvf
             f_args[i] = f_args[i].substring(1); //kick the first "-" away
             //for every variable in the flagset look if we have a long version and store
             //it in the Hashtable with value TRUE
             for (int j = 0; j < f_args[i].length(); j++) {
               l_params.put(Filter(f_args[i].substring(j,j+1),f_argFilter), new Boolean("true"));
           }
          }
        }
        else // f_args[i] doesnt start with - or --
        {
          switch(i)
          {
            case 0:
              l_params.put("Teamname",f_args[0]);
              break;
            case 1:
              l_params.put("Servername",f_args[1]);
              break;
            case 2:
              l_params.put("Serverport",f_args[2]);
              break;
            default:
              l_params.put(f_args[i],"-1");
          }//switch
        }
      }//for (..)
      return ParseAgainForJais(l_params);
   }

   /**
    * Filter
    * This function checks if f_arg is a key in f_argFilter, if so it returns its
    * value,else f_arg
    * @param f_arg
    * @param f_argFilter
    * @return f_arg if f_arg doesnt exists as key in f_argFilter, else its value in
    * f_argFilter
    * @roseuid 3CE6899402F0
    */
   private static String Filter(String f_arg, java.util.Hashtable f_argFilter)
   {
    return (f_argFilter.containsKey(f_arg) ? (String)f_argFilter.get(f_arg) : f_arg);
   }

   /**
    * This method checks the Hashtable for specific values for Jais
    * and sets some default values for it
    * @param f_params
    * @return Hashtable with paramters for Jais or JaisTeam, or null
    */
    public static Hashtable ParseAgainForJais(Hashtable f_params)
    {
        try
        {

            if (!(f_params.get("help")==null)) //user needs help ?
              if (((Boolean)f_params.get("help")).booleanValue())
                return null;
            if(f_params.get("Teamname")==null)
            {
              f_params.remove("Teamname");
              f_params.put("Teamname","Jais");
            }
            if(f_params.get("Servername")==null)
            {
              f_params.remove("Servername");
              f_params.put("Servername","localhost");
            }
            if(f_params.get("Serverport")==null || (Integer.parseInt((String)f_params.get("Serverport")) < 0))
            {
              f_params.remove("Serverport");
              f_params.put("Serverport",new String("6000"));
            }
            if (f_params.get("numberOfPlayers")==null || (Integer.parseInt((String)f_params.get("numberOfPlayers")) > 11) || (Integer.parseInt((String)f_params.get("numberOfPlayers")) < 1))
            {
                f_params.remove("numberOfPlayer");
                f_params.put("numberOfPlayers",new String("11"));
            }
            if (f_params.get("loglevel")==null || ((Integer.parseInt((String)f_params.get("loglevel"))) > 6) || (((Integer.parseInt((String)f_params.get("loglevel")))) < 0))
            {
                f_params.remove("loglevel");
                f_params.put("loglevel",new String("WARN"));
            }
            else
            {
                int l_loglevel=Integer.parseInt((String)f_params.get("loglevel"));
                f_params.remove("loglevel");
                switch (l_loglevel)
                {
                    case 0:
                        f_params.put("loglevel","ALL");
                        break;
                    case 1:
                        f_params.put("loglevel","DEBUG");
                        break;
                    case 2:
                        f_params.put("loglevel","INFO");
                        break;
                    case 3:
                        f_params.put("loglevel","WARN");
                        break;
                    case 4:
                        f_params.put("loglevel","ERROR");
                        break;
                    case 5:
                        f_params.put("loglevel","FATAL");
                        break;
                    case 6:
                        f_params.put("loglevel","OFF");
                        break;
                }
            }
            if (f_params.get("g")==null)
            {
                f_params.remove("g");
                f_params.put("g",new Boolean ("false"));
            }
        } //end of try block
        catch (java.lang.Exception ex)
        {
            return null;
        }
        return f_params;
    }
}
