package jais.utils;


/**
 * <p>Title: Class Timer</p>
 * <p>Description: This Timer Class provides a easy to use timer, e.g. to get the
 * time your method is working
 * on somewhat</p>
 * <p>  </p>
 * <p>   USAGE:</p>
 * <p>     Timer blub=new Timer();</p>
 * <p>     blub.Start();</p>
 * <p>      //your code</p>
 * <p>     blub.Stop();</p>
 * <p>     blub.GetTime(); //returns the time in milliseconds since Start();</p>
 * <p>     blub.Start(); //resets the Timer</p>
 * <p>      //code</p>
 * <p>     blub.Pause();</p>
 * <p>      //do something really uninteresting;</p>
 * <p>     blub.Resume();</p>
 * <p>      //do something</p>
 * <p>     blub.Stop();</p>
 * <p>     blub.GetTime(); // return the time from (Start() to Pause() + Resume()
 * to Stop())</p>
 * <p>  </p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p>Company: JAIS </p>
 * @author Stephan Diederich
 */
public class Timer
{
   private boolean m_running = false;
   private boolean m_paused = false;
   private long m_starting_time = 0;
   private long m_actual_time = 0;
   private long m_pause_time = 0;

   /**
    * Constructor nothing to do here yet
    * @roseuid 3CC97C8F0019
    */
   public Timer()
   {

   }

   /**
    * (re)starts the timer to count from this moment up
    * @roseuid 3CC97C8E03D9
    */
   public void Start()
   {
    m_running = true;
    m_starting_time=System.currentTimeMillis(); //save when we started
   }

   /**
    * stops the timer
    * @roseuid 3CC97C8E03E2
    */
   public void Stop()
   {
    m_running=false;
    m_actual_time=System.currentTimeMillis(); //save when we stopped
   }

   /**
    * return the time, that went up since Start()
    * @return time which went up since Start() or Resume()
    * @roseuid 3CC97C8F0004
    */
   public long GetTime()
   {
    if (m_paused || !m_running) //check if were have a pause or are not running
      return (m_actual_time-m_starting_time);
    else
      return (System.currentTimeMillis()-m_starting_time);
   }

   /**
    * Pause the timer which was formerly m_started with Start()
    * @return false, if the Timer was never m_started before
    * @roseuid 3CC97C8F0005
    */
   public boolean Pause()
   {
    if(!m_running)
      return false;
    else
    {
      m_paused=true;
      m_actual_time=System.currentTimeMillis(); // save the time to output
      return true;
    }
   }

   /**
    * Resume the timer which was formerly m_paused with Pause()
    * @return false, if the Timer was never m_paused before
    * @roseuid 3CC97C8F000F
    */
   public boolean Resume()
   {
    if (!m_paused)
      return false;
    else
    {
      m_pause_time=System.currentTimeMillis()-m_actual_time; // how long was the pause ?
      System.out.println("pause war:"+m_pause_time);
      m_starting_time += m_pause_time; //correct the starting time, so that the pause wont be seen;
      m_paused=false;
      return true;
    }
   }

   /**
    * Checkout if the Timer is currently m_running
    * @return true, if Timer is m_running
    * @roseuid 3CC97C8F0010
    */
   public boolean IsRunning()
   {
    return m_running;
   }

   /**
    * overwrites the toString method to get a nice output (same as GetTime() ) if
    * used in strings
    * @return String version of GetTime() with "ms"
    * @roseuid 3CE6899803E6
    */
   public String toString()
   {
    return (GetTime() + " ms");
   }
}
