package jais.visualization;

import javax.swing.*;
import java.util.*;
import java.awt.*;
import java.awt.event.*;

import org.apache.log4j.*;

import jais.world.WorldModel;
import jais.objects.*;


/**
 *
 * <p>Title: PalyFieldPanel class</p>
 * <p>Description: Instantiating this class creates a new JPanel in which can be drawn.
 * Methods of this class fetch (from world) and draw objects on playfield.
 * All calculations are only done by proportion calculations between *.jpg
 * and real monitored playfield.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p> </p>
 * @author Raul Zanzinger and Sven Scheffelmeier
 *
 */
class PlayFieldPanel extends JPanel
{
    //object wide variable
    private Image m_playFieldImage;            //images which will be loaded
    private int m_playFieldDetermedMiddleX;
    private int m_playFieldDetermedMiddleY;
    private final double PLAYFIELD_SIZE_WORLD_X=127;   //on world orientated x size of playfield
    private final double PLAYFIELD_SIZE_WORLD_Y=90;    //on world orientated y size of playfield
    private int m_myselfXPos;                          //my new fetched x pos. on playfield image.
    private int m_myselfYPos;                          //my new fetched y pos. on playfield image
    private int m_myselfNewXPos;
    private int m_myselfNewYPos;
    private double m_myselfBodyAngle;

    private Graphics m_graphics;              //for drawing objects on it

    //object wide constants
    private WorldModel m_myWorld;                  //link on world
    private final int OVALDIAMETER=10;             //the diameter of the displayed players
    private final int VIRTUALSQUAREWIDTH=
                            (int)((OVALDIAMETER)/Math.sqrt(2d));       //the needed value for the 'visir'
                                                                        //of myself
    private final int LENGTHOFBODYDIRECTIONLINE=4;   //length of body direction line in pixel
    private final int LENGTHOFNECKDIRECTIONLINE=1;   //lengths of neck direction lines


    //package (e.g. for legend panel) wide constant colors for objects
    protected static final Color COLOR_FRIENDLY_PLAYER_VISIBLE   = Color.red;
    protected static final Color COLOR_FRIENDLY_PLAYER_INVISIBLE = Color.orange;
    protected static final Color COLOR_ENEMY_PLAYER_VISIBLE      = Color.lightGray;
    protected static final Color COLOR_ENEMY_PLAYER_INVISIBLE    = Color.gray;
    protected static final Color COLOR_BALL_VISIBLE              = Color.cyan;
    protected static final Color COLOR_BALL_INVISIBLE            = Color.yellow;
    protected static final Color COLOR_FLAG_VISIBLE              = Color.pink;
    protected static final Color COLOR_MYSELF                    = Color.blue;
    protected static final Color COLOR_SEEN_GOAL                 = Color.red;
    protected static final Color COLOR_SEEN_LINE                 = Color.pink;

    static Logger logger=Logger.getLogger("jais.visualization");


    /**
     * This is the constructor of the class PlayFieldPanel:
     * accepts model from view
     *
     * @param f_world s
     */
    public PlayFieldPanel(WorldModel f_world)
    {
        //do not paint the playfield background
        setOpaque(true);

        m_myWorld=f_world;
        //this.setPreferredSize(new Dimension(508 , 360));

        //some playfield proportion calculations

        //determ playfield middle (for soccer server '(0,0)')
        m_playFieldDetermedMiddleX=(int)(this.getSize().getWidth()/2);
        m_playFieldDetermedMiddleY=(int)(this.getSize().getWidth()/2);

        //*.jpg playfield proportion is 127/90 (=1,411)

        //load image with playfield
        loadPlayFieldImage();
    }


    /**
     * This method is called whenever the playfiled is
     * redrawn (e.g. on repaint by OS or Jais)
     *
     * @param f_g s
     */
    public void paintComponent(Graphics f_g)
    {
        f_g.drawImage(m_playFieldImage,0,0,(int)this.getSize().getWidth(),(int)this.getSize().getHeight(),this);
        //f_g.setColor(Color.lightGray);
        //f_g.fillRect(0,0,(int)(this.getSize().getWidth()),(int)(this.getSize().getHeight()));
        this.m_graphics = f_g;
        try
        {
            getNewDataFromWorld();
        }
        catch (Exception ex)
        {
            logger.debug("Caught exception in getNewDataFromWorld"+ex.getMessage()+"...running on");
            //if(logger.getLevel().isGreaterOrEqual(Level.toPriority(Level.WARN_INT)))
            //    ex.printStackTrace();
        }
    }


    /**
     * This method loads the playfield image from a file:
     * here ext directory of jais
     * The actual image's size is x:960 * y:680 pixel large
     * (side relation=>1,41)
     */
    private void loadPlayFieldImage()
    {
        m_playFieldImage = getToolkit().getImage("../ext/PlayField.gif");
        MediaTracker l_imageWatcher = new MediaTracker(this);
        //fetch image from file and set it's ID on 0
        l_imageWatcher.addImage(m_playFieldImage, 0);
        try
        {
              //wait till the play field image is completely loaded
              logger.info("Waiting for image been loaded!");
              l_imageWatcher.waitForAll();
              logger.info("Play field image has been loaded!");
         }
         catch (InterruptedException e)
         {
               logger.info("Error loading image from file!");
         }
    }


    /**
     * This method gets new data from world; called by visu.
     *
     */
    public void getNewDataFromWorld()
    {
        logger.info("Starting reading infos from world!");

        //here fall every cloned list in
        ArrayList l_localArrayList;
        LinkedList l_localLinkedList;

        //for checking later the attributes of
        //the following objekts
        PlayerInfo l_playerInfo;
        BallInfo   l_ballInfo;
        FlagInfo   l_flagInfo;
        GoalInfo   l_goalInfo;
        LineInfo   l_lineInfo;

        //determ play direction

        //fetch the list of friendly players
        //and search for objects; check first if empty, then clone
        //logger.info("Checking if friendly list is empty!");
        //make a copy of friend. player list from world
        l_localArrayList = (ArrayList)(m_myWorld.getFriendlyPlayers().clone());
        if (!l_localArrayList.isEmpty())
        {
            logger.info("Copied friendly list!");
            for (ListIterator it = l_localArrayList.listIterator(); it.hasNext();)
            {
                //convert to needed player info stuff
                l_playerInfo = (PlayerInfo) it.next();
                drawSeenFriendlyPlayers(determNewXPosition((int)l_playerInfo.getAbsPosition().getX()),
                                            determNewYPosition((int)l_playerInfo.getAbsPosition().getY()),
                                            l_playerInfo.isVisible()
                                        );
            }
        }
        logger.info("Read friendly player info!");

        //for consistence:
        //the next two block ask for seen and used flags in the world!

        //fetch the list of enemy players
        //and search for objects; check first if empty, then clone
        logger.info("Checking if enemy list is empty!");
        //make a copy of enemy. player list from world
        l_localArrayList = (ArrayList)(m_myWorld.getEnemyPlayers().clone());
        if (!l_localArrayList.isEmpty())
        {
            logger.info("Copied enemy list!");
	        for (ListIterator it = l_localArrayList.listIterator(); it.hasNext();)
            {
                //convert to needed player info stuff
                l_playerInfo = (PlayerInfo) it.next();
                drawSeenEnemyPlayers(determNewXPosition((int)l_playerInfo.getAbsPosition().getX()),
                                            determNewYPosition((int)l_playerInfo.getAbsPosition().getY()),
                                            l_playerInfo.isVisible()
                                        );
            }
        }
        logger.info("Read enemy player info!");


        //fetch the x/y-pos. of myself and draw

        //object wide myXPos from world
        m_myselfXPos = (int)(m_myWorld.getMyself().getAbsPosition().getX());
        m_myselfNewXPos = (int)determNewXPosition(m_myselfXPos);
        //object wide myYPos from world
        m_myselfYPos = (int)(m_myWorld.getMyself().getAbsPosition().getY());
        m_myselfNewYPos = (int)determNewYPosition(m_myselfYPos);
        //draw myself
        drawMyself(m_myselfNewXPos,m_myselfNewYPos);
        logger.info("Read/Drew myself info!");


        //fetch the list of currently seen flags
        //and search for objects; check first if empty, then clone
        logger.info("Checking if currently seen flag list is emtpy!");
        l_localLinkedList = (LinkedList)(m_myWorld.getCurrentylSeenFlags().clone());
        if (!l_localLinkedList.isEmpty())
        {
            logger.info("Copied curently seen flag list!");
    	    for (ListIterator it = l_localLinkedList.listIterator(); it.hasNext();)
            {
                //convert to needed flag info stuff
                boolean l_isSeenFlag=true;
                l_flagInfo = (FlagInfo) it.next();
                Coordinate l_coord=m_myWorld.getMyself().getAbsPosition();
                drawFlag((int)determNewXPosition(l_flagInfo.calculateAbsPosition(l_coord,m_myWorld.getAbsViewAngle()).getX()),
                         (int)determNewYPosition(l_flagInfo.calculateAbsPosition(l_coord,m_myWorld.getAbsViewAngle()).getY()),
                         l_isSeenFlag
                        );
            }
        }
        logger.info("Read curently seen flag info!");


        //fetch the list of currently used flags
        //and search for objects; check first if empty, then clone
        logger.info("Checking if currently used flag list is emtpy!");
        l_localLinkedList = (LinkedList)(m_myWorld.getCurrentylUsedFlags().clone());
        if (!l_localLinkedList.isEmpty())
        {
            logger.info("Copied currently used flag list!");
	        for (ListIterator it = l_localLinkedList.listIterator(); it.hasNext();)
            {
                //convert to needed flag info stuff for drawing
                boolean l_isSeenFlag=false;
                l_flagInfo = (FlagInfo) it.next();
                Coordinate l_coord=m_myWorld.getMyself().getAbsPosition();
                drawFlag((int)determNewXPosition(l_flagInfo.calculateAbsPosition(l_coord,m_myWorld.getAbsViewAngle()).getX()),
                                (int)determNewYPosition(l_flagInfo.calculateAbsPosition(l_coord,m_myWorld.getAbsViewAngle()).getY()),
                                l_isSeenFlag
                                );
            }
        }
        logger.info("Read curently used flag info!");


        //fetch the list of currently seen goals
        //and search for objects
        logger.info("Checking if currently seen goal list is empty!");
        //make a copy of currently seen goals list from world
        l_localLinkedList = (LinkedList)(m_myWorld.getCurrentylSeenGoals().clone());
        if (!l_localLinkedList.isEmpty())
        {
            logger.info("Copied currently seen goal list!");
    	    for (ListIterator it = l_localLinkedList.listIterator(); it.hasNext();)
            {
                //convert to needed player info stuff
                boolean l_isRightGoal=true; //false left, true right
                l_goalInfo = (GoalInfo) it.next();
                //from world int 1 -> right goal,
                //from world int 0 -> left goal
                if(l_goalInfo.getGoalType()==0)l_isRightGoal=false;
                drawSeenGoal(l_isRightGoal);
            }
        }
        logger.info("Read/Drew currently seen goal info!");


        //fetch the list of currently seen lines
        //and search for objects
        logger.info("Checking if currently seen line list is empty!");
        //make a copy of currently seen line list from world
        l_localLinkedList = (LinkedList)(m_myWorld.getCurrentylSeenLines().clone());
        if (!l_localLinkedList.isEmpty())
        {
            logger.info("Copied currently seen line list!");
    	    for (ListIterator it = l_localLinkedList.listIterator(); it.hasNext();)
            {
                //convert to needed line info stuff
                l_lineInfo = (LineInfo) it.next();
                //line type: 0 left, 1 right, 2 top and 3 bottom
                drawLine(l_lineInfo.getLineType(),true);
            }
        }
        logger.info("Read/Drew currently seen line info!");


        //fetch the list of currently used lines
        //and search for objects
        logger.info("Checking if currently used line list is empty!");
        //make a copy of currently used line list from world
        l_localLinkedList = (LinkedList)(m_myWorld.getCurrentylUsedLines().clone());
        if (!l_localLinkedList.isEmpty())
        {
            logger.info("Copied currently used line list!");
    	    for (ListIterator it = l_localLinkedList.listIterator(); it.hasNext();)
            {
                //convert to needed line info stuff
                l_lineInfo = (LineInfo) it.next();
                //line type: 0 left, 1 right, 2 top and 3 bottom
                drawLine(l_lineInfo.getLineType(), false);
            }
        }
        logger.info("Read/Drew currently used line info!");


        //fetch body direction angel of myself and draw.
        //determ first the x and y pos. of the second point of
        //body direction line
        drawMyselfBodyDirection(determNewXPosition(determSecondLinePointOfMyselfBodyDirectionX(m_myWorld.getAbsBodyDir()))
                                  ,determNewYPosition(determSecondLinePointOfMyselfBodyDirectionY(m_myWorld.getAbsBodyDir()))
                                );
        logger.info("Read/Drew body direction info!");


        //fetch body direction angel of myself and draw.
        //determ first the x and y pos. of the second point of
        //body direction line
        m_myselfBodyAngle = m_myWorld.getAbsBodyDir();
        int l_myselfBodyDirX=determNewXPosition(determSecondLinePointOfMyselfBodyDirectionX(m_myselfBodyAngle));
        int l_myselfBodyDirY=determNewYPosition(determSecondLinePointOfMyselfBodyDirectionY(m_myselfBodyAngle));
        drawMyselfBodyDirection(l_myselfBodyDirX,l_myselfBodyDirY);
        logger.info("Read/Drew body direction info!");


        // fetch the ball's x/y-pos. and draw
        drawSeenBall((int)determNewXPosition((m_myWorld.getBall().getAbsPosition().getX())),
                        (int)determNewYPosition((m_myWorld.getBall().getAbsPosition().getY())),
                         m_myWorld.getBall().isVisible()
                     );

        //get for test cases the pos of ball in next round
        if(m_myWorld.getBall().getAbsPositionOfNextRound()!=null)
        {
            drawnBallPosInNextRound((int)determNewXPosition((Math.round(m_myWorld.getBall().getAbsPositionOfNextRound().getX()))),
                                        (int)determNewYPosition((Math.round(m_myWorld.getBall().getAbsPositionOfNextRound().getY())))
                                    );
        }


        //fetches neck direction of myself, modifys(calc.) and draws.
        //fetch start angle of visor from world
        int l_startAngle = -1*(int)m_myWorld.getAbsViewAngle();
        l_startAngle-=SenseBodyInfo.getAngleFromViewMode(m_myWorld.getViewMode())/2;
        int l_visorSize = SenseBodyInfo.getAngleFromViewMode(m_myWorld.getViewMode());
        //determ the start angle of the visor and drawn the visor
        drawMyselfNeckDirection(l_startAngle,l_visorSize);
    }


    /**
     * This method draws the pos. of the ball in next round
     *
     * @param f_nextPosX s
     * @param f_nextPosY s
     */
    private void drawnBallPosInNextRound(int f_nextPosX, int f_nextPosY)
    {
        int l_ballDiameterInNextRound = 8;

        //set color of ball in next round
        m_graphics.setColor(Color.gray);
        //draw ball in next round
        m_graphics.fillOval(f_nextPosX-4,f_nextPosY,l_ballDiameterInNextRound-4,l_ballDiameterInNextRound);
    }


    /**
     *  This method draws the body direction of myself
     *
     * @param f_secondNewPointBodyDirectionX
     * @param f_secondNewPointBodyDirectionY
     */
    private void drawMyselfBodyDirection(int f_secondNewPointBodyDirectionX,
                                            int f_secondNewPointBodyDirectionY)
    {
        logger.info("Drawing body direction!");
        //set color of symbolic body direction
        m_graphics.setColor(Color.black);
        //draw line of body direct.
        m_graphics.drawLine(m_myselfNewXPos,
                                m_myselfNewYPos,
                                f_secondNewPointBodyDirectionX,
                                f_secondNewPointBodyDirectionY
                            );
    }


    /**
     * This method draws the symbolic neck direction
     *
     * @param f_neckDirectionX
     * @param f_neckDirectionY
     */

    private void drawMyselfNeckDirection(int f_startAngle,int f_visorSize)
    {
        logger.debug("Drawing neck direction!");
        //set color of symbolic neck direction
        m_graphics.setColor(Color.red);
        //draw the visor and use the actual view mode from the world
        m_graphics.fillArc((int)(m_myselfNewXPos-OVALDIAMETER/2),
                                    (int)(m_myselfNewYPos-OVALDIAMETER/2),
                                    OVALDIAMETER,
                                    OVALDIAMETER,
                                    f_startAngle,
                                    f_visorSize
                            );

        logger.debug("Drew neck direction!");
    }


    /**
     * This method draws the seen lines
     *
     * @param f_lineSide
     * @param f_isSeenLine
     */
    private void drawLine(int f_lineSide, boolean f_isSeenLine)
    {
        logger.debug("Drawing seen lines!");
        //set for all seen lines the same color
        m_graphics.setColor(COLOR_SEEN_LINE);

        //2 pixel width lines

        //top side line
        if(f_lineSide==2)
        {
            if(f_isSeenLine)
                m_graphics.fillRect(determNewXPosition(-52),determNewYPosition(-34),
                                    determNewXPosition(52) - determNewXPosition(-52),1
                                );
            //is used line
            else if(!f_isSeenLine)
                m_graphics.fillRect(determNewXPosition(-52),determNewYPosition(-34),
                                    determNewXPosition(52) - determNewXPosition(-52),3
                                );
        }

        //right side line
        else if(f_lineSide==1)
        {
            if(f_isSeenLine)
                m_graphics.fillRect(determNewXPosition(52)+1,determNewYPosition(-34),
                                    1,determNewYPosition(34) - determNewYPosition(-34)
                                );
            else if(!f_isSeenLine)
                m_graphics.fillRect(determNewXPosition(52)+1,determNewYPosition(-34),
                                    3,determNewYPosition(34) - determNewYPosition(-34)
                                );
        }

        //bottom side line
        else if(f_lineSide==3)
        {
               if(f_isSeenLine)
                    m_graphics.fillRect(determNewXPosition(-52),determNewYPosition(34),
                                    determNewXPosition(52) - determNewXPosition(-52),1
                                );
               else if(!f_isSeenLine)
                    m_graphics.fillRect(determNewXPosition(-52),determNewYPosition(34),
                                    determNewXPosition(52) - determNewXPosition(-52),3
                            );
       }

        //left side line
        else if(f_lineSide==0)
        {
            if (f_isSeenLine)
                m_graphics.fillRect(determNewXPosition(-52),determNewYPosition(-34),
                                  1,determNewYPosition(34) - determNewYPosition(-34)
                                );
            else if(!f_isSeenLine)
            {
                m_graphics.fillRect(determNewXPosition(-52),determNewYPosition(-34),
                                  3,determNewYPosition(34) - determNewYPosition(-34)
                            );
            }
        }
        logger.debug("Drew seen lines!");
    }


    /**
     * This method draws a seen goal
     *
     * @param f_isRightGoal
     */
    private void drawSeenGoal(boolean f_isRightGoal)
    {
        logger.debug("Drawing seen goal!");
        //set goal color line, 2 pixel width
        m_graphics.setColor(COLOR_SEEN_GOAL);
        //draw if it is the right goal
        if(f_isRightGoal)
        {
                m_graphics.drawLine(determNewXPosition(54),determNewYPosition(7),
                                        determNewXPosition(54),determNewYPosition(-7)
                                    );
        }
        //draw if it is the left goal
        else
        {
                m_graphics.drawLine(determNewXPosition(-54),determNewYPosition(7),
                                            determNewXPosition(-54),determNewYPosition(-7)
                                    );
        }
        logger.debug("Drew seen goal!");
    }


    /**
     * This method draws new enemy players
     *
     * @param f_enemyXPos
     * @param f_enemyYPos
     * @param f_visible
     */
    private void drawSeenEnemyPlayers(int f_enemyXPos,int f_enemyYPos, boolean f_visible)
    {
        logger.debug("Drawing seen enemy player!");
        if(f_visible) m_graphics.setColor(COLOR_ENEMY_PLAYER_VISIBLE);
        else m_graphics.setColor(COLOR_ENEMY_PLAYER_INVISIBLE);

        m_graphics.fillOval(f_enemyXPos,f_enemyYPos,10,10);
        logger.debug("Drew seen enemy player!");
    }


    /**
     * This method draws new friendly players
     *
     * @param f_friendlyXPos
     * @param f_friendlyYPos
     * @param f_visible
     */
     private void drawSeenFriendlyPlayers(int f_friendlyXPos,int f_friendlyYPos,boolean f_visible)
     {
        logger.debug("Drawing seen friendly player!");
        if(f_visible) m_graphics.setColor(COLOR_FRIENDLY_PLAYER_VISIBLE);
        else m_graphics.setColor(COLOR_FRIENDLY_PLAYER_INVISIBLE);
        m_graphics.fillOval(f_friendlyXPos,f_friendlyYPos,10,10);
        logger.debug("Drew seen friendly player!");
     }


    /**
     * This method draws the seen ball
     *
     * @param f_ballXPos
     * @param f_ballYPos
     * @param f_visible
     */
     private void drawSeenBall(int f_ballXPos,int f_ballYPos,boolean f_visible)
     {
        int l_ballDiameter=8;
        logger.debug("Drawing seen ball!");
        if(f_visible)
            m_graphics.setColor(COLOR_BALL_VISIBLE);
        else
            m_graphics.setColor(COLOR_BALL_INVISIBLE);
        m_graphics.fillOval(f_ballXPos-4,f_ballYPos-4,l_ballDiameter,l_ballDiameter);
        logger.debug("Drew seen seen ball!");
     }


     /**
      * This method draws seen flags
      *
      * @param f_flagXPos
      * @param f_flagYPos
      * @param f_isSeenFlag
      */
     private void drawFlag(int f_flagXPos,int f_flagYPos, boolean f_isSeenFlag)
     {
         logger.debug("Drawing seen flag!");
         //here flags can only be seen, otherwise the crosses from playfield are seen
         m_graphics.setColor(COLOR_FLAG_VISIBLE);
         //is a seen flag
         if(f_isSeenFlag)
            m_graphics.fillOval(f_flagXPos,f_flagYPos,5,5);
         //is a used flag
         else
            m_graphics.fillOval(f_flagXPos-OVALDIAMETER/2,
                                    f_flagYPos-OVALDIAMETER/2,OVALDIAMETER,OVALDIAMETER
                                );
         logger.debug("Drew seen flag!");
     }


     /**
      * This method draws myself
      *
      * @param f_myselfNewXPos
      * @param f_myselfNewYPos
      */
     private void drawMyself(int f_myselfNewXPos,int f_myselfNewYPos)
     {
         logger.debug("Drawing myself!");

         //set myself color for output
         m_graphics.setColor(COLOR_MYSELF);
         m_graphics.fillOval(f_myselfNewXPos-OVALDIAMETER/2,f_myselfNewYPos-OVALDIAMETER/2,OVALDIAMETER,OVALDIAMETER);
         logger.debug("Drew myself!");
     }

    /**
     * This method determs the proportional new x coordinate element
     * from world which has to be drawn
     *
     * @param f_worldXPos
     *
     * @return the new x pos. which has to be drawn
     */
     private int determNewXPosition(double f_worldXPos)
     {
        return (int)(f_worldXPos*this.getSize().getWidth()/PLAYFIELD_SIZE_WORLD_X + this.getSize().getWidth()/2d);
     }

    /**
     * This method determs the proportional new y coordinate element
     * from world which has to be drawn
     *
     * @param f_worldYPos
     *
     * @return the new y pos. which has to be drawn
     */
     private int determNewYPosition(double f_worldYPos)
     {
        return (int)(f_worldYPos*this.getSize().getHeight()/PLAYFIELD_SIZE_WORLD_Y + this.getSize().getHeight()/2d);
     }


    /**
     * This method determs the x element of second point of the displayed
     * line of myself body direction.
     *
     * @param f_myselfBodyDirectionAngel
     * @return the determed new second line point of myself body direction; x element
     */
    private int determSecondLinePointOfMyselfBodyDirectionX(double f_myselfBodyDirectionAngel)
    {
        int l_determedNewSecondLinePointOfMyselfBodyDirectionX;

        //test if the given angel is above the virtual x axies
        //of myself
        if(Math.abs(f_myselfBodyDirectionAngel) > 90)
        {
            l_determedNewSecondLinePointOfMyselfBodyDirectionX=(int)(
                                     m_myselfXPos
                                   - Math.abs(Math.cos(Math.abs(Angle.toRadian(f_myselfBodyDirectionAngel))))
                                   *LENGTHOFBODYDIRECTIONLINE
                                );
        }
        else
       {
           l_determedNewSecondLinePointOfMyselfBodyDirectionX=(int)(
                                  m_myselfXPos
                                + Math.abs(Math.cos(Math.abs(Angle.toRadian(f_myselfBodyDirectionAngel))))
                                *LENGTHOFBODYDIRECTIONLINE
                            );
       }
       return l_determedNewSecondLinePointOfMyselfBodyDirectionX;
    }


    /**
     * This method determs the y element of second point of the displayed
     * line of myself body direction.
     *
     * @param f_myselfBodyDirectionAngel
     * @return the determed new second line point of myself body direction; y element
     */
    private int determSecondLinePointOfMyselfBodyDirectionY(double f_myselfBodyDirectionAngel)
    {
        int l_determedNewSecondLinePointOfMyselfBodyDirectionY;

        //test if the given angel is left or right of the virtual y axies
        //of myself
        if(f_myselfBodyDirectionAngel > 0 )
        {
            l_determedNewSecondLinePointOfMyselfBodyDirectionY=(int)(
                                  m_myselfYPos
                                + Math.abs(Math.sin(Math.abs(Angle.toRadian(f_myselfBodyDirectionAngel))))
                                *LENGTHOFBODYDIRECTIONLINE
                            );
        }
        else
        {
           l_determedNewSecondLinePointOfMyselfBodyDirectionY=(int)(
                                  m_myselfYPos
                                - Math.abs(Math.sin(Math.abs(Angle.toRadian(f_myselfBodyDirectionAngel))))
                                *LENGTHOFBODYDIRECTIONLINE
                            );
       }
       return l_determedNewSecondLinePointOfMyselfBodyDirectionY;
    }
}
