package jais.visualization;


import org.apache.log4j.Logger;

import jais.world.WorldModel;
import jais.objects.*;
import jais.player.*;
import jais.world.WorldModel;


/**
 *
 * <p>Title: Visualization class</p>
 * <p>Description: This class provides all the needed things to
 * visualize the world model of one player. It creates a new window and
 * let display all the seen things.</p>
 * <p>Copyright: Copyright (c) 2002</p>
 * <p> </p>
 * @author Raul Zanzinger, Stephan Diederich
 *
 */
public class Visualization implements Runnable, VisualizationStuffInterface
{
    //class wide variables
    boolean m_threadSuspended=false;        //is our thread currently suspended?
    private Thread m_visuThread;            //for checking if still should run

    //class wide constants
    private MainFrame mainFrame;                         //references
    private WorldModel m_myModel;                        //
    private int m_playerNumber;                          //back number of displayed player
    private String m_teamName;                           //team name of displayed pplayer
    private static Logger logger=Logger.getLogger("jais.visualization");


    /**
     *  This is the constructor of Visualization class
     *
     * @param f_myWorld s
     * @param f_teamName s
     * @param f_playerNumber s
     */
    public Visualization(WorldModel f_myWorld,String f_teamName, int f_playerNumber)
    {
        m_playerNumber = f_playerNumber;
        m_teamName = f_teamName;
        m_myModel=f_myWorld;
    }


////////////////////////////////////////////////////////
//The implementation of VisualizationStuffInterface
////////////////////////////////////////////////////////

    /**
     * This Method instantiates a JFrame class and creates
     * a new window
     */
    public void createWindow()
    {
        //this instantiation creates a new JFrame window;
        //createWindow(); called by JAIS
        mainFrame = new MainFrame(m_myModel,m_teamName, m_playerNumber);
    }


    /**
     * This method instantiates a JPanel class and
     * starts displaying playfield and all the other objects
     * in a running thread
     *
     */
    public synchronized void startDisplay()
    {
         logger.info("Starting display!");
         if (m_visuThread==null)
         {
             m_visuThread = new Thread(this);
             //set a name for thread which will be created
             m_visuThread.setName("Visu. of player#: "+m_playerNumber+", Team: "+m_teamName);
             //go to run method and start the thread
             m_visuThread.start();
         }
    }


    /**
     *  This method wakes up the sleeping display thread
     *  to update the drawn panel itself in the created windows
     */
    public synchronized void updateDisplay()
    {
        //this call lets the displayed JPanel in created window
        //refresh itself by help of a reference on MainFrame
        // ok, he can run again if he wants to
        m_threadSuspended=false;
        //inform the running thread
        notify();
    }


    /**
     * This method is the run method for the diplaying
     * and updating the JPanel with the playfield and it's
     * objects
     */
    public void run()
    {
        while(m_visuThread != null)
        {
            //fetch the new objects from world
            mainFrame.repaintPlayfield();
            //we want to suspend ourself after one run
            m_threadSuspended=true;
            try
            {
                synchronized(this)
                {
                    //check again if we should sleep
                    while (m_threadSuspended && m_visuThread!=null)
                    {
                        // this command lets our thread sleep until we wake him up
                        wait();
                    }
                }
            }
            catch(java.lang.Exception ex)
            {
                logger.fatal("Catched exception while sleeping!");
            }
        }
    }


    /**
     *  This method stops displaying objects on the panel:
     *  it stops the running thread
     */
    public synchronized void stopDisplay()
    {
         //running thread should stop
         m_visuThread=null;
         //inform the running thread
         notify();
         logger.info("Stopped displaying!");
    }


    /**
     *
     */
    public void destroyWindow()
    {
         mainFrame.setVisible(false);
         //this frees the reserved memory for OS
         mainFrame.dispose();
         //erase the created window for user's eyes
         logger.info("Successful closed window!");
    }
}

