package jais.world;

import java.util.*;
import jais.objects.*;

/**
 * <p>Title: WorldDataAccessInterface</p>
 * <p>Description: This interface defines access to the WorldModel for the KI and the GUI modul.</p>
 * <p>Copyright: (c) 2002</p>
 * @author Stephan Diederich, Florian Mnz
 **/
public interface WorldDataAccessInterface
{
  /**
   * gives the simulation cycle in which we are
   * @return int cycle
   */
  public int sorry_whatTimeDoWeHave();

  /**
   * gives the playerinfos about the enemys
   * @return Arraylist playerinfos
   * @roseuid 3CE8BBA70270
   */
  public ArrayList getEnemyPlayers();

  /**
   * gives the playerinfos about our team
   * @return ArrayList playerinfos
   * @roseuid 3CE8BBB401E3
   */
  public ArrayList getFriendlyPlayers();

  /**
   * gives the playmode in which we are
   * @return int playmode
   * @roseuid 3CE8BBBF03DD
   */
  public int getPlaymode();

  /**
   * gives the last SenseBodyInfo we get
   * @return SenseBodyInfo
   * @roseuid 3CE8BBC80047
   */
  public SenseBodyInfo getSenseBodyInfo();

  /**
   * gives the last BallInfo we get
   * @return BallInfo
   * @roseuid 3CE8BBDC0028
   */
  public BallInfo getBall();

  /**
   * gives the last Playerinfo about our agent
   * @return Playerinfo
   * @roseuid 3CE8BBE10061
   */
  public PlayerInfo getMyself();

  /**
   * gives the last RefereeMessage that arrived (blocks until there was one)
   * @return AurealInfo RefereeMessage
   */
  public AurealInfo getLastRefereeMessage();

  /**
   * gives the last SayMessage from a player that arrived
   * @return AurealInfo SayMessage
   */
  public AurealInfo getLastPlayerMessage();

  /**
   * gives the last SayMessage from our coach that arrived
   * @return AurealInfo SayMessage
   */
  public AurealInfo getLastCoachMessage();

  /**
   * says if we play on leftside or rightside
   * @return true for left, false for right
   */
  boolean isPlayingLeft();

  /**
   * gives the absolute viewAngle from our Agent to the x-Axis
   * @return double viewAngle in degrees
   */
  public double getAbsViewAngle();

  /**
   * gives the absolute bodyDirection from our Agent to the x-Axis
   * @return double bodyDirection in degrees
   */
  public double getAbsBodyDir();

  /**
   * gives the angle from our bodyDirection to an object
   * @param f_obj is the object to which we want our BodyDirection
   * @return double bodyDirection in degrees
   */
  public double getBodyDirectionTo(ObjectInfo f_obj);

  /**
   * gives the angle from our bodyDirection to an object
   * @param f_obj is the coordinate to which we want our BodyDirection
   * @return double bodyDirection in degrees
   */
  public double getBodyDirectionTo(Coordinate f_obj);

  /**
   * gives the angle from our neck to our body
   * @return double neckDirection in degrees
   */
  public double getNeckDirection();

  /**
   * gives the distance from our agent to coordinates
   * @param f_coord h
   * @return double MyDistanceTo
   */
  public double getMyDistanceTo(Coordinate f_coord);

  /**
   * gives the distance from our agent to an object
   * @param f_object h
   * @return double MyDistanceTo
   */
  public double getMyDistanceTo(ObjectInfo f_object);

  /**
   * gives the viewMode
   * @return String viewMode
   */
  public String getViewMode();


  public int getViewModeAsInt();

  /**
   *
   * @return boolean, true if HighQuality, false if low
   */
  public String getViewQuality();

  /**************************************************************************************************/
/* Here follow methods provided for the visualization of our WorldModel
 /**************************************************************************************************/
 /**
  * gives the flags we saw this round (without them we used for calculating our Position)
  * @return LinkedList seenFlags
  */
  public LinkedList getCurrentylSeenFlags();

  /**
   * gives the flags we used this round for calculating our Position
   * @return LinkedList usedFlags
   */
  public LinkedList getCurrentylUsedFlags();

  /**
   * gives the lines we saw this round
   * @return LinkedList seenLines
   */
  public LinkedList getCurrentylSeenLines();

  /**
   * gives the lines we used this round for calculating our Position
   * @return LinkedList usedLines
   */
  public LinkedList getCurrentylUsedLines();

  /**
   * gives the goals we saw this round
   * @return LinkedList seenGoals
   */
  public LinkedList getCurrentylSeenGoals();

  /**
   * This method checks if any player is behind me (as then it is not in the lists)
   * @return true if a player is behind me, otherwise false
   */
  public boolean isPlayerBehind();

  /**
   * This method checks the ball is behind me (as then it is not visible)
   * @return true if ball is behind me, otherwise false
   */
  public boolean isBallBehind();
}