package jais.world;

import java.util.*;
import org.apache.log4j.Logger;
import org.apache.log4j.BasicConfigurator;
import jais.objects.*;
import jais.communication.*;
import javax.sound.sampled.*;
import java.io.*;

/**
 * <p>Title: WorldModel</p>
 * <p>Description: This is the WorldModel. It provides access to all data our KI and GUI needs.</p>
 * <p>Copyright: (c) 2002</p>
 * @author Stephan Diederich, Florian Mnz
 **/

public class WorldModel implements WorldUpdateInterface, WorldDataAccessInterface, WorldActionInformInterface
{
  private static Logger logger=Logger.getLogger("jais.world"); // get our logger

  protected BallInfo m_ball; // where the ball is stored
  private String m_teamName; // where the teamName is stored
  private int m_playerNumber;  // where the playerNumber is stored

  private final int FORGET_TIME= 2*10; // x-seconds
  private final double NOT_MOVING=0.01; // if deltaVector is smaller than we say Object did not move

  private ArrayList m_enemyTeam;  // where the teams are stored
  private ArrayList m_ownTeam;

  // storage for the objectinfos which came from one visualInfoEvent
  // we use LinkedLists here, as there is no modification from several threads, only our's is working here
  private LinkedList m_currentlySeenPlayers;
  private LinkedList m_currentlySeenFlags;
  private LinkedList m_currentlySeenGoals;
  private LinkedList m_currentlySeenLines;
  private LinkedList m_currentlyUsedFlags;
  private LinkedList m_currentlyUsedLines;
//  private short[] m_scoring={0,0};

  boolean m_playingOnLeftSide; // where we store on which side we play

  // storage for the last hearInfos and senseBodyInfos with one Info
  private final int STORAGE_COUNT_OF_INFOS=5;
  LinkedList m_senseBodyInfos;
  AurealInfo m_lastCoachMessage;
  AurealInfo m_lastPlayerMessage;
  AurealInfo m_lastRefereeMessage;

  // these variables are for our agent
  private PlayerInfo m_myself; // i have to be somewhere too
  private double m_absViewAngle; // from x-axis, positiv down, negativ up
  private double m_absBodyDir;

  private boolean m_aPlayerIsBehind;
  private boolean m_ballIsBehind;

  /**
   * Constructor initializes the member variables.
   * @param f_teamName is the teamName we connect with. Later used for looking who is in our team
   * @param f_isGoalie says if this guy is a goalie?
   * @param f_playerNumber is our playerNumber
   */
  public WorldModel(String f_teamName,boolean f_isGoalie,int f_playerNumber)
  {
    m_playerNumber=f_playerNumber;
    m_teamName=f_teamName;
    m_enemyTeam=new ArrayList(11);
    m_ownTeam=new ArrayList(11);
    m_senseBodyInfos=new LinkedList();
    m_currentlyUsedFlags=new LinkedList();
    m_currentlyUsedLines=new LinkedList();
    FlagInfo.initFlagPositions();       // init the Hashtable with the flag positions
    GoalInfo.initGoalPositions();       // init the Hashtable with the goal positions
    m_ball=new BallInfo();              // init the ball
    m_myself=new PlayerInfo();          // init ourself
    m_myself.setGoalie(f_isGoalie);     // init if goalie or not
  }

  /****************************************************************************************/
/* Here starts the implementation of the WorldDataAcessInterface
/* Comments and usage for this methods are declared in the interface
 /****************************************************************************************/
  public double getAbsViewAngle() {return m_absViewAngle;}

  public double getAbsBodyDir() {return m_absBodyDir;}

  public double getBodyDirectionTo(ObjectInfo f_obj)
  {
/*
    if (this.sorry_whatTimeDoWeHave()-f_obj.getTime()==0)
    {
      return (Angle.normalize(f_obj.getDirection()+getNeckDirection()));
    }
    else// the direction values are not available as we moved away
    {
      return getBodyDirectionTo(f_obj.getAbsPosition());
    }
*/
    return getBodyDirectionTo(f_obj.getAbsPosition());
  }

  public double getBodyDirectionTo(Coordinate f_coord)
  {
    double l_angle=0;
    l_angle=Math.atan2(f_coord.getY()-m_myself.getAbsPosition().getY(),(f_coord.getX()-m_myself.getAbsPosition().getX()));
    l_angle=(Math.toDegrees(l_angle)-getAbsBodyDir());
    return Angle.normalize(l_angle);
  }

  public double getMyDistanceTo(Coordinate f_coord)
  {
    return (Math.sqrt((Math.pow(Math.abs(m_myself.getAbsPosition().getX() - f_coord.getX()), 2)) +
                      (Math.pow(Math.abs(m_myself.getAbsPosition().getY() - f_coord.getY()), 2))));
    //Pythagoras *gg*
  }

  public double getMyDistanceTo(ObjectInfo f_object)
  {
    return getMyDistanceTo(f_object.getAbsPosition());
  /*
    if(this.sorry_whatTimeDoWeHave()-f_object.getTime()==0) //we saw this guy this round
    {
      return f_object.getDistance();
    }
    else
    {
      return getMyDistanceTo(f_object.getAbsPosition());
    }
  */
  }

  public double getNeckDirection()
  {
    return getSenseBodyInfo().m_neckDirection;
  }

  public boolean isPlayingLeft(){return m_playingOnLeftSide;}

  public int getPlaymode()
  {
    return getLastRefereeMessage().m_playmode; //if we dont have one we're before kickoff
  }

  public ArrayList getEnemyPlayers()
  {
    return m_enemyTeam;
  }

  public ArrayList getFriendlyPlayers()
  {
    return m_ownTeam;
  }

  public SenseBodyInfo getSenseBodyInfo()
  {
    try
    {
      return(SenseBodyInfo)m_senseBodyInfos.getLast();
    }
    catch (Exception ex)
    {
      while(m_senseBodyInfos.isEmpty())
          { }  //wait until we have an object
      return(SenseBodyInfo)m_senseBodyInfos.getLast();
    }
  }

  public BallInfo getBall()
  {
    return m_ball;
  }

  public PlayerInfo getMyself()
  {
    return m_myself;
  }

  public AurealInfo getLastPlayerMessage()
  {
    return m_lastPlayerMessage;
  }

  public AurealInfo getLastCoachMessage()
  {
    return m_lastCoachMessage;
  }

  public AurealInfo getLastRefereeMessage()
  {
    while(m_lastRefereeMessage==null) //wait until we have a referee Message
    {
    }
    return m_lastRefereeMessage;
  }

  public int sorry_whatTimeDoWeHave()
  {
    return (getSenseBodyInfo().getTime());
  }

  public LinkedList getCurrentylSeenFlags()
  {
    return m_currentlySeenFlags;
  }

  public LinkedList getCurrentylSeenLines()
  {
    return m_currentlySeenLines;
  }

  public LinkedList getCurrentylSeenGoals()
  {
    return m_currentlySeenGoals;
  }

  public LinkedList getCurrentylUsedFlags()
  {
    return m_currentlyUsedFlags;
  }

  public LinkedList getCurrentylUsedLines()
  {
    return m_currentlyUsedLines;
  }

  public String getViewMode()
  {
    return getSenseBodyInfo().m_viewMode;
  }

  public String getViewQuality()
  {
    return getSenseBodyInfo().m_ViewQuality;
  }

  public boolean isPlayerBehind()
  {
    return m_aPlayerIsBehind;
  }

  public boolean isBallBehind()
  {
    return m_ballIsBehind;
  }

  public int getViewModeAsInt()
  {
    String l_blub = getViewMode();
    if (l_blub.startsWith("normal"))
      return 1;
    else if (l_blub.startsWith("wide"))
      return 2;
    else
      return 0;
  }

  /****************************************************************************************/
/* Here starts the implementation of the WorldUpdateInterface
/* Comments and usage for this methods are declared in the interface
 /****************************************************************************************/
  public void updateAurealInfo(HearEvent f_hearEvent)
  {
    AurealInfo l_aurealInfo=(AurealInfo)f_hearEvent.getObjects().get(0);
    //get the right message into the right basket
    switch (l_aurealInfo.getSender())
    {
      case AurealInfo.AUREAL_SENDER_REFEREE:
        m_lastRefereeMessage=l_aurealInfo;
        if(m_lastRefereeMessage.m_playmode== AurealInfo.GOAL_L && m_playingOnLeftSide || m_lastRefereeMessage.m_playmode== AurealInfo.GOAL_R && !m_playingOnLeftSide)
        {
//          m_scoring[0]++;
          if (m_playerNumber==1)
          {
            ClipPlayer l_blub=new ClipPlayer(new File("../ext/1.wav"),1);
          }
        }
        else if (m_lastRefereeMessage.m_playmode== AurealInfo.GOAL_L || m_lastRefereeMessage.m_playmode== AurealInfo.GOAL_R)
//          m_scoring[1]++;
        break;
      case AurealInfo.AUREAL_SENDER_OTHER:
        m_lastPlayerMessage=l_aurealInfo;
        break;
      case AurealInfo.AUREAL_SENDER_COACH_LEFT:
        if (m_playingOnLeftSide)
          m_lastCoachMessage=l_aurealInfo;
        break;
      case AurealInfo.AUREAL_SENDER_COACH_RIGHT:
        if (!m_playingOnLeftSide)
          m_lastCoachMessage=l_aurealInfo;
        break;
    }
//    logger.info("Player"+m_playerNumber+"added a hearEvent to my list");
  }

  public synchronized void updateVisualInfo(VisualEvent f_visualEvent)
  {
//    logger.info("updating my Visual Info and calculating absolute positions now");
    ArrayList l_visualInfos=f_visualEvent.getObjects();
    ObjectInfo l_objectInfo;
    m_currentlySeenFlags=new LinkedList();
    m_currentlySeenPlayers=new LinkedList();
    m_currentlySeenGoals=new LinkedList();
    m_currentlySeenLines=new LinkedList();
    m_ball.setVisible(false);
    m_ballIsBehind=false;
    m_aPlayerIsBehind=false;

    //store the objects in this visual event in the correct list
    for(ListIterator it=l_visualInfos.listIterator();it.hasNext();)
    {
      l_objectInfo=(ObjectInfo)it.next();
      if (l_objectInfo instanceof FlagInfo) //store the flags
      {
        if (!l_objectInfo.isBehind())
          m_currentlySeenFlags.addLast(l_objectInfo);
      }
      else if (l_objectInfo instanceof PlayerInfo) //store the players
      {
        if (!l_objectInfo.isBehind())
          m_currentlySeenPlayers.addLast(l_objectInfo);
        else
          m_aPlayerIsBehind=true;
      }
      else if (l_objectInfo instanceof BallInfo) //if we see the ball, store it
      {
        if(!l_objectInfo.isBehind())
        {
          m_ball=(BallInfo)l_objectInfo;
//          logger.fatal("time:"+m_ball.getTime()+"direction "+ m_ball.getDirection() +" distance"+ m_ball.getDistance());
          m_ball.setVisible(true);
        }
        else
          m_ballIsBehind=true;
      }
      else if (l_objectInfo instanceof GoalInfo) // store the goal(s)
      {
        if (!l_objectInfo.isBehind())
        {
          m_currentlySeenGoals.addLast(l_objectInfo);
        }
      }
      else if (l_objectInfo instanceof LineInfo) //store some lines
      {
        m_currentlySeenLines.add(l_objectInfo);
      }
    }
    l_objectInfo=null; // we dont need them anymore
    l_visualInfos=null;
    updateAbsolutePositions();
    updatePlayerLists();
  }

  public void updateSenseBodyInfo(SenseBodyEvent f_senseBodyEvent)
  {
//   logger.fatal("turn count "+((SenseBodyInfo)f_senseBodyEvent.getObjects().get(0)).m_turnCount);
    while(m_senseBodyInfos.size()>=STORAGE_COUNT_OF_INFOS)
      m_senseBodyInfos.removeFirst();
    if(!f_senseBodyEvent.getObjects().isEmpty())
    {
      m_senseBodyInfos.addLast(f_senseBodyEvent.getObjects().get(0));
//      logger.info("added a SenseBodyInfo to my List");
    }

  }

  public synchronized void updateInitInfo(InitialEvent f_initialEvent)
  {
    if (!f_initialEvent.getObjects().isEmpty())
    {
//      logger.info("updating initial info's");
      //get the one and only init info object
      InitInfo l_initInfo=(InitInfo)f_initialEvent.getObjects().get(0);
      //save the side on which we play
      m_playingOnLeftSide=l_initInfo.m_playingOnLeftSide;
      //save our uniform Number in myself
      m_myself.setBackNumber(l_initInfo.m_uniformNumber);
      //create and add a dummy aurealInfo for the first playmode
      AurealInfo l_aurealInfo=new AurealInfo();
      l_aurealInfo.m_playmode=l_initInfo.m_playmode;
      m_lastRefereeMessage=l_aurealInfo;
    }
  }

  /****************************************************************************************/
/* Here starts the implementation of the WorldActionInformInterface
/* Comments and usage for this methods are declared in the interface
 /****************************************************************************************/

    /*THIS METHODS DO NOTHING AT THE MOMENT THEY ARE HERE ONLY FOR LATER POSSIBLE USE*/
  public void informCatchBall(double f_angle) {}

    /*THIS METHODS DO NOTHING AT THE MOMENT THEY ARE HERE ONLY FOR LATER POSSIBLE USE*/
  public void informDash(double f_power)
  {
    /*
    f_power=Math.min(100,f_power);
    double l_absBodyDirection=  (getAbsBodyDir())*Math.PI/180;
    m_myself.setAbsPosition(new Coordinate(
                        Math.cos(l_absBodyDirection)*f_power*0.01f+m_myself.getAbsPosition().getX(),
                        Math.sin(l_absBodyDirection)*f_power*0.01f+m_myself.getAbsPosition().getY()
                                            ));
    */
  }

    /*THIS METHODS DO NOTHING AT THE MOMENT THEY ARE HERE ONLY FOR LATER POSSIBLE USE*/
  public void informKick(double f_power, double f_angle) {}

    /*THIS METHODS DO NOTHING AT THE MOMENT THEY ARE HERE ONLY FOR LATER POSSIBLE USE*/
  public void informTurnNeck(double f_angle)
  {
    //getSenseBodyInfo().setNeckDirection(getSenseBodyInfo().getNeckDirection()+f_angle);
  }

    /*THIS METHODS DO NOTHING AT THE MOMENT THEY ARE HERE ONLY FOR LATER POSSIBLE USE*/
  public void informMove(double f_x, double f_y)
  {
    //m_myself.setAbsPosition(new Coordinate(f_x, f_y));
  }

    /*THIS METHODS DO NOTHING AT THE MOMENT THEY ARE HERE ONLY FOR LATER POSSIBLE USE*/
  public void informTurn(double f_angle)
  {
    /*
    double l_tempAngle=getAbsViewAngle()+f_angle;
    if (l_tempAngle>180)
        m_absViewAngle=-360+l_tempAngle;
    else if(l_tempAngle<-180)
        m_absViewAngle=360+l_tempAngle;
    l_tempAngle=getAbsBodyDir()+f_angle;
    if (l_tempAngle>180)
        m_absBodyDir=-360+f_angle;
    else if(l_tempAngle<-180)
        m_absBodyDir=360+f_angle;
//    logger.debug("abs view angle from inform:"+ getAbsViewAngle());
    logger.debug("abs bodyDir from inform:"+ getAbsBodyDir());
    */
  }

  /****************************************************************************************/
/* here start the private methods
 /****************************************************************************************/
 /**
  * This Method calculates my Abs Position and viewAngle
  * with one line and one flag, or 2 flags, or leaves as it was
  * if we dont see any objects
  */
  private void calculateMyAbsStuff()
  {
    m_currentlyUsedFlags.clear(); // remove the items in this list, as we calculate here our new position
    m_currentlyUsedLines.clear();
    boolean l_2flagUpdateHappened=false;
    double l_oldViewAngle=0;
    Coordinate l_oldPosition=null;
    if (m_currentlySeenFlags.size()>1) // do this allways to look if we were correct
    {
      l_2flagUpdateHappened=true;
      calculateMyAbsStuffWith2Flags();
//      logger.info("2Flags: Absolute Position : " + "(" + (int) m_myself.getAbsPosition().getX() + "|" +(int) m_myself.getAbsPosition().getY() + ")" + " Abs Angle: " + (int) getAbsViewAngle());
    }
    if(!m_currentlySeenLines.isEmpty())
    {
      if (l_2flagUpdateHappened) // check only if we have something to compare
      {
        l_oldViewAngle=getAbsViewAngle();
        l_oldPosition=m_myself.getAbsPosition();
      }
      if(m_currentlySeenFlags.isEmpty())
      {
                /*with one line is pretty bad :( because if we are not in the field we get a wrong position*/
        //calculateMyAbsViewAngleWithALine();
        //logger.fatal("Angle: Absolute Position : " + "(" + (int) m_myself.getAbsPosition().getX() + "|" +(int) m_myself.getAbsPosition().getY() + ")" + " Abs Angle: " + (int) getAbsViewAngle());
      }
      else
      {
        calculateMyAbsStuffWithLineAndFlag();
//        logger.info("Line+Flag Absolute Position : " + "(" + (int) m_myself.getAbsPosition().getX() + "|" +(int) m_myself.getAbsPosition().getY() + ")" + " Abs Angle: " + (int) getAbsViewAngle());
      }
      if (l_2flagUpdateHappened)
        if (Math.abs(getAbsViewAngle()-l_oldViewAngle)>100)
        {
          m_absViewAngle=l_oldViewAngle;
          m_myself.setAbsPosition(l_oldPosition);
//          logger.info("Corrected view Angle to " +l_oldViewAngle + "and Pos to" + l_oldPosition);
        }
    }
    else
    {
//      logger.debug("Not enough information in the last see message for updating Position.");
    }
        /*set our body direction here and correct if necessary*/
    m_absBodyDir=Angle.normalize(getAbsViewAngle()-getNeckDirection());
    m_myself.setAbsPositionOfNextRound(new Coordinate(
        m_myself.getAbsPosition().getX()+Math.cos(Angle.toRadian(m_absBodyDir)) * getSenseBodyInfo().m_amountOfSpeed,
        m_myself.getAbsPosition().getY()+Math.sin(Angle.toRadian(m_absBodyDir)) * getSenseBodyInfo().m_amountOfSpeed
        ));
  }

  /**
   * This Method calculates my Abs Position and viewAngle
   * with one line and one flag (it also removes that flag)
   */
  private void calculateMyAbsStuffWithLineAndFlag()
  {
    calculateMyAbsViewAngleWithALine();
    try
    {
      FlagInfo l_flag = (FlagInfo)getNearestObject(m_currentlySeenFlags);
      m_currentlySeenFlags.remove(l_flag);
      m_currentlyUsedFlags.add(l_flag);
      Coordinate l_flagCoord = l_flag.getAbsPosition();
      //arg, cos and sin need a radiant
      double l_absDirection = (getAbsViewAngle() + l_flag.getDirection())*Math.PI/180;
      Coordinate l_absPosition=new Coordinate(
          (l_flagCoord.getX()- l_flag.getDistance()*Math.cos(l_absDirection)),
          (l_flagCoord.getY()- l_flag.getDistance()*Math.sin(l_absDirection))
          );
      if (!m_playingOnLeftSide)
      {
        m_absViewAngle=Angle.normalize(m_absViewAngle += 180.0);
        l_absPosition.setX(l_absPosition.getX()*-1);
        l_absPosition.setY(l_absPosition.getY()*-1);
      }
      m_myself.setAbsPosition(l_absPosition);

      }catch(NullPointerException e){}
  }

  /**
   * This Method calculates my Abs Position and viewAngle
   * with 2 Flags. It also removes these two out of the list
   */
  private void calculateMyAbsStuffWith2Flags()
  {
    Coordinate l_coord=new Coordinate(0,0);
    FlagInfo l_flag1=null;
    FlagInfo l_flag2=null;
    Coordinate l_pos1=null;
    Coordinate l_pos2=null;
    double l_dist, l_dist2;
    double l_flag1_x, l_flag2_x, l_flag1_y, l_flag2_y, l_radius1, l_radius2, a, h, x3, y3;
    double l_ratio;
    l_flag1 = (FlagInfo) getNearestObject(m_currentlySeenFlags);
    m_currentlySeenFlags.remove(l_flag1);
    m_currentlyUsedFlags.add(l_flag1);
    l_pos1 = l_flag1.getAbsPosition();
    l_flag2 = (FlagInfo) getNearestObject(m_currentlySeenFlags);
    m_currentlySeenFlags.remove(l_flag2);
    m_currentlyUsedFlags.add(l_flag2);
    l_pos2 = l_flag2.getAbsPosition();

    // do the calculations
    l_radius1 = l_flag1.getDistance();
    l_radius2 = l_flag2.getDistance();

    l_flag1_x = l_pos1.getX();
    l_flag2_x = l_pos2.getX();

    l_flag1_y = l_pos1.getY();
    l_flag2_y = l_pos2.getY();

    // calculate the square distance of the two flags
    l_dist2 = (l_flag2_x - l_flag1_x)*(l_flag2_x - l_flag1_x) + (l_flag2_y - l_flag1_y)*(l_flag2_y - l_flag1_y);
    l_dist = Math.sqrt(l_dist2);
    if (l_dist > l_radius1 + l_radius2)
    {
      // the circles would not intersect (because of inaccuracy)
      l_dist = l_radius1 + l_radius2;
      l_dist2 = l_dist * l_dist;
    }
    else if ((l_radius1 > l_radius2) && (l_dist + l_radius2 < l_radius1))
    {
      // the circles would not intersect (inaccuracy)
      l_dist = l_radius1 - l_radius2;
      l_dist2 = l_dist * l_dist;
    }
    else if ((l_radius2 > l_radius1) && (l_dist + l_radius1 < l_radius2))
    {
      // the circles would not intersect
      l_dist = l_radius2 - l_radius1;
      l_dist2 = l_dist * l_dist;
    }
    l_radius1 *= l_radius1;
    l_radius2 *= l_radius2;

    // a = (r1^2 - r2^2 + d^2 ) / (2 d)
    a = (l_radius1 - l_radius2 + l_dist2) / (2.0 * l_dist);
    // h^2 = r1^2 - a^2
    h = l_radius1 - a*a;
    if (h < 0.0)
      h = 0.0;
    else
      h = Math.sqrt(h);

    // P3 = P1 + a ( P2 - P1 ) / d
    x3 = l_flag1_x + a * (l_flag2_x - l_flag1_x) / l_dist;
    y3 = l_flag1_y + a * (l_flag2_y - l_flag1_y) / l_dist;

    // two circles intersect usually in 2 points. Find out which one to select
    if (l_flag1.getDirection() > l_flag2.getDirection())
    {
      // result x = x3 + h ( y2 - y1 ) / d
      l_coord.setX(x3 + h * (l_flag2_y - l_flag1_y) / l_dist);
      l_coord.setY(y3 - h * (l_flag2_x - l_flag1_x) / l_dist);
    }
    else
    {
      l_coord.setX(x3 - h * (l_flag2_y - l_flag1_y) / l_dist);
      l_coord.setY(y3 + h * (l_flag2_x - l_flag1_x) / l_dist);
    }


    // calculate the absolute direction
    l_radius1 = l_flag1.getDistance();
    double l_absViewAngle=0;
    l_ratio = (l_flag1_y - l_coord.getY()) / l_radius1;
    int beta = (int) Math.toDegrees(Math.asin(l_ratio));
    if (l_coord.getX() > l_flag1_x)
      l_absViewAngle = 180 - beta - l_flag1.getDirection();
    else
      l_absViewAngle = beta - l_flag1.getDirection();

    if (!m_playingOnLeftSide)
    {
      l_coord.setX(l_coord.getX()*-1.0);
      l_coord.setY(l_coord.getY()*-1.0);
      l_absViewAngle += 180.0;
    }
    m_absViewAngle=Angle.normalize(l_absViewAngle);
    m_myself.setAbsPosition(l_coord);
  }

  /**
   * This method calculates my absolute view angle with one line
   */
  private void calculateMyAbsViewAngleWithALine()
  {
    boolean l_sign_pos=true;
    LineInfo l_line = (LineInfo)getNearestObject(m_currentlySeenLines);
    m_currentlyUsedLines.add(l_line);
    if (l_line.getDirection() < 0)
      l_sign_pos = false;
    if(l_line.getLineType() == LineInfo.LINE_TOP)
      if(l_sign_pos==true)
        m_absViewAngle=-l_line.getDirection();
    else
      m_absViewAngle=-180-l_line.getDirection();
    else if(l_line.getLineType() == LineInfo.LINE_BOTTOM)
      if(l_sign_pos==true)
        m_absViewAngle=180- l_line.getDirection();
    else
      m_absViewAngle=-l_line.getDirection();
    else if(l_line.getLineType() == LineInfo.LINE_RIGHT)
      if(l_sign_pos==true)
        m_absViewAngle=90- l_line.getDirection();
    else
      m_absViewAngle=-90-l_line.getDirection();
    else if(l_line.getLineType() == LineInfo.LINE_LEFT)
      if(l_sign_pos==true)
        m_absViewAngle=-90 -l_line.getDirection();
    else
      m_absViewAngle=90-l_line.getDirection();

    m_absViewAngle=Angle.normalize(m_absViewAngle);
  }



  /**
   * This method updates the absolute position of our agent, the ball and the players.
   * @roseuid 3CE8BF730250
   */
  private void updateAbsolutePositions()
  {
//    logger.debug("Going to update my absolute positions");
/*************************************************************************************/
//update myself here
    calculateMyAbsStuff();
//    logger.info("Playing left? "+m_playingOnLeftSide);
//    logger.info("Pos:" + m_myself.getAbsPosition() + "V Angle" + getAbsViewAngle() +"B Angle" + getAbsBodyDir());
/*************************************************************************************/
// update the ball here
if (m_ball.isVisible())
{
  calculateAbsPosOfBall();
}
    /*else if (sorry_whatTimeDoWeHave()-m_ball.getTime()< 7)
    {
      Coordinate l_nextRound =(m_ball.getAbsPositionOfNextRound()!=null)? m_ball.getAbsPositionOfNextRound() : m_ball.getAbsPosition();
      Coordinate l_moving=l_nextRound.sub(m_ball.getAbsPosition());
      m_ball.setAbsPosition(l_nextRound);
      m_ball.setAbsPositionOfNextRound(m_ball.getAbsPosition().add(l_moving.scaleWith(ServerConf.BALL_DECAY)));
//      logger.info("I dont see the ball :((");
    }
*/
/*************************************************************************************/
// update the players here
double l_absAngle;
double l_dist;
ObjectInfo l_tempObject;
for (ListIterator it=m_currentlySeenPlayers.listIterator();it.hasNext();)
{
  l_tempObject=(ObjectInfo)it.next();     //get the next player
  l_absAngle= Angle.toRadian(l_tempObject.getDirection()+getAbsViewAngle()); //get its direction and convert to Radian
  l_dist = l_tempObject.getDistance();    //get its distance
  l_tempObject.setAbsPosition(new Coordinate (    //and write its absPosition
      m_myself.getAbsPosition().getX()+Math.cos(l_absAngle)*l_dist,
      m_myself.getAbsPosition().getY()+Math.sin(l_absAngle)*l_dist
      ));
  if (l_tempObject.getDirChange()!=ObjectInfo.NOT_EXISTING && l_tempObject.getDistChange()!=ObjectInfo.NOT_EXISTING)
  {
    l_absAngle=Angle.toRadian(l_tempObject.getDirection()+l_tempObject.getDirChange() + getAbsViewAngle());
    l_dist = l_tempObject.getDistance()+l_tempObject.getDistChange();
    l_tempObject.setAbsPositionOfNextRound(new Coordinate(
        m_myself.getAbsPositionOfNextRound().getX()+Math.cos(l_absAngle)*l_dist,
        m_myself.getAbsPositionOfNextRound().getY()+Math.sin(l_absAngle)*l_dist)
        );
    if (Math.abs(l_tempObject.getAbsPositionOfNextRound().getX()-l_tempObject.getAbsPosition().getX())<NOT_MOVING
        && Math.abs(l_tempObject.getAbsPositionOfNextRound().getY()-l_tempObject.getAbsPosition().getY())<NOT_MOVING)
      l_tempObject.setAbsPositionOfNextRound(null);
  }
  else
    l_tempObject.setAbsPositionOfNextRound(null);
}
l_tempObject=null; //dont need it anymore
  }

  /**
   * This method calculates the absolute Position of the ball with the values
   * it has stored in direction and distance
   */
  private void calculateAbsPosOfBall()
  {
    double l_absAngle=Angle.toRadian(m_ball.getDirection() + getAbsViewAngle());
    double l_dist = m_ball.getDistance();
    m_ball.setAbsPosition(new Coordinate(
        m_myself.getAbsPosition().getX()+Math.cos(l_absAngle)*l_dist,
        m_myself.getAbsPosition().getY()+Math.sin(l_absAngle)*l_dist)
        );
    /*calculate Position of ball it has next round*/
    if (m_ball.getDirChange()!=ObjectInfo.NOT_EXISTING && m_ball.getDistChange()!=ObjectInfo.NOT_EXISTING)
    {
      l_absAngle=Angle.toRadian(m_ball.getDirection()+m_ball.getDirChange() + getAbsViewAngle());
      l_dist = m_ball.getDistance()+m_ball.getDistChange();

      m_ball.setAbsPositionOfNextRound(new Coordinate(
          m_myself.getAbsPositionOfNextRound().getX()+Math.cos(l_absAngle)*l_dist,
          m_myself.getAbsPositionOfNextRound().getY()+Math.sin(l_absAngle)*l_dist)
          );
      if (Math.abs(m_ball.getAbsPositionOfNextRound().getX()-m_ball.getAbsPosition().getX())<NOT_MOVING &&
          Math.abs(m_ball.getAbsPositionOfNextRound().getY() -m_ball.getAbsPosition().getY())<NOT_MOVING)
        m_ball.setAbsPositionOfNextRound(null);
    }
    else
      m_ball.setAbsPositionOfNextRound(null);
//    logger.debug("Pos of ball"+m_ball.getAbsPosition());
//    logger.debug("Pos of ball next round:"+m_ball.getAbsPositionOfNextRound());
  }

  /**
   * This method searches a nearest object to our agent.
   * @param f_list an ArrayList of ObjectInfos
   * @return the nearest ObjectInfo in this list
   */
  private ObjectInfo getNearestObject(LinkedList f_list)
  {
    if(f_list.isEmpty())
      return null;
    double l_distance=10000;
    ObjectInfo l_objectInfo=null;
    ObjectInfo l_tempObjectInfo=null;
    for(Iterator it = f_list.listIterator(); it.hasNext();)
    {
      l_tempObjectInfo=((ObjectInfo)it.next());
      if(l_tempObjectInfo.getDistance()<l_distance)
      {
        l_distance=l_tempObjectInfo.getDistance();
        l_objectInfo=l_tempObjectInfo;
      }
    }
    return l_objectInfo;
  }

  /**
   * This method checks all currently seen players and add it to the correct list
   * there are 3 possibilities:
   *   we get 'null' when we call the teamname
   *   we see our teamname
   *   we see a teamname, but its not ours
   */
  private void updatePlayerLists()
  {
    //our variables for this method
    PlayerInfo l_newPlayer;
    PlayerInfo l_oldPlayer;
    PlayerInfo l_playerInfo;
    LinkedList l_playersWithOutTeamname=null;

    //set visibility of both teams to false
    m_enemyTeam=setVisible(m_enemyTeam,false);
    m_ownTeam=setVisible(m_ownTeam,false);

    //go through the player list and save them into our corresponding list
    for(ListIterator it=m_currentlySeenPlayers.listIterator();it.hasNext();)
    {
      l_newPlayer=(PlayerInfo)it.next(); //ok, here we get the next player
      l_newPlayer.setVisible(true); //ok, we see him
      //look in which team he is
      String l_teamname=l_newPlayer.getTeamName();
      if (l_teamname==null) //ok, we see a player but not his teamname....going to store them for later use
      {
        if (l_playersWithOutTeamname==null) //list not yet initialized ?
        {
          l_playersWithOutTeamname=new LinkedList();
        }
        l_playersWithOutTeamname.addLast( l_newPlayer);
      }
      else if (l_teamname.equalsIgnoreCase(m_teamName)) //ok, hes in our team
      {
        if (l_newPlayer.getBackNumber()==ObjectInfo.NOT_EXISTING) // we dont see his back-number
        {
          //check if we have one guy with similar position in our team
          l_oldPlayer=playerIsInList(m_ownTeam,l_newPlayer,false,true);
        }
        else  // we see his backnumber
        {
          l_oldPlayer=playerIsInList(m_ownTeam,l_newPlayer,true,false);//check if we have that player in our ownTeamList
          if (l_oldPlayer==null)
            l_oldPlayer=playerIsInList(m_ownTeam,l_newPlayer,false,true); //maybe we saw him last round without a number
        }

        if (l_oldPlayer!=null) //check if we found a similar player, then remove the old and add the new
        {
          m_ownTeam.remove(l_oldPlayer);
          m_ownTeam.add(l_newPlayer);
        }
        else
        {
          m_ownTeam.add(l_newPlayer); //ok, hes really new, add him simply
        }
      }
      else //ok, we see a teamname, its not ours => its a enemy ..HELP! :)
      {
        if(l_newPlayer.getBackNumber()==ObjectInfo.NOT_EXISTING) //we dont see his number
          l_oldPlayer=playerIsInList(m_enemyTeam,l_newPlayer,false, true); //so check for AbsPosition
        else
        {
          l_oldPlayer=playerIsInList(m_enemyTeam,l_newPlayer,true,false); //is he in the enemy list ?
          if (l_oldPlayer==null)
            l_oldPlayer=playerIsInList(m_enemyTeam,l_newPlayer,false,true); //maybe we saw him last round without a number

        }
        if(l_oldPlayer!=null)
        {
          m_enemyTeam.remove(l_oldPlayer);
          m_enemyTeam.add(l_newPlayer);
        }
        else
        {
          m_enemyTeam.add(l_newPlayer);
        }
      }
    }

    //ok, check if we had players without teamname and check if it might be one we already know
    //we do this with a position check and only with guys we dont see this moment
    if (l_playersWithOutTeamname!=null)
    {
      while(!l_playersWithOutTeamname.isEmpty()) //dont forget one ;)
      {
        l_newPlayer=(PlayerInfo)l_playersWithOutTeamname.removeFirst();
        l_oldPlayer=playerIsInList(m_enemyTeam,l_newPlayer,false,true); //check if he is maybe in the enemy team
        if (l_oldPlayer!=null)
        {
          m_enemyTeam.remove(l_oldPlayer);
          m_enemyTeam.add(l_newPlayer);
//          logger.info("we found someone cause of its position");
        }
        else //maybe he is in our team
        {
          l_oldPlayer=playerIsInList(m_ownTeam,l_newPlayer,false,true); //check if he is maybe in the enemy team
          if (l_oldPlayer!=null)
          {
            m_ownTeam.remove(l_oldPlayer);
            m_ownTeam.add(l_newPlayer);
//            logger.info("we found someone cause of its position");
          }
        }
        //ok, we cant identify that sucker.... so discard him
      }
    }
    //killing old players of my team
    for (ListIterator it=m_ownTeam.listIterator();it.hasNext();)
    {
      l_playerInfo=(PlayerInfo)it.next();
      if (this.sorry_whatTimeDoWeHave()-l_playerInfo.getTime()> FORGET_TIME)
        it.remove();
    }
    //killing old players of enemy team
    for (ListIterator it=m_enemyTeam.listIterator();it.hasNext();)
    {
      l_playerInfo=(PlayerInfo)it.next();
      if (this.sorry_whatTimeDoWeHave()-l_playerInfo.getTime()> FORGET_TIME)
        it.remove();
    }
  } //processNewDynamicObjects

  /**
   * Checks if l_player is in f_playerInfoList ...
   *
   * @param f_playerInfoList List where to look
   * @param f_player Player for which we want to look
   * @param f_checkForBackNumber should i compare the backnumbers ?
   * @param f_checkForPosition sould i compare the absolute positions ?
   * @return PlayerInfo Object that was in that list, or null
   */
  private PlayerInfo playerIsInList(ArrayList f_playerInfoList, PlayerInfo f_player,boolean f_checkForBackNumber,boolean f_checkForPosition)
  {
    PlayerInfo l_playerInList;
    if (f_checkForBackNumber)
    {
      int l_number = f_player.getBackNumber();
      for (ListIterator it = f_playerInfoList.listIterator();it.hasNext();)
      {
        l_playerInList=(PlayerInfo)it.next();
        if (l_playerInList.getBackNumber()==l_number)
          return l_playerInList;
      }
    }
    else if (f_checkForPosition) // we search for a player we dont see but has nearly the same position
    {
      Coordinate l_coord = f_player.getAbsPosition();
      for (ListIterator it=f_playerInfoList.listIterator();it.hasNext();)
      {
        l_playerInList=(PlayerInfo)it.next();
        if (!l_playerInList.isVisible()) // we dont look for one we already added this round in this cycle
        {
          if (l_playerInList.getAbsPositionOfNextRound()!=null) //he is moving
          {
            Coordinate l_movingVector= l_playerInList.getAbsPositionOfNextRound().sub(l_playerInList.getAbsPosition());
            if ( (l_playerInList.getAbsPositionOfNextRound().add(l_movingVector.add(l_movingVector))).nearlyTheSameAs(l_coord))
              return l_playerInList;
          }
          else if (l_playerInList.getAbsPosition().nearlyTheSameAs(l_coord))
            return l_playerInList;
        }
      }
    }
    return null;
  }//playerIsInList()

 /**
   * Sets a whole ObjectInfo list to visible true or false
   * @param f_objectInfoList list to modify
   * @param f_isVisible set visibility of the Objects in list to true or false
   * @return the given List with all objectInfos in it with isVisible()==f_isVisible
   */
  private ArrayList setVisible(ArrayList f_objectInfoList, boolean f_isVisible)
  {
    ObjectInfo l_object;
    int l_index;
    for (ListIterator it=f_objectInfoList.listIterator();it.hasNext();)
    {
      l_index=it.nextIndex();it.next();
      l_object=(ObjectInfo)f_objectInfoList.get(l_index);
      l_object.setVisible(f_isVisible);
      f_objectInfoList.set(l_index,l_object);
    }
    return f_objectInfoList;
  }
}
class blub implements LineListener
{
  public void update (LineEvent f_event)
  {
  }
}

class ClipPlayer implements LineListener
{
  private Clip m_clip;


/*
  *The clip will be played nLoopCount + 1 times.
 */
  public ClipPlayer(File clipFile, int nLoopCount)
  {
    AudioInputStream audioInputStream = null;
    try
    {
      audioInputStream = AudioSystem.getAudioInputStream(clipFile);
    }
    catch (Exception e)
    {
      //e.printStackTrace();
    }
    if (audioInputStream != null)
    {
      AudioFormat format = audioInputStream.getFormat();
      DataLine.Info info = new DataLine.Info(Clip.class, format);
      try
      {
        m_clip = (Clip) AudioSystem.getLine(info);
        m_clip.addLineListener(this);
        m_clip.open(audioInputStream);
      }
      catch (LineUnavailableException e)
      {
        //e.printStackTrace();
      }
      catch (IOException e)
      {
        //e.printStackTrace();
      }
      m_clip.loop(nLoopCount);
    }
  }
  public void update(LineEvent event)
  {
    if (event.getType().equals(LineEvent.Type.STOP))
    {
      m_clip.close();
    }
    else if (event.getType().equals(LineEvent.Type.CLOSE))
    {
    }

  }
}